@ECHO OFF
::
::   _gbssetup.bat
::	Alias for gbssetup.bat.
::	Used for initial setup of GBS
::	Makes sure command is on top of list
::	You MUST cd to This directory (GBS_SCRIPTS_PATH) to call this script
::

echo _GBSSETUP_:
echo _GBSSETUP_: Setup a GBS environment (Initial)
echo _GBSSETUP_:
echo.

::
::  Sanity Checks
::
if not defined APPDATA echo _GBSSETUP_: Fatal Error: System EnvVar APPDATA is not defined
if not defined APPDATA goto END

if not defined TEMP echo _GBSSETUP_: Fatal Error: System EnvVar TEMP is not defined
if not defined TEMP goto END

set GBS_CONDITION=0
if not exist .\_gbssetup.bat set GBS_CONDITION=1
if not exist .\glo           set GBS_CONDITION=1
if not exist .\plugins       set GBS_CONDITION=1
if not exist .\build.dat     set GBS_CONDITION=1
if %GBS_CONDITION% equ 1 echo _GBSSETUP_: Fatal Error: You are not in a valid GBS_SCRIPTS_PATH directory
if %GBS_CONDITION% equ 1 goto END

::
::  Setup
::
if NOT [%GBS_PID%]==[] goto gbs_notok
    ::
    ::  Set GBS_BOOT_PATH, GBS_BASE_PATH and GBS_PERL_CMD
    ::
    call "%APPDATA%\.gbs\boot.bat"

    ::
    ::  Execute profile if it exists
    ::	Defines GBS_SCRIPTS_ROOT, GBS_SCRIPTS_REL, GBS_SCRIPTS_PATH and more
    ::
    if exist "%GBS_BASE_PATH%\profile.bat" call "%GBS_BASE_PATH%\profile.bat"

    ::
    ::  Determine GBS_SCRIPTS_PATH possibly overwriting the current one
    ::  Convert to short notation first
    ::
    set THIS_DIR=%~dp0
    for %%A in ("%THIS_DIR%") do set GBS_SCRIPTS_PATH=%%~sA

    if NOT exist %GBS_SCRIPTS_PATH%\build.dat echo _GBSSETUP_: Not in proper GBS setup directory (%GBS_SCRIPTS_PATH%)
    if NOT exist %GBS_SCRIPTS_PATH%\build.dat goto END
    echo _GBSSETUP_: GBS_SCRIPTS_PATH set to %GBS_SCRIPTS_PATH%

    ::
    ::  Setup GBS_PID
    ::
    %GBS_SCRIPTS_PATH%\win32utils\ppid.exe
    set GBS_PID=%ERRORLEVEL%

    ::
    ::  Run gbssetup
    ::
    set GBS_INITIAL_SETUP=1
    call %GBS_SCRIPTS_PATH%\gbssetup.bat %*
    set GBS_INITIAL_SETUP=
    goto gbs_end
:gbs_notok
    echo _GBSSETUP_: ** GBS already running. Use 'gbssetup' command **
    set GBS_RC=1
:gbs_end

::
::  Completion
::
:end
echo.
set /p GBS_WAIT=_GBSSETUP_: Hit ^<Enter^> to finish (and maybe close window):

::
::  Cleanup
::
set GBS_PID=
set GBS_WAIT=
set GBS_SCRIPTS_PATH=
set GBS_CONDITION=

:: **EOF**
