#===============================================================
#
#   bootstrap.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package bootstrap;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
BOOTSTRAP_file
BOOTSTRAP_install
BOOTSTRAP_change
);
}




use glo::env;
use glo::slurp;
use glo::spit;
use glo::shell;




sub BOOTSTRAP_file();
sub BOOTSTRAP_install();
sub BOOTSTRAP_change($);




my $GBS_BOOT_PATH = ENV_getenv_perl_path( 'GBS_BOOT_PATH');	# %APPDATA%/.gbs or ~/.gbs
my $GBS_BASE_PATH = ENV_getenv_perl_path( 'GBS_BASE_PATH');	# %APPDATA%/.gbs/base or ~/.gbs/base or other
my $SHELL_FILETYPE = ENV_shell_filetype();
my $IS_WIN32 = ENV_is_win32();




my $BOOTSTRAP_FILE = "bootstrap$SHELL_FILETYPE";




my $IN_INSTALL = 0;




sub BOOTSTRAP_file()
{
return $BOOTSTRAP_FILE;
}




sub BOOTSTRAP_install()
{
$IN_INSTALL = 1;
my @boot_files = SLURP_dir_all( $GBS_BOOT_PATH, 0);
if (@boot_files == 1 && $boot_files[0] =~ /^bootstrap\.(bat|sh)$/)
{
my $mydocuments_path = ENV_get_user_path( 0);
my $short_mydocuments_path = ENV_perl_paths( ENV_short_path_name( $mydocuments_path));
$GBS_BASE_PATH = "$short_mydocuments_path/.gbs/base";
ENV_say( 1, "Created $GBS_BASE_PATH")
if (ENV_mkpath( $GBS_BASE_PATH, undef, 'EE'));
BOOTSTRAP_change( $GBS_BASE_PATH);
} else
{
ENV_whisper( 1, "GBS_BASE_PATH = $GBS_BASE_PATH");
}

return $GBS_BASE_PATH;
}




sub BOOTSTRAP_change($)
{
my ($new_base_path,
) = @_;




ENV_say( 1, "New GBS_BASE_PATH defined: $new_base_path")
if (!$IN_INSTALL);
ENV_setenv( GBS_BASE_PATH => $new_base_path);




SPIT_file( "$GBS_BOOT_PATH/$BOOTSTRAP_FILE", SHELL_setenv( GBS_BASE_PATH => $new_base_path));
ENV_say( 2, "Updated $BOOTSTRAP_FILE")
if (!$IN_INSTALL);

$GBS_BASE_PATH = $new_base_path;

return $GBS_BASE_PATH;
}

1;


