#===============================================================
#
#   files.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package files;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
FILES_reset
FILES_attrib
FILES_get_ignore_spec
FILES_mandatory_dirs
FILES_all
);
}




use glo::env;
use glo::slurp;
use gbsglo;
use gbsenv;




sub FILES_reset($);
sub FILES_attrib($);
sub FILES_get_ignore_spec($);
sub FILES_mandatory_dirs($$);
sub FILES_all();

sub init();
sub attrib($);
sub mandatory_dirs($$);
sub all_files_node($$);
sub dummy_check_mandatory($$);
sub get_ignore($);
sub get_subsys_key($);

sub get_node_members($);
sub node_is_dir($);
sub node_data($);
sub node_recurse($$$);
sub next_dir_node($$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_ROOT_PATH_L = length( $GBS_ROOT_PATH);

my %GBS_ALL_TARGETS;	    # Do not initialize!
my %GBS_ALL_AUDITS;	    # Do not initialize!
my %GBS_ALL_TOOLS;	    # Do not initialize!

my %PLUGIN_ITEMS;;	    # Do not initialize!



my $MUST_INIT = 1;

my ($THIS_CMD_EXT, $OTHR_CMD_EXT) = ENV_shell_filetype();

my $DIR_ENTRIES_LEVEL;
my @DIR_ENTRIES;








my %TOPLEVELS = (

GBS	    => 'SUBSYS_GBS',
MSVS    => 'SUBSYS_MSVS',
make    => 'SUBSYS_MAKE',
Other   => 'SUBSYS_OTHER',
UNK	    => 'SUBSYS_UNK',
);

my @SUBSYS_ALL = (







'ssprop.gbs'	=> [ 'M', 1 ],
'.*.lck'	=> [ 'O', 0 ],
audit =>
{
'.'	  => [ 'M', 1, 0, 1, 0, 0 ],
'<A>' => # Audits
{
'.'   => [ 'O', 1, 0, 0, 1, 1 ],
'<B>' => # Targets
{
'.' => [ 'O', 1, 0, 0, 1, 1 ],
},
},
},
build =>
{
'.'	          => [ 'M', 1, 0, 0, 0, 0 ],
'incs_*.gbs'  => [ 'O', 1 ],
'flags_*.gbs' => [ 'O', 1 ],
'export.gbs'  => [ 'O', 1 ],
'*.mk'	      => [ 'O', 0 ],
'*.usr'	      => [ 'O', 0 ],
'<B>' => #Targets
{
'.' => [ 'O', 1, 0, 0, 1, 1 ],
}
},
export =>
{
'.'	=> [ 'O', 1, 1, 0, 1, 0 ],
},
import =>
{
'.'	=> [ 'O', 1, 1, 1, 1, 1 ],
},
tools =>
{
'.'	=> [ 'M', 1, 1, 1, 0, 0 ],
'<B>' =>  # Tools
{
'.' => [ 'O', 1, 1, -1, 1, -1 ],
},
},

);
my %DIRS = (







SYSTEM =>
{
'.'		 => [ 'M', 1, 1, -1, 1, -1 ],
'system.gbs'		    => [ 'M', 1 ],
"switch.gbs$THIS_CMD_EXT"   => [ 'M', 1 ],
"switch.gbs$OTHR_CMD_EXT"   => [ 'O', 1 ],
'switch.usr.*'		    => [ 'O', 0 ],
dev =>
{
'.'	     => [ 'M', 1, 1, 1, 0, 0 ],
'.gbsrc' => [ 'O', 0 ],
'*'      =>			    # SubSystems
{
'.'	=> [ 'O', 1, 1, 1, 1, 1 ],
},
},
ext =>
{
'.'	=> [ 'M', 1, 1, 1, 1, 1 ],
},
MOCKS =>
{
'.'	=> [ 'O', 1, 1, 1, 1, 1 ],
},
res =>
{
'.'	     => [ 'M', 1, 1, 1, 0, 0 ],
'.*.lck' => [ 'O', 0 ],
'*'      =>			    # SubSystems
{
'.' => [ 'O', 1, 1, 0, 1, 0 ],
},
},
silo =>
{
'.'	    => [ 'M', 1, 1, 0, 1, 0 ],
'.gbs'  =>
{
'.'	=> [ 'M', 1, 1, 0, 1, 0 ],
cache   =>
{
'.'		=> [ 'M', 1, 1, 0, 1, 0 ],
html  =>
{
'.'	=> [ 'M', 1, 1, 0, 1, 0 ],
},
'*'      =>			    # Audit Plugins
{
'.' => [ 'O', 1, 1, 0, 1, 0 ],
'*'      =>			    # Audit Releases
{
'.' => [ 'O', 0, 1, 0, 1, 0 ],
},
},
},
gbssum	=>
{
'.'	=> [ 'M', 1, 1, 0, 1, 0 ],
},
},
},
sys =>
{
'.'		  => [ 'M', 1, 1, 1, 1, 1 ],
'owners.gbs'  => [ 'O', 1 ],
'steps.gbs'   => [ 'O', 1 ],
'sys.gbs'     => [ 'M', 1 ],
'.*.lck'	  => [ 'O', 0 ],
'*.usr'	  => [ 'O', 0 ],
'*' =>
{
'.' => [ 'O', 1, 1, 1, 1, 1 ],
}
},
sysaudit =>
{
'.'		  => [ 'M', 1, 1, 1, 1, 1 ],
'incs_*.gbs'  => [ 'O', 1 ],		    # in audits?
'flags_*.gbs' => [ 'O', 1 ],		    # in audits?
'*.usr'	  => [ 'O', 0 ],		    # in audits?
'<A>' =>  # Audits
{
'.'		 => [ 'M', 1, 1, 1, 1, 1 ],
'incs_*.gbs'     => [ 'O', 1 ],		    # in audits?
'flags_*.gbs'	 => [ 'O', 1 ],		    # in audits?
'bld_incs_*.gbs' => [ 'O', 1 ],		    # in audits?
'bld_flags_*.gbs'=> [ 'O', 1 ],		    # in audits?
'audit.gbs'      => [ 'O', 1 ],
'*.usr'	         => [ 'O', 0 ],
},
'*' =>
{
'.' => [ 'O', 1, 1, 1, 1, 1 ],
}
},
sysbuild =>
{
'.'		    => [ 'M', 1, 1, 1, 1, 1 ],
'incs_*.gbs'    => [ 'O', 1 ],
'flags_*.gbs'   => [ 'O', 1 ],
'*.usr'	    => [ 'O', 0 ],
templates	    =>
{
'.'	=> [ 'M', 1, 0, 0, 1, 1 ],
},
makemake_stubs  =>
{
'.'	=> [ 'M', 1, 1, 1, 1, 1 ],
'ALL'	=>
{
'.'	=> [ 'M', 1, 1, 1, 1, 1 ],
},
'<B>' =>  # Targets
{
'.'	=> [ 'M', 1, 1, 1, 1, 1 ],
},
},
'<B>'	    =>  # Targets
{
'.'		 => [ 'M', 1, 1, 1, 1, 1 ],
'incs_*.gbs'     => [ 'O', 1 ],
'flags_*.gbs'	 => [ 'O', 1 ],
'bld_incs_*.gbs' => [ 'O', 1 ],
'bld_flags_*.gbs'=> [ 'O', 1 ],
'target.gbs'     => [ 'O', 1 ],
'*.usr'	         => [ 'O', 0 ],
},
'*'		    =>
{
'.' => [ 'O', 1, 1, 1, 1, 1 ],
}
},
systools =>
{
'.'	  => [ 'M', 1, 1, 1, 1, 1 ],
'<T>' => # Tools
{
'.'		=> [ 'M', 1, 1, 1, 1, 1 ],
'tool.gbs'      => [ 'M', 1 ],
'*.usr'		=> [ 'O', 0 ],
},
'*' =>
{
'.' => [ 'O', 1, 1, 1, 1, 1 ],
}
},
tmp =>
{
'.'	  => [ 'M', 1, 1, 0, 1, 0 ],
'<A>' => # Audits
{
'.' => [ 'M', 1, 1, 0, 1, 0 ],
},
'<B>' => # Targets
{
'.' => [ 'M', 1, 1, 0, 1, 0 ],
},
'<T>' => # Tools
{
'.' => [ 'M', 1, 1, 0, 1, 0 ],
},
},
},








SUBSYS_GBS =>
{
'.'		=> [ 'M', 1, 1, 1, 1, 1 ],
@SUBSYS_ALL,
'.gbsrc' => [ 'O', 0 ],
comp =>
{
'.'	     => [ 'M', 1, 1, 1, 0, 0 ],
'.gbsrc' => [ 'O', 0 ],
},
},
















SUBSYS_MSVS =>
{
'.'        => [ 'M', 1, 1, -1, 1, -1 ],
@SUBSYS_ALL,
app =>
{
'.'		=> [ 'M', 1, 1, 1, 1, -1 ],
'*.sln'	=> [ 'O', 1 ],
'*.vcxproj'	=> [ 'O', 1 ],
'*.sdf'	=> [ 'O', 0 ],
'*.suo'	=> [ 'O', 0 ],
'*.user'	=> [ 'O', 0 ],
Debug =>
{
'.'	=> [ 'O', 1, 1, -1, 1, 0 ],
},
Release =>
{
'.'	=> [ 'O', 1, 1, -1, 1, 0 ],
},
Testresults =>
{
'.'	=> [ 'O', 1, 1, -1, 1, 0 ],
},
'*'	=>
{
'.'	    => [ 'O', 1, 1, 1, 1, -1 ],
'ReadMe.txt'	=> [ 'O', 1 ],
'*.sln'		=> [ 'O', 1 ],
'*.vcxproj'	=> [ 'O', 1 ],
'*.sdf'		=> [ 'O', 0 ],
'*.suo'		=> [ 'O', 0 ],
'*.user'	=> [ 'O', 0 ],
Debug =>
{
'.'	=> [ 'O', 1, 1, -1, 1, 0 ],
},
Release =>
{
'.'	=> [ 'O', 1, 1, -1, 1, 0 ],
},
},
},
opts =>
{
'.'	=> [ 'M', 1, 0, 0, 1, 0 ],
'aud'	=>
{
'.' => [ 'M', 1, 1, 0, 1, 0 ],
},
'bld'	=>
{
'.' => [ 'M', 1, 1, 0, 1, 0 ],
}
},
},








SUBSYS_MAKE =>
{
'.'        => [ 'M', 1, 1, -1, 1, -1 ],
@SUBSYS_ALL,
app =>
{
'.'	    => [ 'M', 1, 1, -1, 1, -1 ],
'*.mk'  => [ 'O', 1 ],
},
opts =>
{
'.'	=> [ 'M', 1, 0, 0, 1, 0 ],
'aud'	=>
{
'.' => [ 'M', 1, 1, 0, 1, 0 ],
},
'bld'	=>
{
'.' => [ 'M', 1, 1, 0, 1, 0 ],
}
},
},








SUBSYS_OTHER =>
{
'.'        => [ 'M', 1, 1, -1, 1, -1 ],
@SUBSYS_ALL,
app =>
{
'.'	=> [ 'M', 1, 1, -1, 1, -1 ],
},
opts =>
{
'.'	=> [ 'M', 1, 0, 0, 1, 0 ],
'aud'	=>
{
'.' => [ 'M', 1, 1, 0, 1, 0 ],
},
'bld'	=>
{
'.' => [ 'M', 1, 1, 0, 1, 0 ],
}
},
},

SUBSYS_UNK =>
{
'.'        => [ 'M', 1, 1, -1, 1, -1 ],
@SUBSYS_ALL,
},








COMPONENT =>
{
'.'          => [ 'M',  1, 0, 0, 0, 0 ],
'export.gbs' => [ 'O',  1, ],
'export.usr' => [ 'O',  0, ],
'scope.gbs'  => [ 'O',  1, ],
'scope.usr'  => [ 'O',  0, ],
'*.dsp'      => [ 'O', -1, ],	# Microsoft Visual Studio 6
'*.dsw'      => [ 'O', -1, ],	# Microsoft Visual Studio 6
'*.sln'      => [ 'O', -1, ],	# Microsoft Visual Studio 8
'*.mcp'      => [ 'O', -1, ],	# Code Warrior
aud =>
{
'.'	=> [ 'M', 1, 0, 0, 0, 0 ],
'<A>' => #audits
{
'.'   => [ 'O', 1, 1, -1, 1, -1 ],
'<B>' => #Targets
{
'.' => [ 'O', 1, 1, 0, 1, 0 ],
},
'ALL' => #All-Targets
{
'.' => [ 'O', 1, 1, 0, 1, 0 ],
}
}
},
bld =>
{
'.'	    => [ 'M', 1, 0, 0, 0, 0 ],
'*.mk'  => [ 'O', 0 ],
'*.ref' => [ 'O', 0 ],
'<B>' => #Targets
{
'.' => [ 'O', 1, 0, 0, 1, 0 ],
}
},
dat =>
{
'.'	=> [ 'M', 1, 1, 1, 1, 1 ],
},
inc =>
{
'.'	=> [ 'M', 1, 0, 0, 1, 1 ],
},
loc =>
{
'.'	=> [ 'M', 1, 0, 0, 1, 1 ],
},
opt =>
{
'.'	=> [ 'M', 1, 0, 0, 0, 0 ],
'incs_*.gbs'  => [ 'O', 1 ],
'flags_*.gbs' => [ 'O', 1 ],
'*.usr'	  => [ 'O', 0 ],
'<B>' =>  # Targets
{
'.'	=> [ 'O', 1, 0, 0, 0, 0 ],
'incs_*.gbs'  => [ 'O', 1 ],
'flags_*.gbs' => [ 'O', 1 ],
'*.usr'	      => [ 'O', 0 ],
}
},
sav =>
{
'.'	=> [ 'M', 1, 1, -1, 1, -1 ],
},
src =>
{
'.'	=> [ 'M', 1, 0, 0, 1, 1 ],
},
},
);






sub FILES_reset($)
{
my ($gbs_root_path) = @_;

if (defined $gbs_root_path)
{
$GBS_ROOT_PATH = $gbs_root_path;
$GBS_ROOT_PATH_L = length( $GBS_ROOT_PATH);
}

$MUST_INIT = 1;
}




sub FILES_attrib($)
{
my ($spec,		# file or dir
) = @_;


init()
if ($MUST_INIT);

return attrib( $spec);
}




sub FILES_get_ignore_spec($)
{
my ($spec,		# file or dir
) = @_;
my $ignores_ref;		   #	[ @ignores ]	(may be wildcards)

init()
if ($MUST_INIT);

$ignores_ref = attrib( $spec);

return $ignores_ref;
}




sub attrib($)
{
my ($spec) = @_;	    # $file_or_dir
my ($ignores_ref, $scm, $avail, $type);
$ignores_ref = [];




my $top_level;
my $remain;
{
ENV_sig( F => "'$spec' not in '$GBS_ROOT_PATH")
if (substr( $spec, 0, $GBS_ROOT_PATH_L) ne $GBS_ROOT_PATH);
my $sub_path = substr( $spec, $GBS_ROOT_PATH_L);
my ($subsys, $component);
if ($sub_path =~ m!^/dev/([^/]+)/comp/([^/]+)/?(.*)!)
{
$subsys = $1;
$component = $2;
$remain = $3;

$top_level = 'COMPONENT';
} elsif ($sub_path =~ m!^/dev/([^/]+)/?(.*)!)
{
$subsys = $1;
$remain = $2;

$top_level = get_subsys_key( $subsys);
} else
{
$remain = ($sub_path eq '') ? '' : substr( $sub_path, 1);

$top_level = 'SYSTEM';
}

}




my $node_ref = [ \%DIRS, $top_level, $DIRS{$top_level} ];
foreach my $name (split( '/', $remain))
{
$node_ref = next_dir_node( $node_ref, $name);
last if (!defined $node_ref || $node_ref->[1] eq '.');
}


if (defined $node_ref)
{
my $name = $node_ref->[1];
if ($name eq '.')
{
my (undef, undef, undef, $extra_dirs, $scm_extra_dirs, $extra_files, $scm_extra_files) =
node_data( $node_ref);
$type = 'U';
$avail = 'O';
if ($scm_extra_dirs == 1 && $scm_extra_files == 1)
{
$scm = 1;
} elsif ($scm_extra_dirs == 0 && $scm_extra_files == 0)
{
$scm = 0;
} else
{
$scm = -1;
}
} else
{
($type, $avail, $scm) = node_data( $node_ref);
$ignores_ref = get_ignore( $node_ref);
}
}


return (wantarray) ? ($ignores_ref, $scm, $avail, $type) : $ignores_ref;
}




sub FILES_mandatory_dirs($$)
{
my ($level_path,
$top_level,	  # SYSTEM, SUBSYS or COMPONENT
) = @_;
my @dirs;

init()
if ($MUST_INIT);

if ($top_level eq 'SUBSYS')
{
my $subsys = ENV_parent_dir( $level_path, -1);
$top_level = get_subsys_key( $subsys);	    # SUBSYS_GBS, SUBSYS_MSVS, etc
}

$DIR_ENTRIES_LEVEL = 0;
@DIR_ENTRIES = ($level_path);
my $top_ref = [ \%DIRS, $top_level , $DIRS{$top_level} ];
node_recurse( $top_ref, \&mandatory_dirs, \@dirs);

return @dirs;
}




sub mandatory_dirs($$)
{
my ($dirs_ref,  # $data
$node_ref,  # [ $hash_ref, $name, $members_ref ]
) = @_;

if (node_is_dir( $node_ref))
{
my $dir_name = $node_ref->[1];
my (undef, $avail) = node_data( $node_ref);


if ($avail eq 'M')
{
if ($dir_name =~ /<.>/)	# <B>, <A>, <T>
{
my @plugin_items = @{$PLUGIN_ITEMS{$dir_name}};
push @{$dirs_ref}, map { join( '/', @DIR_ENTRIES, $_) } @plugin_items;
} else
{
push @{$dirs_ref}, join( '/', @DIR_ENTRIES, $dir_name);
}
}
}
}










sub FILES_all()
{
my @line_refs;

init()
if ($MUST_INIT);

my $node_ref = [ \%DIRS, undef, undef ];




$node_ref->[1] = 'SYSTEM';
$node_ref->[2] = $DIRS{'SYSTEM'};
@line_refs = all_files_node( $node_ref, '');





foreach my $subsys (GBSGLO_subsystems())
{
my $top_level = get_subsys_key( $subsys);	    # SUBSYS_GBS, SUBSYS_MSVS, etc
$node_ref->[1] = $top_level;
$node_ref->[2] = $DIRS{$top_level};
push @line_refs, all_files_node( $node_ref, "dev/$subsys");
if ($top_level eq 'SUBSYS_GBS')
{



foreach my $component (GBSGLO_components( $subsys))
{
$node_ref->[1] = 'COMPONENT';
$node_ref->[2] = $DIRS{'COMPONENT'};
push @line_refs, all_files_node( $node_ref, "dev/$subsys/comp/$component");
}
}
}

return @line_refs;
}












sub all_files_node($$)
{
my ($node_ref,	# [ $hash_ref, $name, $members_ref ]
$sub_path,
) = @_;
my @line_refs;	#   ( [ $type, $avail, $scm, $spec ], ... )

my $this_root_path = ($sub_path eq '') ? $GBS_ROOT_PATH : "$GBS_ROOT_PATH/$sub_path";





my %this_dir_entries;


my @this_dir_keys;
if (-d $this_root_path)
{
map { $this_dir_entries{ $_} = 'D' } GBSGLO_select_specs( SLURP_dir_dirs( $this_root_path, 0));
map { $this_dir_entries{ $_} = 'F' } GBSGLO_select_specs( SLURP_dir_files( $this_root_path, 0));
$this_dir_entries{'.gbsrc'} = 1
if (-e "$this_root_path/.gbsrc");	# Add invisible (skipped) file if present
$this_dir_entries{'.gbs'} = 1
if (-e "$this_root_path/.gbs");	# Add invisible (skipped) file if present (silo/.gbs)
@this_dir_keys = keys %this_dir_entries;

}






my @next_refs;


foreach my $member_ref (get_node_members( $node_ref))

{
my $name = $member_ref->[1];

my $member_is_dir = node_is_dir( $member_ref);
my $member_is_wild = ENV_is_wildcard( $name);

my ($type, $avail, $scm) = node_data( $member_ref);

my @found;
my @missing;
if ($name =~ /<.>/)	    	# <B>, <A>, <T>
{
foreach my $target_or_audit (@{$PLUGIN_ITEMS{$name}})
{
if (exists $this_dir_entries{$target_or_audit})
{
push @found, $target_or_audit;
} else
{
push @missing, $target_or_audit;
}
}
} elsif ($member_is_wild)
{
@found = ENV_wildcard( $name, [ keys %this_dir_entries ]);
@missing = ($name) if (!@found);
} else
{
if (exists $this_dir_entries{$name})
{
@found = ($name);
} else
{
@missing = ($name);
}
}

if (@found)
{

foreach my $this_entry (@found)
{
my $member_sub_path = ($sub_path eq '') ? $this_entry : "$sub_path/$this_entry";
my $entry_type = $type;
if ($name eq '*')
{
$entry_type = (-d "$GBS_ROOT_PATH/$member_sub_path") ? 'D' : 'F';
}

if ($entry_type eq 'D')
{
if (-d "$GBS_ROOT_PATH/$member_sub_path")
{
push @next_refs, [ $member_sub_path, $member_ref ]
if ($member_is_dir);
} else
{
$avail = 'i';
}
delete $this_dir_entries{$this_entry};
} else # $entry_type eq 'F'
{
if (-d "$GBS_ROOT_PATH/$member_sub_path")
{
$avail = 'i';
} else
{
delete $this_dir_entries{$this_entry};
}
}
push @line_refs, [ $entry_type, $avail, $scm, $member_sub_path ];
}
}
if (@missing)	# actual file/dir not found
{
foreach my $name (@missing)
{
my $member_sub_path = ($sub_path eq '') ? $name : "$sub_path/$name";
if ($avail eq 'M')
{
push @line_refs, [ $type, 'm', $scm, $member_sub_path ];
}
push @next_refs, [ $member_sub_path, $member_ref ]
if ($member_is_dir);
}
}
}





if (%this_dir_entries)
{




my (undef, undef, undef, $extra_dirs, $scm_extra_dirs, $extra_files, $scm_extra_files) =
node_data( $node_ref);
foreach my $entry (keys %this_dir_entries)
{
my ($type, $avail, $scm, $member_sub_path) = ( undef, 'I', 0, ($sub_path eq '') ? $entry : "$sub_path/$entry" );
$type = $this_dir_entries{ $entry};
if ($type eq 'D')
{
if ($extra_dirs)
{
$avail = 'O';
$scm = $scm_extra_dirs;
}
} else
{
if ($extra_files)
{
$avail = 'O';
$scm = $scm_extra_files;
}
}
push @line_refs, [ $type, $avail, $scm, $member_sub_path ];

}
}




foreach my $ref (@next_refs)
{

push @line_refs, all_files_node( $ref->[1], $ref->[0]);		# $member_ref $member_sub_path
}

return @line_refs;
}





sub dummy_check_mandatory($$)
{
my ($base_path,
$parent,
) = @_;
my $ok = 1;


my $top_ref = [ \%DIRS, $parent, $DIRS{$parent} ];
foreach my $member_ref (get_node_members( $top_ref ))
{
my (undef, $avail) = node_data( $member_ref);
if ($avail eq 'M')
{

if (!-e "$base_path/$member_ref->[1]")
{
$ok = 0;
last;
}
}
}

return $ok;
}




sub get_ignore($)
{
my ($node_ref) = @_;    # [ $hash_ref, $name, $members_ref ]
my $ignores_ref = [];

if (node_is_dir( $node_ref))
{
my @ignores;
foreach my $member_ref (get_node_members( $node_ref))
{
my (undef, $avail, $scm) = node_data( $member_ref);

push @ignores, $member_ref->[1]
if ($scm == 0);
}
if (!@ignores)
{
my (undef, undef, undef, $extra_dirs, $scm_extra_dirs, $extra_files, $scm_extra_files) =
node_data( $node_ref);
@ignores = ('*')
if ($scm_extra_dirs == 0 && $scm_extra_files == 0);
}
$ignores_ref = [ @ignores ];
}

return $ignores_ref;
}




sub get_subsys_key($)
{
my ($subsys) = @_;
my $key;

my $sstype = GBSGLO_subsystem_type( $subsys);
$key = $TOPLEVELS{ $sstype};
ENV_sig( F => "Invalid sstype '$sstype' for SubSys '$subsys'")
if (!defined $key);
ENV_sig( E => "No sstype for SubSys '$subsys'")
if ($key eq '');

return $key;
}





sub get_node_members($)
{
my ($node_ref) = @_;    # [ $hash_ref, $name, $members_ref ]
my (@member_refs);


my $members_ref = $node_ref->[2];
my @member_names = sort( keys( %{$members_ref}));
my (@plain_member_names, @wild_member_names, @wildest_member_name);
foreach my $member_name (@member_names)
{
if ($member_name eq '.')
{

} elsif ($member_name eq '*')
{
@wildest_member_name = ($member_name);
} elsif (ENV_is_wildcard( $member_name))
{
push @wild_member_names, $member_name;
} else
{
push @plain_member_names, $member_name;
}
}

foreach my $member_name (@plain_member_names, @wild_member_names, @wildest_member_name)
{
push @member_refs, [ $members_ref, $member_name, $members_ref->{$member_name} ];
}

return @member_refs;
}






sub node_is_dir($)
{
my ($node_ref) = @_;    # [ $hash_ref, $name, $members_ref ]


return (ref $node_ref->[2] eq 'HASH') ? 1 : 0;  # $members_ref
}




sub node_data($)
{
my ($node_ref) = @_;    # [ $hash_ref, $name, $members_ref ]


my ($type);

my $attrs_ref;
if (node_is_dir( $node_ref))
{
$type = 'D';
$attrs_ref = $node_ref->[2]->{'.'};	# $members_ref
} else
{
$type = 'F';
$attrs_ref = $node_ref->[2];	    	# $members_ref
}

return ($type, @{$attrs_ref});
}





sub node_recurse($$$)
{
my ($node_ref,  # [ $hash_ref, $name, $members_ref ]
$func_ref,  # $func_ref->( $data, $member_ref)
$data,
) = @_;

$DIR_ENTRIES_LEVEL++;
foreach my $member_ref (get_node_members( $node_ref))
{
my $name = $member_ref->[1];

$func_ref->( $data, $member_ref);
if (node_is_dir( $member_ref))
{
push @DIR_ENTRIES, $name;
node_recurse( $member_ref, $func_ref, $data);
pop @DIR_ENTRIES;
}
}
$DIR_ENTRIES_LEVEL--;
}






sub next_dir_node($$)
{
my ($node_ref,      # [ $hash_ref, $name, $members_ref ]
$next_name,
) = @_;
my $next_node_ref;	# undef == not_found


my $members_ref = $node_ref->[2];
if (exists $members_ref->{$next_name})
{
$next_node_ref = [ $members_ref, $next_name ];
} else
{




if (exists $members_ref->{'*'})
{

$next_node_ref = [ $members_ref, '*' ];
} elsif (exists $members_ref->{'<B>'} && exists( $GBS_ALL_TARGETS{$next_name}))
{

$next_node_ref = [ $members_ref, '<B>' ];
} elsif (exists( $members_ref->{'<A>'}) && exists( $GBS_ALL_AUDITS{$next_name}))
{

$next_node_ref = [ $members_ref, '<A>' ];
} elsif (exists $members_ref->{'<T>'} && exists( $GBS_ALL_TOOLS{$next_name}))
{

$next_node_ref = [ $members_ref, '<T>' ];
} else
{

foreach my $member_ref (get_node_members( $node_ref))
{
my $member_name = $member_ref->[1];
next if (! ENV_is_wildcard( $member_name));
my $member_name_re = ENV_wild_2_re( $member_name);
if ($next_name =~ /$member_name_re/)
{
$next_node_ref = [ $members_ref, $member_name ];

last;
}
}
if (!defined $next_node_ref)
{
my (undef, undef, undef, $extra_dirs, $scm_extra_dirs, $extra_files, $scm_extra_files) =
node_data( $node_ref);
if ($extra_dirs == 1 && $extra_files == 1)
{
$next_node_ref = [ $members_ref, '.' ];
}
}
}
}



$next_node_ref->[2] = $next_node_ref->[0]->{$next_node_ref->[1]}	# $member_ref, $hash_ref, $name
if (defined $next_node_ref);
return $next_node_ref;
}




sub init()
{
my @gbs_all_targets = ENV_getenv( 'GBS_ALL_TARGETS');
my @gbs_all_audits = ENV_getenv( 'GBS_ALL_AUDITS');
my @gbs_all_tools = ENV_getenv( 'GBS_ALL_TOOLS');

%PLUGIN_ITEMS = (
'<A>'    => [ @gbs_all_audits ],    # Audit
'<B>'    => [ @gbs_all_targets ],   # Build
'<T>'    => [ @gbs_all_tools ],	    # Tool
);

%GBS_ALL_TARGETS = map { $_ => 1 } @gbs_all_targets;
%GBS_ALL_AUDITS = map { $_ => 1 } @gbs_all_audits;
%GBS_ALL_TOOLS = map { $_ => 1 } @gbs_all_tools;

$MUST_INIT = 0;
}

1;
