#===============================================================
#
#   gbsexit.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSEXIT @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::genopt;
use glo::slurp;
use glo::shell;
use gbsenv;
use swr;
use swglo;








my $IS_WIN32 = (ENV_is_win32());
my $IS_LUNIX = ! $IS_WIN32;

my $GBS_BOOT_PATH = ENV_getenv_perl_path( 'GBS_BOOT_PATH');	# %APPDATA%/.gbs or ~/.gbs
my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SYSTEM_NAME = ENV_getenv( 'GBS_SYSTEM_NAME');

my $SHELL_FILETYPE = ENV_shell_filetype();	# .bat or .sh






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
);
GENOPT_set_optdefs( 'gbsexit', \@genopts,
[ 'Exit and cleanup the GBS environment'],
undef);
GENOPT_parse();




{
my @lines;

ENV_say( 1, "Preparing exit...");
push @lines, SHELL_echo( "Starting Cleanup...");





if ($GBS_ROOT_PATH ne '')
{



push @lines, SWR_exit( $GBS_ROOT_PATH)
if ($GBS_ROOT_PATH ne '');




push @lines, SWGLO_set_title( '-');
}





foreach my $envname (grep( /^GBS_/, keys %ENV))
{
if (GBSENV_is_predefined_env( $envname) || $envname eq 'GBS_SCRIPT_LEVEL')
{
ENV_whisper( 1, "SKIPPED: $envname");
} else
{
push @lines, SHELL_unsetenv( $envname);
}
}





if ($IS_WIN32)
{

foreach my $line (grep( /GBS/i, SLURP_file( "$GBS_SCRIPTS_PATH/gbsdef.doskey")))  # lines containing GBS
{
my ($command_name) = $line =~ /^([^=]*)/;
push @lines, SHELL_unsetfunc( $command_name);
}
} else
{


foreach my $line (grep( /GBS/i, SLURP_file( "$GBS_SCRIPTS_PATH/gbsdef.sh")))  # lines containing GBS
{
next if (substr( $line, 0, 1) eq '#');	    # skip comments
my ($command, $command_name) = $line =~ /^(function|alias)\s+([^=\s{]*)/;
if (defined $command)
{
push @lines, SHELL_unsetfunc( $command_name, ($command eq 'alias'));
}
}
}





{
my $new_cwd = SWGLO_new_cwd( '', '', '');

push @lines, SHELL_echo( "CWD set to $new_cwd");
push @lines, SHELL_cd( $new_cwd);
push @lines, SHELL_echo( "Cleanup Done");

}




push @lines, SHELL_source( "$GBS_BOOT_PATH/define$SHELL_FILETYPE", 'cur');




GBSENV_write_result_script( \@lines, 0);
}




ENV_say( 1, "After the 'Done' message it may take some time for the prompt to return...");
ENV_exit( $RC);




END
{
ENV_print_end_msg( 1);
}


