#===============================================================
#
#   gbsgen.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSGEN @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::genopt;
use gbsenv;
use gbsoptenv;
use validate;
use sys;
use plugin;
use cfo;
use gbsbgjob;
use gbsexec;




sub get_post_build_lines();




my $GBS_ROOT_PATH;
my $GBS_SUBSYS;

my $GBS_COMP_PATH;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<*>',    'comp_file_opts',   'sam', "", "src files to generate with 'generate' options" ],
[ 'i' ,     'ignore_errors',    'bso',  0, "Continue generation regardless of error(s)" ],
[ 'export', 'run_gbsexport',    'bso',  0, "Run 'gbsexport' on completion" ],
[ 'mm',     'run_makemake',	    'bso',  0, "Run 'makemake' on completion" ],
[ 'jobs',	'jobs',	       'isor1..9',  2, 'Max nr parallel jobs' ],
);
my @genenvs = qw( DEBUGGER MODE OPT MAP FLAGS_* APP_* );
GENOPT_set_optdefs( 'gbsgen', \@genopts,
'Generate one or more files',
undef);
GENOPT_set_envdefs( \&GBSOPTENV_set, \&GBSOPTENV_print, 'GBS_', \@genenvs);
GENOPT_parse();
my @COMP_FILE_OPTS = GENOPT_get( 'comp_file_opts');
my $IGNORE_ERRORS = GENOPT_get( 'ignore_errors');
my $RUN_GBSEXPORT = GENOPT_get( 'run_gbsexport');
my $RUN_MAKEMAKE = GENOPT_get( 'run_makemake');
my $JOBS = GENOPT_get( 'jobs');

my $GEN_ALL = (scalar @COMP_FILE_OPTS && $COMP_FILE_OPTS[0] eq '*:*.*');




ENV_setenv( GBS_IGNORE_ERRORS => $IGNORE_ERRORS);




($GBS_ROOT_PATH, $GBS_SUBSYS) = VALIDATE_subsys_full_gbs();
VALIDATE_target();
$GBS_COMP_PATH = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp";

if (GBSENV_mode_is_interactive())
{




SYS_set_envs();
PLUGIN_setup( 'build');
}

{



GBSBGJOB_batch_init( $JOBS);




my $nr_cfo = CFO_parse_gen( \@COMP_FILE_OPTS);
if ($nr_cfo > 0)
{
ENV_whisper( 1, "Executing 'Gen'...");




$RC = GBSEXEC_gen( $nr_cfo, $IGNORE_ERRORS);




if (($RC == 0 || $IGNORE_ERRORS) &&
($RUN_MAKEMAKE || $RUN_GBSEXPORT))
{
my @lines = get_post_build_lines();
if (@lines)
{
ENV_whisper( 1, "Run gbsgen_post");
my $rc = GBSBGJOB_batch_run( 'gbsgen_post', '-', \@lines);
$RC = $rc
if ($rc > $RC);
}
}
} else
{
ENV_sig( W => "Nothing to do");
}




GBSBGJOB_batch_finish();
}



ENV_exit( $RC);




END
{
ENV_print_end_msg( 1);
}





sub get_post_build_lines()
{
my @lines;

my @all_comps = CFO_get_components();




if ($RUN_MAKEMAKE)
{
if ($GEN_ALL)
{
push @lines, GBSBGJOB_banner( 'gbsmakemake ALL');
} else
{
push @lines, GBSBGJOB_banner( "gbsmakemake for Component(s): @all_comps");
}

my @makemake_opts;
push @makemake_opts, @all_comps
if (!$GEN_ALL);
push @makemake_opts, '--i'
if ($IGNORE_ERRORS);

my $command_ref = ENV_command_ref( 'gbsmakemake.pl', \@makemake_opts);
push @lines, GBSBGJOB_exec_short( 'gbsmakemake', 0, undef, $command_ref);
}




if ($RUN_GBSEXPORT)
{
my @texts = ("Explicit: gbsexport");
if ($GEN_ALL)
{
push @lines, GBSBGJOB_banner( 'gbsexport ALL');
} else
{
push @lines, GBSBGJOB_banner( "gbsexport for Component(s): @all_comps");
}

my @gbsexport_opts;
push @gbsexport_opts, grep( -e "$GBS_COMP_PATH/$_/export.gbs", @all_comps)
if (!$GEN_ALL);
push @gbsexport_opts, '--i'
if ($IGNORE_ERRORS);

my $command_ref = ENV_command_ref( 'gbsexport.pl', \@gbsexport_opts);
push @lines, GBSBGJOB_exec_short( 'gbsexport', 0, undef, $command_ref);
}

return @lines;
}


