#==============================================================
#
#   gbsinit.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSINIT @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use 5.016_003;
use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::time;
use glo::genopt;
use glo::build;
use glo::banner;
use glo::ask;
use gbsenv;
use gbsres;
use gbscmd;
use roots;
use setup;
use profile;
use gbsinit;








my $GBS_SCRIPTS_ROOT = ENV_getenv_perl_path( 'GBS_SCRIPTS_ROOT');







$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ 'rel',    'rel',    'sso', '.' , "Specific GBS release. You can also specify 'cur', 'current' or '?'" ],
[ 'noroot', 'noroot', 'bso', 0   , 'Do not implicitly swr to current root' ],
);
GENOPT_set_optdefs( [ 'gbsinit', 'gbs' ], \@genopts,
'Startup GBS',
"Alias for this command is 'gbs'");
GENOPT_parse();
my $REL = GENOPT_get( 'rel');
my $NOROOT = GENOPT_get( 'noroot');

$REL = ''
if ($REL eq '.' || $REL eq 'cur' || $REL eq 'current');




{



if ($REL eq '?')
{
my @rels = ENV_glob( "$GBS_SCRIPTS_ROOT/*/gbsswr.pl");
my @gbs_rels = map { ENV_parent_dir( $_, -2) } @rels;
my $gbs_scripts_rel = ENV_getenv( 'GBS_SCRIPTS_REL');
$REL = ASK_value_from_menu( 'Select a GBS Version', \$gbs_scripts_rel, undef, [ @gbs_rels ]);
}




my @LINES = GBSINIT_setup( $REL);




my $gbs_log_root = ENV_getenv_perl_path( 'GBS_LOG_ROOT');
my $gbs_log_path = ENV_getenv_perl_path( 'GBS_LOG_PATH');
if ($gbs_log_root eq '' and $gbs_log_path ne '')
{
ENV_say( 1, "Converting GBS_LOG_PATH to GBS_LOG_ROOT...");
PROFILE_open( undef);
ENV_setenv( GBS_LOG_ROOT => $gbs_log_path);
PROFILE_set( GBS_LOG_ROOT => $gbs_log_path);
$gbs_log_root = $gbs_log_path;
PROFILE_close( 1);	# $must write
ENV_say( 2, '*',
'* You must use gbsmaint 5 1 1 (Log Files) => Fix Log Root',
'* To move your logfiles to the proper directories',
'*');
}
ENV_setenv( GBS_LOG_PATH => '');




SETUP_check_first_invocation();
GBSINIT_validate();




{
my $TITLE = GBSRES_title();
my $gbs_show_version = BUILD_get_show_version();
my ($LICS, $LICN, $LICE, $LICD) = BUILD_get_lic();
my $TIME_START = TIME_time2num( time);
my $UNAME;
{
my ($os, $host, @rest) = ENV_uname();
$UNAME = "$host ($os [@rest])";
}



my @lines;

push @lines,  [ '',      $TITLE ];
push @lines,  undef;						# prints a separator line
push @lines,  [ 'VERSION:', $gbs_show_version ];
push @lines,  [ '',	    $GBS_SCRIPTS_ROOT ];
push @lines,  [ 'LICENSE:', "$LICN ($LICS)" ];
push @lines,  [ '',         "Expires:  $LICE. Number of days left: $LICD" ];
push @lines,  [ '',	        "*** Expired ***" ]
if ($LICD < 0);
push @lines,  [ 'START:',   "$TIME_START" ];
push @lines,  [ 'HOST:',    $UNAME ];

BANNER_print( 'JOB', [ @lines ]);

if ($LICD < 0)
{
ENV_sig( W => "This license of GBS has expired.",
"Please contact Randy Marques Consultancy for a new license.",
"You may keep using it free of change for MAINTENANCE purposes for old, finished projects.",
"- Some functionality will be disabled.");
my $rand = time % 10000;
if (ASK_int( "Please enter '$rand'", '0', undef, 0) != $rand)
{
ENV_sig( EE => "Invalid", "Cannot continue - GBS not started");
}
}
}

push @LINES, GBSENV_changed_setenv_commands( 0);	# $must_log = 0




if (!$NOROOT)
{
if (ROOTS_get_nr() > 0)
{



push @LINES, GBSCMD_get_gbs_command( 'gbsswr', '.');
} else
{
ENV_say( 1, "No Roots/Systems defined for this user",
"- Type 'swr --add' to use an existing Root/System",
"- or   'swr --new' to define a new Root/System",
"- or   'gbshelp'   to get extensive help on how to proceed");
}
}




GBSENV_write_result_script( \@LINES, 0);

}

ENV_exit( $RC);




END
{
ENV_print_end_msg( 1);

}


