#===============================================================
#
#   gbsmakemake.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSMAKEMAKE @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::genopt;
use gbsenv;
use gbsglo;
use validate;
use mm;








my $GBS_ROOT_PATH;
my $GBS_SUBSYS;
my $GBS_COMP_PATH;
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');	# commalist






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<*>',	'components',	  'sao', "", ". == current component, <empty> == all" ],
[ 'i',	'ignore_errors',  'bso',  0, "Continue generation after error(s)" ],
);
GENOPT_set_optdefs( 'gbsmakemake', \@genopts,
'Make make-file(s) for the current SubSystem',
undef);
GENOPT_parse();
my @COMPONENTS = GENOPT_get( 'components');
my $IGNORE_ERRORS = GENOPT_get( 'ignore_errors');




ENV_setenv( GBS_IGNORE_ERRORS => $IGNORE_ERRORS);




($GBS_ROOT_PATH, $GBS_SUBSYS) = VALIDATE_subsys_full_gbs();
VALIDATE_target();
$GBS_COMP_PATH = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp";




my @SELCOMP;
{
if (@COMPONENTS)
{
ENV_sig( W => "Makemake of single component(s) may compromise the integrity of the make.");
my %dup_components;
foreach my $component (@COMPONENTS)
{
if ($component eq '.')
{
$component = ENV_getenv( "GBS_COMPONENT");
ENV_sig( EE => "No current component")
if ($component eq '');
} else
{
ENV_sig( EE => "No such component '$component'")
if (! GBSGLO_component_exists( $GBS_ROOT_PATH, $GBS_SUBSYS, $component));
}

if (GBSGLO_component_does_build( $GBS_SUBSYS, $component, $GBS_TARGET))
{
if (exists $dup_components{$component})
{
ENV_say( 1, "Duplicate Component '$component' - Skipped");
} else
{
push @SELCOMP, $component;
$dup_components{$component} = 1;
}
} else
{
ENV_sig( W => "Component '$component' does not generate for Target '$GBS_TARGET' - Skipped");
}
}
}
}





$RC = MM_makemake( @SELCOMP);




ENV_exit( $RC);




END
{
ENV_print_end_msg( 1);
}


