#===============================================================
#
#   gbssub.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSSUB @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use gbsenv;
use gbscmd;
use fail;
use target;
use audit;
use bldopt;
use exec;
use run;




sub set_command_envvars();
sub set_command_envvar($$$);
sub get_command_args();












my ($JOBTYPE,	    #  audit|gen|make|cleanp_bld|cleanup_aud
$IGNORE_ERRORS, # bool
);

my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_SSTYPE = ENV_getenv( 'GBS_SSTYPE'); 	# GBS MSVS make Other

my $GBS_AUDIT;	    # DO not initilaize!

my $FOR_GEN = 0;
my $FOR_MAKE = 0;
my $FOR_AUDIT = 0;
my $FOR_CLEANUP_BLD = 0;
my $FOR_CLEANUP_AUD = 0;
my $FOR_BUILD = 0;	    # gen | make
my $FOR_CLEANUP = 0;	    # cleaup_aud | cleaup_build
















$| = 1;
my $RC = 0;




GBSENV_init();

get_command_args();

if ($JOBTYPE eq 'gen')
{
$FOR_GEN = 1;
$FOR_BUILD = 1;
} elsif ($JOBTYPE eq 'make')
{
$FOR_MAKE = 1;
$FOR_BUILD = 1;
} elsif ($JOBTYPE eq 'audit')
{
$FOR_AUDIT = 1;
$GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
} elsif ($JOBTYPE eq 'cleanup_bld')
{
$FOR_CLEANUP_BLD = 1;
$FOR_CLEANUP = 1;
} elsif ($JOBTYPE eq 'cleanup_aud')
{
$FOR_CLEANUP_AUD = 1;
$FOR_CLEANUP = 1;
$GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
} else
{
ENV_sig( F => "Invalid JOBTYPE '$JOBTYPE'");
}

ENV_setenv( GBS_IGNORE_ERRORS => $IGNORE_ERRORS);
ENV_setenv( GBS_MAKE => GBSCMD_get_os_command( 'make'));




{
my $options_banner = BLDOPT_init( $GBS_AUDIT);



if ($FOR_BUILD || $FOR_AUDIT)	    # exclude FOR_CLEANUPs
{





foreach my $src_type (TARGET_get_src_types( $GBS_TARGET))
{
my $type_postfix = '_' . uc substr( $src_type, 1);
my @env_def_refs = BLDOPT_get_envs_settings( $GBS_SUBSYS, '', $src_type);

foreach my $ref (@env_def_refs)
{
my ($envvar_name, @envvar_values) = @{$ref};
if ($envvar_name eq 'GBS_FLAGS')
{
push @envvar_values, BLDOPT_get_options_flags( $src_type, '$@.map');    # '%@' == current make target
}
$ENV{"$envvar_name$type_postfix"} = "@envvar_values";	# we do not want to use ENV_setenv because of the "" if empty
}
}

}




my $gbs_ignore_errors = ENV_getenv( 'GBS_IGNORE_ERRORS');
if ($GBS_SSTYPE eq 'make')
{



my @make_flags;
push @make_flags, '-i'
if ($gbs_ignore_errors);
$ENV{GBS_MAKE_FLAGS} = "@make_flags";	    # we do not want to use ENV_setenv because of the "" if empty
set_command_envvars();
} elsif ($GBS_SSTYPE eq 'MSVS')
{



my $gbs_mode = ENV_getenv( 'GBS_MODE');
my $msbuild_config = ($gbs_mode eq 'FINAL') ? 'Release' : ($gbs_mode eq 'DEBUG') ? 'Debug' : '';
$ENV{GBS_MSBUILD_CONFIG} = $msbuild_config; # we do not want to use ENV_setenv because of the "" if empty
} elsif ($GBS_SSTYPE eq 'Other')
{



set_command_envvars();
} else
{
ENV_sig( EE => "Invalid SubSys Type '$GBS_SSTYPE");
}




ENV_say( 1, "** $options_banner");




$RC = RUN_gbssub_script( $JOBTYPE);
ENV_say( 0, '-');
FAIL_log( "gbssub$JOBTYPE", $RC, $GBS_SUBSYS, '', '')
if ($RC != 0);

if ($FOR_BUILD)
{



if (-d "$GBS_ROOT_PATH/res/$GBS_SUBSYS" || -d "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/export")
{
if ($IGNORE_ERRORS || $RC == 0)
{
ENV_say( 1, "Implicit gbsexport of $GBS_SUBSYS");
my @gbsexport_opts;
push @gbsexport_opts, '--i' if ($IGNORE_ERRORS);
my $this_rc = EXEC_run( 'gbsexport.pl', \@gbsexport_opts);
$RC = $this_rc if ($this_rc > $RC);
FAIL_log( 'gbsexport', $this_rc, $GBS_SUBSYS, '', '')
if ($this_rc != 0);
} else
{
ENV_say( 1, "SKIPPED: gbsexport of $GBS_SUBSYS");
}
}
}
}




ENV_exit( $RC);




END
{
ENV_print_end_msg( 1);
}




sub set_command_envvars()
{
foreach my $src_type (TARGET_get_src_types( $GBS_TARGET))
{
my $command_data_refs_ref = TARGET_get_src_command_data_refs( $GBS_TARGET, $src_type);
set_command_envvar( TARGET => $src_type, $command_data_refs_ref);
}
if ($FOR_AUDIT)
{
foreach my $src_type (AUDIT_get_src_types( $GBS_AUDIT))
{
my $command_data_refs_ref = AUDIT_get_src_command_data_refs( $GBS_AUDIT, $src_type);
set_command_envvar( AUDIT => $src_type, $command_data_refs_ref);
}
}
}




sub set_command_envvar($$$)
{
my ($uc_att_type,		    # AUDIT, TARGET or TOOL
$src_type,
$command_data_refs_ref,	    # [$command_type, $command_line, $ok_values, $exec_condition, $pos_arg_tokens];

) = @_;

my $command_type = $command_data_refs_ref->[0]->[0];    # not used (yet)
my $command_line = $command_data_refs_ref->[0]->[1];
my @command_words = split( ' ', $command_line);
my $command = shift @command_words;
$command = ENV_expand_envs( $command);
my @command = ($command);
while (defined $command_words[0] && $command_words[0] !~ /^[\$\%]/)
{
push @command, shift @command_words;
}

my $type_uc = uc substr( $src_type, 1);
ENV_setenv( "GBS_${uc_att_type}_$type_uc" => "@command");

}




sub get_command_args()
{


($JOBTYPE,	     #  audit|gen|make|cleanp_bld|cleanup_aud
$IGNORE_ERRORS, # bool
) = ARGS_get( 2, 'perl gbssub.pl audit|gen|make|cleanp_bld|cleanup_aud <ignore_errors>', undef);
}



