#==============================================================
#
#   gbsswa.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSSWA @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::ask;
use glo::genopt;
use gbsenv;
use gbsglo;
use gbscmd;
use run;
use validate;
use swa;
use swglo;
use plugin;




sub ask_audit();




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');

my $GBS_AUDITS = ENV_getenv( 'GBS_AUDITS');	# commalist
my @GBS_AUDITS = ENV_getenv( 'GBS_AUDITS');

my $CANCELLED = 0;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',	'audit',	    'sso',  '', "Audit: '' == ASK, '-' == none, '.' == current" ],
[ 'new',	'create_new_audit', 'bso',   0, "Create a new Audit" ],
[ 'remove', 'remove_audit',     'bso',   0, "Remove a Audit from the Audits List" ],
);
my @genconflicts = (
[ [ new => 1 ], [ '=' => remove => 1] ],
);
GENOPT_set_optdefs( [ 'gbsswa', 'swt' ], \@genopts,
'Set and/or Create new Current Audit',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $AUDIT = GENOPT_get( 'audit');
my $NEW = GENOPT_get( 'create_new_audit');
my $REMOVE = GENOPT_get( 'remove_audit');




VALIDATE_root();




if (!$NEW && @GBS_AUDITS == 0)
{
if (ASK_YN( "No Audit(s) defined. Define now?", 'Y') eq 'Y')
{
$NEW = 1;
}
}




if ($NEW)
{
$AUDIT = RUN_gbsnew( 'new_audit', $AUDIT);
@GBS_AUDITS = ENV_getenv( 'GBS_AUDITS');
if ($AUDIT eq '')
{
$AUDIT = '.';
$CANCELLED = 1;
} else
{
ENV_say( 1, "Current Audit is $AUDIT");
}
} elsif ($REMOVE)
{
my $removed_audit = RUN_gbsnew( 'rem_audit', $AUDIT);
@GBS_AUDITS = ENV_getenv( 'GBS_AUDITS');
if ($GBS_AUDIT eq $removed_audit)
{
ENV_say( 1, "Current Audit removed");
$AUDIT = '';	# No current
} else
{
ENV_say( 1, "Current Audit is $GBS_AUDIT");
$AUDIT = '.';	# Keep current
}
}

if (!$CANCELLED)
{



if ($AUDIT eq '')
{
$AUDIT = ask_audit();
}
my $new_audit = SWGLO_validate_audit( $GBS_SUBSYS, $GBS_COMPONENT, $GBS_TARGET, $AUDIT);






if ($new_audit ne ENV_getenv( 'GBS_AUDIT'))
{
my $must_write = $new_audit ne '' || $AUDIT eq '-';
SWA_set( $new_audit, $must_write);
}
ENV_sig( W => "No current Audit")
if ($new_audit eq '' && $AUDIT ne '-');








{
my @lines;
if (ENV_get_changed_envs())
{
push @lines, GBSENV_changed_setenv_commands( 0);
}
push @lines, GBSCMD_get_gbs_command( 'gbsshow', '--brief');

GBSENV_write_result_script( \@lines, 0);
}
}

ENV_exit( $RC);




END
{
ENV_print_end_msg( 0);
}




sub ask_audit()
{
my $audit = '';




if (scalar @GBS_AUDITS == 0)
{
ENV_sig( EE => "No Audit(s) defined - use 'swt --new' to define Audits");
} elsif (scalar @GBS_AUDITS == 1)
{
$audit = $GBS_AUDITS[0];
ENV_say( 1, "Single Audit '$audit' taken by default");
} else
{



my @audit_refs;


if ($GBS_SUBSYS eq '')
{



foreach my $audit (@GBS_AUDITS)
{
push @audit_refs, [ $audit, '  ' ];
}
} elsif ($GBS_COMPONENT eq '')
{



foreach my $audit (@GBS_AUDITS)
{
my $is_subsys = (GBSGLO_subsystem_does_audit( $GBS_SUBSYS, $audit, $GBS_TARGET)) ? '+' : '-';
push @audit_refs, [ $audit, "$is_subsys " ];
}
} else
{



foreach my $audit (@GBS_AUDITS)
{
my $is_subsys = (GBSGLO_subsystem_does_audit( $GBS_SUBSYS, $audit, $GBS_TARGET)) ? '+' : '-';
my $is_component = (GBSGLO_component_does_audit( $GBS_SUBSYS, $GBS_COMPONENT, $audit, $GBS_TARGET)) ? '+' : '-';
push @audit_refs, [ $audit, "$is_subsys$is_component" ];
}
}

my @menu_refs = ( [ '', '-' ] );
foreach my $ref (@audit_refs)
{
my ($audit, $availability) = @{$ref};
my $plugin_name = PLUGIN_get_att_plugin_name( $audit);
my $display_text_ref = [ $audit, "[$availability] [$plugin_name]" ];
push @menu_refs, [ $display_text_ref , $audit ];
}

my $default_audit = ENV_getenv( 'GBS_AUDIT');
my $prompt = "Select Audit ($GBS_SUBSYS,$GBS_COMPONENT)";
my $help   = '[<s><c>] -> s = Active for current Subsys, c = Active for current Component';
$audit = ASK_value_from_menu( [ $prompt, $help ], \$default_audit, undef, [ @menu_refs ]);
$audit = '-'
if ($audit eq '');
}

return $audit;
}


