#==============================================================
#
#   gbsswr_set.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSSWR_SET @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::shell;
use glo::genopt;
use glo::slurp;
use gbsenv;
use gbscmd;
use gbsglo;
use plugin;
use swt;
use swb;
use swa;
use swc;
use sws;
use swr;
use swglo;
use gbsdb;




sub cleanup_tmp_dir();




my $CWD = ENV_cwd();
my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();

my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SYSTEM_NAME = ENV_getenv( 'GBS_SYSTEM_NAME');
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_TOOL = ENV_getenv( 'GBS_TOOL');






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ 'new',	'new_root',	   'bso',	0, "Must create new Target" ],
[ 'exec',	'execute_command', 'sso',      '', "(GBS-)Command(s) to execute. 'none' = OK" ],
[ 'nocd',	'no_cd',	   'bso',	0, "Do not 'cd' to the selected directories "],
);
my @genconflicts = (
);
GENOPT_set_optdefs( [ 'gbsswr_set', '' ], \@genopts,
'Internal Use',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $NEW = GENOPT_get( 'new_root');
my $EXEC = GENOPT_get( 'execute_command');
my $NO_CD = GENOPT_get( 'no_cd');




{



{
my $must_write = ($GBS_ROOT_PATH ne '');
SWR_set( $GBS_ROOT_PATH, $must_write);
}
{
my $must_write = ($GBS_SUBSYS ne '');
SWS_set( $GBS_SUBSYS, $must_write);
}
{
my $must_write = ($GBS_COMPONENT ne '');
SWC_set( $GBS_SUBSYS, $GBS_COMPONENT, $must_write);
}
if ($GBS_ROOT_PATH ne '')
{
SWB_set( $GBS_TARGET, 1);
SWA_set( $GBS_AUDIT, 1);
SWT_set( $GBS_TOOL, 1);
}

if ($GBS_ROOT_PATH ne '')
{
if (GBSENV_mode_is_interactive())
{
GBSDB_last_root_write( $GBS_ROOT_PATH);

ENV_sig( W => "No current SubSystem")
if ($GBS_SUBSYS eq '');
ENV_sig( W => "No current Component")
if ($GBS_COMPONENT eq '' && GBSGLO_subsystem_is_full_gbs( $GBS_SUBSYS));
ENV_sig( W => "No current Target")
if ($GBS_TARGET eq '');
}
} else
{
ENV_sig( W => "No current Root");
}




{



my @lines;




if ($GBS_ROOT_PATH ne '')
{
push @lines, SHELL_echo( '')
if ($GBS_ROOT_PATH ne '');




push @lines, GBSENV_changed_setenv_commands( 0);	# $must_log = 0




push @lines, SWGLO_set_title( $GBS_ROOT_PATH, $GBS_SUBSYS, $GBS_COMPONENT);
}




my $new_cwd = ($NO_CD) ? $CWD : SWGLO_new_cwd( $GBS_ROOT_PATH, $GBS_SUBSYS, $GBS_COMPONENT);
push @lines, SHELL_conditional_eq( GBS_RC => 0, SHELL_cd( $new_cwd));




push @lines, SHELL_conditional_ne( GBS_RC => 0, SHELL_setenv( GBS_ROOT_PATH => '""'),
SHELL_setenv( GBS_SUBSYS => '""'),
SHELL_setenv( GBS_COMPONENT => '""'),
SHELL_setenv( GBS_TARGET => '""'),
SHELL_setenv( GBS_AUDIT => '""'),
SHELL_setenv( GBS_TOOL => '""'),
SWR_cleanup(),
SWGLO_set_title(),
SHELL_cd( $GBS_ROOT_PATH));




push @lines, SHELL_conditional_eq( GBS_RC => 0, GBSCMD_get_gbs_command( 'gbsshow', '--brief'));




push @lines, SHELL_conditional_ne( GBS_RC => 0, SHELL_echo_failed());

if ($NEW)
{



my @this_platform_targets = PLUGIN_get_att_names( 'build');

push @lines, SHELL_conditional_eq( GBS_RC => 0, GBSCMD_get_gbs_command( 'gbsswt', '--new'))
if (@this_platform_targets == 0);
} elsif ($EXEC ne '' && $EXEC ne 'none')
{



my @exec_args = split( ' ', $EXEC);
foreach my $arg (@exec_args)
{
$arg =~ s/%21/ /g;		# set by GBSJOB_sub
}
my $command = shift @exec_args;
$command = 'gbsshow'
if ($command eq 'gbs');
my $script_command = ENV_command_filespec( "$GBS_SCRIPTS_PATH/$command");
my $command_ref;
if (-f $script_command)
{
$command_ref = ENV_command_ref( $script_command, \@exec_args);
} elsif (-f "$GBS_SCRIPTS_PATH/$command.pl")
{
$command_ref = ENV_command_ref( "$command.pl", \@exec_args);
} else
{
ENV_sig( EE => "Invalid/Empty '--exec' command: $EXEC");
}
my $exec_command = ENV_command_line( $command_ref);


push @lines, SHELL_conditional_eq( GBS_RC => 0, $exec_command);
push @lines, SHELL_setrc();
} else
{
if ($GBS_ROOT_PATH ne '' && GBSENV_mode_is_interactive())
{
cleanup_tmp_dir();
}
}

push @lines, SHELL_exit_rc( 1);			# $is_sourced = 1
GBSENV_write_result_script( \@lines, 0);	# $print_lines = 0
}
}

ENV_exit( $RC);




END
{
ENV_print_end_msg( 0);
}




sub cleanup_tmp_dir()
{
ENV_say( 1, 'Cleanup tmp dir...');

my $tmp_path = "$GBS_ROOT_PATH/tmp";
my $now = time();				# get current time
my $days = 7;				# delete older than $days
my $age = $now - (60 * 60 * 24 * $days);	# convert days into seconds
my $count = 0;
foreach my $ref (SLURP_dir_tree( $tmp_path, 0))
{
my ($is_dir, $rel_spec) = @{$ref};
if (!$is_dir)
{
my $filespec = "$tmp_path/$rel_spec";
if ((stat $filespec)[9] < $age)
{

unlink $filespec;
$count++;
} else
{

}
}
}
ENV_say( 2, "Deleted $count tmp files")
if ($count > 0);
}


