#===============================================================
#
#   gbssysall.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSSYSALL @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::genopt;
use gbsenv;
use gbsoptenv;
use validate;
use gbscmd;














$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<*>',	 'steps',	  'sao',      "", "Steps and/or Aliases. <empty> == all select steps, 'ALL' == force all steps, '?' == show steps" ],
[ 'i' ,	 'ignore_errors', 'bso',       0, "Continue generation after error(s)" ],
[ 'vs',	 'view_sum',	  'bso',       1, "View summary at completion" ],
[ 'audits',  'audits',	  'sao',     '*', "(wild-)Audits to run. '' or '*' == All, '.' == Current" ],
[ 'targets', 'targets',	  'sao',     '*', "(wild-)Targets to Audit. '' or '*' == All, '.' == Current" ],
[ 'tools',   'tools',	  'sao',     '*', "(wild-)Tools to Execute. '' or '*' == All, '.' == Current, '-' == None" ],
[ 'files',	 'files',	  'sao', '*:*.*', "Files to audit" ],
[ 'make',	 'make',	  'bso',       0, "Run gbssysmake instead of gbssysgen" ],
[ 'fg',	 'foreground',	  'bso',       0, "Runs in the foreground if set" ],
[ 'at',	 'delay',	  'tso',   'now', "Starting time (10:20), delta (+10:10) or now" ],
[ 'sm',	 'mail',	  'bso',       0, "Send mail on completion" ],
[ 'n',	 'notify',	  'bso',       1, "Notify user on completion (bg/nowait only)" ],
[ 'wait',    'wait',          'bso',       0, "Wait for completion of bg jobs" ],
[ 'jobs',	 'jobs',     'isor1..9',       2, 'Max nr parallel jobs within a submitted job' ],
[ 'c',	 'comment',	  'sso',      "", 'Comments - will be shown in Summary - Specify %20 for space' ],
);
my @genconflicts = (
[ [ fg   => 1 ], '=' => 'at', '=' => 'sm', '=' => 'n' ],
[ [ wait => 1 ], '=' => 'n' ],
);
my @genenvs = qw( LOG_PATH FLAGS_* APP_* );
GENOPT_set_optdefs( 'gbssysall', \@genopts,
'Execute gbssysgen/gbssysmake, gbssysaudit and gbssystool commands',
'Commands are only executed if applicable');
GENOPT_set_conflicts( \@genconflicts);
GENOPT_set_envdefs( \&GBSOPTENV_set, \&GBSOPTENV_print, 'GBS_', \@genenvs);
GENOPT_parse();
my @STEPS = GENOPT_get( 'steps');
my $IGNORE_ERRORS = GENOPT_get( 'ignore_errors');
my $MUST_VIEW_SUM = GENOPT_get( 'view_sum');
my @AUDITS = GENOPT_get( 'audits');
my @TARGETS = GENOPT_get( 'targets');
my @TOOLS = GENOPT_get( 'tools');
my @FILES = GENOPT_get( 'files');
my $MAKE = GENOPT_get( 'make');

my $FOREGROUND = GENOPT_get( 'foreground');
my $DELAY = GENOPT_get( 'delay');
my $MAIL = GENOPT_get( 'mail');
my $NOTIFY = GENOPT_get( 'notify');
my $WAIT = GENOPT_get( 'wait');
my $JOBS = GENOPT_get( 'jobs');
my $COMMENT = GENOPT_get( 'comment');




$NOTIFY = 0
if ($WAIT);




VALIDATE_root();

my @COMMAND_REFS = (

[ Targets => gbssysgen   => undef,	    [ qw( make audits tools view_sum ) ] ],
[ Audits  => gbssysaudit => undef,	    [ qw( make tools ) ] ],
[ Tools   => gbssystool  => \@TOOLS,    [ qw( make audits targets tools files ) ] ],
);
if ($MAKE)
{
$COMMAND_REFS[0]->[1] = 'gbssysmake';
}

foreach my $ref (@COMMAND_REFS)
{
my ($name, $command, $atts_ref, $excludes_ref) = @{$ref};
my $envvar_name = uc "GBS_$name";

if (ENV_getenv( $envvar_name))
{
if (!defined $atts_ref || "@{$atts_ref}" ne '-')
{
ENV_say( 1, "$command...");
my @args = GENOPT_get_changed( $excludes_ref);
ENV_trace( "args=@args");
my $rc = ENV_system( GBSCMD_get_gbs_command( $command, @args), undef);
$RC = $rc
if ($rc > $RC);
}
} else
{
ENV_say( 1, "No $name defined");
}
}




ENV_exit( $RC);




END
{
ENV_print_end_msg( 1);
}


