#=================================================
#
#   env.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#   Rules:
#	Do not use $&, $` or $'
#
#===============================================================
package glo::env;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
ENV_set_application
ENV_update_application_scripts_path
ENV_get_application_name
ENV_get_application_scripts_path
ENV_get_application_data_path
ENV_get_application_base_path
ENV_get_perl_cmd
ENV_get_main_name
ENV_is_development
ENV_is_lunix
ENV_is_linux
ENV_is_win32
ENV_this_osname
ENV_other_osname
ENV_is_interactive
ENV_enable_beeps
ENV_beeps_are_enabled
ENV_beep
ENV_reap_beeps
ENV_set_verbose
ENV_say
ENV_say_stderr
ENV_whisper
ENV_trace
ENV_print
ENV_get_prefix
ENV_sig
ENV_file_sig
ENV_try
ENV_exit
ENV_get_nr_errors
ENV_stackdump
ENV_debug
ENV_is_debug
ENV_print_end_msg
ENV_printable
ENV_clone
ENV_dump
ENV_eval_empty
ENV_eval_undef
ENV_undef2empty
ENV_getenv
ENV_getenv_bool
ENV_getenv_perl_path
ENV_setenv
ENV_setenv_os_path
ENV_get_changed_envs
ENV_detab
ENV_command_filespec
ENV_shell_filetype
ENV_is_shell_file
ENV_chmod
ENV_get_hostname
ENV_uname
ENV_unique_id
ENV_get_userid
ENV_glob
ENV_glob_files
ENV_wild_2_re
ENV_is_wildcard
ENV_wildcard
ENV_wildcards
ENV_not_wildcards
ENV_get_term_size
ENV_get_term_size_default
ENV_set_background_term_size
ENV_is_abs_path
ENV_is_abs_path_perl
ENV_is_option
ENV_is_in_path
ENV_parent_path
ENV_parent_dir
ENV_set_tmpdir_path
ENV_get_tmpdir_path
ENV_get_os_tmpdir_path
ENV_get_tmp_spec
ENV_get_tmp_path
ENV_os_path
ENV_perl_paths
ENV_perl_canon_paths
ENV_perl_path_nospace
ENV_perl_path_space
ENV_canonicalize_path
ENV_paths_equal
ENV_expand_envs
ENV_expand_arg
ENV_delink_spec
ENV_rel_spec
ENV_abs_path
ENV_rel_path
ENV_shortest_path
ENV_short_path_name
ENV_split_spec_dnt
ENV_split_spec_df
ENV_split_spec_d
ENV_split_spec_f
ENV_split_spec_nt
ENV_split_spec_n
ENV_split_spec_t
ENV_split_path
ENV_join_path
ENV_chdir
ENV_cwd
ENV_pushd
ENV_popd
ENV_file_is_newer
ENV_file_is_same
ENV_mkdir
ENV_mkpath
ENV_rmdir
ENV_rmtree
ENV_readdir
ENV_rename
ENV_dir_count
ENV_unlink
ENV_copy_file
ENV_backup_file
ENV_touch_file
ENV_hide_file
ENV_max_command_line
ENV_command_split
ENV_new_command_ref
ENV_get_command_ref_data
ENV_command_ref
ENV_command_line
ENV_find_in_perl_inc
ENV_system
ENV_backtick
ENV_run3
ENV_exec
ENV_rc_status
ENV_detab
ENV_split_quoted_space
ENV_join_quoted_space
ENV_enquote
ENV_dequote
ENV_split_lines
ENV_redirect_stdouterr
ENV_restore_stdouterr
ENV_get_desktop_path
ENV_get_startmenu_path
ENV_get_programs_path
ENV_get_user_path
ENV_is_administrator
);
}




BEGIN
{
if ($^O eq 'MSWin32')
{
require glo::os_win;
import  glo::os_win;
} else
{
require glo::os_linux;
import  glo::os_linux;
}
}
use File::Glob ':glob';      # Override glob built-in
use Term::ReadKey qw(GetTerminalSize);




sub ENV_set_application($$$$$$);
sub ENV_update_application_scripts_path($);
sub ENV_get_application_name();
sub ENV_get_application_scripts_path();
sub ENV_get_application_base_path();
sub ENV_get_perl_cmd();
sub ENV_get_main_name();
sub ENV_is_development();
sub ENV_is_lunix();
sub ENV_is_linux();
sub ENV_is_win32();
sub ENV_this_osname();
sub ENV_other_osname();
sub ENV_is_interactive();
sub ENV_enable_beeps($);
sub ENV_beeps_are_enabled();
sub ENV_beep();
sub ENV_reap_beeps();
sub ENV_set_verbose(;$);
sub ENV_say($@);
sub ENV_say_stderr($@);
sub ENV_whisper($@);
sub ENV_trace(@);
sub ENV_print($@);
sub ENV_get_prefix($);
sub ENV_sig($@);
sub ENV_file_sig($$@);
sub ENV_try($$);
sub ENV_exit(;$);
sub ENV_get_nr_errors();
sub ENV_stackdump();
sub ENV_debug($@);
sub ENV_is_debug(;$);
sub ENV_print_end_msg($);
sub ENV_printable($);
sub ENV_clone($);
sub ENV_dump($$;$$);
sub ENV_eval_empty($);
sub ENV_eval_undef($);
sub ENV_undef2empty(@);
sub ENV_getenv($);
sub ENV_getenv_bool($);
sub ENV_getenv_perl_path($);
sub ENV_setenv($$);
sub ENV_setenv_os_path($$);
sub ENV_get_changed_envs();
sub ENV_command_filespec($);
sub ENV_shell_filetype();
sub ENV_is_shell_file($);
sub ENV_chmod($@);
sub ENV_get_hostname();
sub ENV_uname();
sub ENV_unique_id();
sub ENV_get_userid();
sub ENV_glob(@);
sub ENV_glob_files(@);
sub ENV_wild_2_re($);
sub ENV_is_wildcard($);
sub ENV_wildcard($$);
sub ENV_wildcards($$$);
sub ENV_not_wildcards($$);
sub ENV_get_term_size();
sub ENV_get_term_size_default();
sub ENV_set_background_term_size();
sub ENV_is_abs_path($);
sub ENV_is_abs_path_perl($);
sub ENV_is_option($);
sub ENV_is_in_path($$);
sub ENV_parent_path($);
sub ENV_parent_dir($$);
sub ENV_get_application_data_path();
sub ENV_set_tmpdir_path($);
sub ENV_get_tmpdir_path();
sub ENV_get_os_tmpdir_path();
sub ENV_get_tmp_spec($);
sub ENV_get_tmp_path($);
sub ENV_os_path(@);
sub ENV_perl_paths(@);
sub ENV_perl_canon_paths(@);
sub ENV_perl_path_nospace(@);
sub ENV_perl_path_space(@);
sub ENV_canonicalize_path($);
sub ENV_paths_equal($$);
sub ENV_expand_envs($);
sub ENV_expand_arg($);
sub ENV_delink_spec($);
sub ENV_rel_spec($$);
sub ENV_abs_path($$);
sub ENV_rel_path($$);
sub ENV_shortest_path($$);
sub ENV_short_path_name(@);
sub ENV_split_spec_dnt($);
sub ENV_split_spec_df($);
sub ENV_split_spec_d($);
sub ENV_split_spec_f($);
sub ENV_split_spec_nt($);
sub ENV_split_spec_n($);
sub ENV_split_spec_t($);
sub ENV_split_path($);
sub ENV_join_path($);
sub ENV_chdir($);
sub ENV_cwd();
sub ENV_pushd($);
sub ENV_popd();
sub ENV_file_is_newer($$);
sub ENV_file_is_same($$);
sub ENV_mkdir($;$$);
sub ENV_mkpath($;$$);
sub ENV_rmdir($;$$);
sub ENV_rmtree($$;$$);
sub ENV_readdir($;$);
sub ENV_rename($$;$);
sub ENV_dir_count($;$);
sub ENV_unlink($;$$);
sub ENV_copy_file($$$);
sub ENV_backup_file($$$);
sub ENV_touch_file($;$);
sub ENV_hide_file($$);
sub ENV_max_command_line();
sub ENV_command_split($);
sub ENV_new_command_ref($$$$);
sub ENV_get_command_ref_data($);
sub ENV_command_ref($$);
sub ENV_command_line($);
sub ENV_find_in_perl_inc($$);
sub ENV_system($$);
sub ENV_backtick($$$);
sub ENV_run3($$$$$);
sub ENV_exec($$);
sub ENV_rc_status($$);
sub ENV_detab($);
sub ENV_split_quoted_space($);
sub ENV_join_quoted_space(@);
sub ENV_enquote(@);
sub ENV_dequote(@);
sub ENV_split_lines($@);
sub ENV_redirect_stdouterr($);
sub ENV_restore_stdouterr();
sub ENV_get_desktop_path($);
sub ENV_get_startmenu_path($);
sub ENV_get_programs_path($);
sub ENV_get_user_path($);
sub ENV_is_administrator();

sub handle_rc($$$;$);
sub set_app_path($$$);
sub set_app_value($$$);
sub deref_lines($$);
sub format_lines($@);
sub exec_beep();
sub handle_sig($$$);
sub get_debug_name();
sub dump_item($$$$$);
sub de_ref($$);
sub slurp_file($$);
sub spit_file($$$);
sub my_die(@);




(my $MAIN_NAME = uc $0 ) =~ s/\.PL$//;	    # UC and remove .pl
$MAIN_NAME =~ s!.*[/\\]!!;		    # remove leading device/directories

my $MAIN_NAME_ = ' ' x ((length $MAIN_NAME) + 1);

my $PREFIX = $MAIN_NAME . '/ENV';

my $IS_WIN32 = ($^O eq 'MSWin32');
my $IS_LINUX = ($^O eq 'linux');
my $IS_LUNIX = !$IS_WIN32;

my $IS_INTERACTIVE = (-t STDIN && -t STDOUT);	    # connected to keyboard and window

my $XTERM_DEFAULT_WIDTH = 100;
my $XTERM_DEFAULT_HEIGHT = 60;
my $XTERM_BACKGROUND_WIDTH = 130;
my $XTERM_WIDTH;
my $XTERM_HEIGHT;

my $PATH_SEPARATOR = ($IS_WIN32) ? ';' : ':';
my @SHELL_FILETYPES = ($IS_WIN32) ? qw( .bat .sh) : qw( .sh .bat);

my $IS_DEVELOPMENT;	    # Do not initialize!

my $APPLICATION_NAME;	    # Do not initialize!	e.g.: GBS
my $APPLICATION_LONGNAME;   # Do not initialize!	e.g.: Generic Build Support
my $APPLICATION_SCRIPTS_PATH;  # Do not initialize!	e.g.: GBS_SCRIPTS_PATH
my $APPLICATION_DATA_PATH;  # Do not initialize!	e.g.:
my $APPLICATION_BASE_PATH;  # Do not initialize!	e.g.: GBS_BASE_PATH
my $PERL_CMD = 'perl -w';

my $APPLICATION_DEBUG_PREFIX;			    # e.g.: GBSDEBUG_
my $DEBUG_ALL;					    # bool
my %DEBUG_NAMES;

my $DEBUG_ACTIVE = 0;

my $OS_TMPDIR_PATH;	    # Do not initialize!
my $TMPDIR_PATH;	    # Do not initialize!
my $LOG_PATH;		    # Do not initialize!

my $MAX_COMMAND_LINE;	    # Do not initialize!

my $VERBOSE;		    # bool

my $BEEPS_ENABLED = 1;
my $NR_BEEPS = 0;
my $NR_WARNINGS = 0;
my $NR_ERRORS = 0;

my $UNIQUE_COUNT = 0;

my $RUNNING_IN_EXEC = 0;

my %CENVS;


my %QUOTED_PATHS;


my $SAVED_STDOUT;
my $SAVED_STDERR;




my %SIG_TEXT = (
I   => 'INFO',
W   => 'WARNING',
E   => 'ERROR',
F   => 'INTERNAL ERROR',
);







sub ENV_set_application($$$$$$)
{
my ($app_name,		    #					    e.g.: GBS
$app_longname,		    #					    e.g.: Generic Build Support
$app_scripts_path,	    # must be canonicalised root_path!	    e.g.: GBS_SCRIPTS_PATH
$app_data_path,		    # must be verified existing directory   e.g.: GBS_HOME_PATH
$app_base_path,		    # must be verified existing directory   e.g.: GBS_BASE_PATH
$perl_cmd,		    # must exist			    e.g.: GBS_PERL_CMD
) = @_;



if (defined $app_name)
{
set_app_value( APPLICATION_NAME => $app_name, \$APPLICATION_NAME);
if (defined $APPLICATION_NAME)
{



my $app_debug_prefix = "${APPLICATION_NAME}DEBUG_";
my @debug_env_names = grep /^$app_debug_prefix/, keys %ENV;
foreach my $env_name (grep /^$app_debug_prefix/, keys %ENV)
{
my $env_value = ENV_getenv( $env_name);
if ($env_value eq '1')
{
my $debug_name = substr( $env_name, length( $app_debug_prefix));
$DEBUG_NAMES{$debug_name} = 1;
ENV_whisper( 1, "DBG: $env_name=$env_value");
$DEBUG_ACTIVE = 1;
}
}
$DEBUG_ALL = $DEBUG_NAMES{ALL};
$VERBOSE = ENV_getenv( "${APPLICATION_NAME}DEBUG_VERBOSE");	    # E.g.: GBSDEBUG_VERBOSE
}
}

if (defined $app_longname)
{
set_app_value( APPLICATION_LONGNAME => $app_longname, \$APPLICATION_LONGNAME);
}

if (defined $app_scripts_path)
{
set_app_path( APPLICATION_SCRIPTS_PATH => $app_scripts_path, \$APPLICATION_SCRIPTS_PATH);
}

if (defined $app_data_path)
{
set_app_path( APPLICATION_DATA_PATH => $app_data_path, \$APPLICATION_DATA_PATH);
} else
{
$APPLICATION_DATA_PATH = ENV_perl_path_nospace( ENV_get_user_path( 0)) . "/$APPLICATION_NAME";	# %MYDOCUMENTS% or ~/
}

if (defined $app_base_path)
{
set_app_path( APPLICATION_BASE_PATH => $app_base_path, \$APPLICATION_BASE_PATH);
}

if (defined $perl_cmd)
{
set_app_value( PERL_CMD => $perl_cmd, \$PERL_CMD);
$PERL_CMD .= ' -w'
if (defined $PERL_CMD && $PERL_CMD !~ /\s-w$/);
}
}




sub ENV_update_application_scripts_path($)
{
my ($app_scripts_path,	    # must be canonicalised root_path!	    e.g.: GBS_SCRIPTS_PATH
) = @_;

set_app_path( APPLICATION_SCRIPTS_PATH => $app_scripts_path, \$APPLICATION_SCRIPTS_PATH);
}




sub set_app_value($$$)
{
my ($name,
$value,		    # value or '<value_envvar>'
$value_ref,	    # Out
) = @_;

my $new_value;	    # value or undef
my $old_value = $$value_ref;

if (substr( $value, 0, 1) eq '<' && substr( $value, -1, 1) eq '>')
{
my $envvar_name = substr( $value, 1, -1);
$new_value = ENV_eval_undef(  $ENV{$envvar_name});
ENV_sig( F => "EnvVar '$envvar_name' was not set")
if (!defined $new_value);
} else
{
$new_value = $value;
}
ENV_debug( 1, "$name ($old_value) changed to '$new_value'")
if (defined $old_value && $new_value ne $old_value);
$$value_ref = $new_value;
}




sub set_app_path($$$)
{
my ($name,
$path,		    # path or '<path_envvar>'
$path_ref	    # Out
) = @_;

my $new_path;	    # path or undef
my $old_path = $$path_ref;

if (substr( $path, 0, 1) eq '<' && substr( $path, -1, 1) eq '>')
{
my $envvar_name = substr( $path, 1, -1);
$new_path = ENV_eval_undef( $ENV{$envvar_name});
ENV_sig( F => "EnvVar '$envvar_name' was not set")
if (!defined $new_path);
$new_path = ENV_perl_paths( $new_path);
ENV_sig( F => "$name: $envvar_name => '$new_path' does not exist")
if (! -d $new_path);
} else
{
$new_path = $path;
ENV_sig( F => "$name: '$new_path' does not exist")
if (! -d $new_path);
}
$new_path = ENV_perl_path_nospace( $new_path)
if ($IS_WIN32);

ENV_debug( 1, "$name ($old_path) path changed to '$new_path'")
if (defined $old_path && $new_path ne $old_path);
$$path_ref = $new_path;
}





sub ENV_get_application_name()
{
return wantarray ? ($APPLICATION_NAME, $APPLICATION_LONGNAME) : $APPLICATION_NAME;
}





sub ENV_get_application_scripts_path()
{
ENV_sig( F => 'ENV_set_application was not called')
if (!defined $APPLICATION_SCRIPTS_PATH);

return $APPLICATION_SCRIPTS_PATH;
}





sub ENV_get_application_data_path()
{
ENV_sig( F => 'ENV_set_application was not called')
if (!defined $APPLICATION_DATA_PATH);

return $APPLICATION_DATA_PATH;
}






sub ENV_get_application_base_path()
{
return $APPLICATION_BASE_PATH;
}





sub ENV_get_perl_cmd()
{
return $PERL_CMD;
}




sub ENV_get_main_name()
{
return $MAIN_NAME;
}




sub ENV_is_development()
{
$IS_DEVELOPMENT = -d "$APPLICATION_SCRIPTS_PATH/Tools"
if (!defined $IS_DEVELOPMENT);
return $IS_DEVELOPMENT;
}




sub ENV_is_lunix()
{
return $IS_LUNIX;
}




sub ENV_is_linux()
{
return $IS_LINUX;
}




sub ENV_is_win32()
{
return $IS_WIN32;
}




sub ENV_this_osname()
{
return $^O;	    # MSWin32 or linux
}




sub ENV_other_osname()
{
return ($IS_WIN32) ? 'linux' : 'MSWin32';
}




sub ENV_is_interactive()
{
return $IS_INTERACTIVE;
}




sub ENV_enable_beeps($)
{
$BEEPS_ENABLED = $_[0];
}




sub ENV_beeps_are_enabled()
{
return $BEEPS_ENABLED;
}




sub ENV_beep()
{
$NR_BEEPS++;
}




sub ENV_reap_beeps()
{
exec_beep() if ($NR_BEEPS != 0);
$NR_BEEPS = 0;
}




sub exec_beep()
{
if ($BEEPS_ENABLED && $IS_INTERACTIVE)
{

OS_beep();
}
}




sub ENV_set_verbose(;$)
{
my ($new_verbose) = @_;	    # bool. Just return cur value if undef.
my $old_verbose = $VERBOSE;

if (defined $new_verbose)
{
$VERBOSE = $new_verbose;
ENV_setenv( "${APPLICATION_NAME}DEBUG_VERBOSE" => $VERBOSE);	# E.g.: GBSDEBUG_VERBOSE

}

return $old_verbose;
}




sub ENV_say($@)
{
my ($indent,    # 0 = no prefix, 1 = $prefix, 2 = $prefix_ - Negative = prefix_level - 1, 3+ = indent
@lines_and_or_refs
) = @_;

ENV_sig( F => "Invalid indent ($indent)")
if ($indent !~ /^-?\d+$/);

map { CORE::say( $_) } format_lines( $indent, deref_lines( 0, \@lines_and_or_refs));
}






sub ENV_say_stderr($@)
{
my ($indent,    # 0 = no prefix, 1 = $prefix, 2 = $prefix_ - Negative = prefix_level - 1
@lines_and_or_refs
) = @_;

ENV_sig( F => "Invalid indent ($indent)")
if ($indent !~ /^-?[012]$/);

map { CORE::say( STDERR $_) } format_lines( $indent, deref_lines( 0, \@lines_and_or_refs));
}




sub ENV_whisper($@)
{
my ($indent,    # 0 = no prefix, 1 = $prefix, 2 = $prefix_ - Negative = prefix_level - 1, 3+ = indent
@lines_and_or_refs
) = @_;

ENV_say( $indent, @lines_and_or_refs)
if ($VERBOSE);
}






sub ENV_trace(@)
{
my (@lines_and_or_refs) = @_;

my @lines = deref_lines( 0, \@lines_and_or_refs);
my ($prefix, $prefix_) = ENV_get_prefix( 0);
CORE::say( "$prefix: " . shift @lines);
map { CORE::say( "$prefix_ $_") } @lines;
}




sub ENV_debug($@)
{
my ($indent,    # 0 = no prefix, 1 = $prefix, 2 = $prefix_ - Negative = prefix_level - 1, 3+ = indent
@lines_and_or_refs
) = @_;
my $debug = 0;

if ($DEBUG_ACTIVE)
{
my $debug_name = get_debug_name();
if (defined $debug_name)
{
if ($indent <= 1)
{
ENV_say( $indent, "DBG: $debug_name", @lines_and_or_refs);
} else
{
ENV_say( $indent, @lines_and_or_refs);
}
$debug = 1;
}
}

return $debug;
}





sub ENV_print($@)
{
my ($indent,    # 0 = no prefix, 1 = $prefix, 2 = $prefix_ - Negative = prefix_level - 1
@lines_and_or_refs
) = @_;

ENV_sig( F => "Invalid indent ($indent)")
if ($indent !~ /^-?[012]$/);

map { print( $_) } format_lines( $indent, deref_lines( 0, \@lines_and_or_refs));
}





sub deref_lines($$)
{
my ($recurse_level,
$line_or_ref,
) = @_;
my @lines;

$recurse_level++;
my_die( "Max Recurse level exceeded ($recurse_level)")
if ($recurse_level > 20);

if (ref $line_or_ref)	# REF
{	if (ref $line_or_ref eq 'SCALAR')
{
push @lines, deref_lines( $recurse_level, $$line_or_ref);
} elsif (ref $line_or_ref eq "ARRAY")
{
push @lines, map { deref_lines( $recurse_level, $_) } @{$line_or_ref};
} elsif (ref $line_or_ref eq "HASH")
{
my_die( 'Say of HASH not implemented (yet)');
} else
{
my_die( 'Say of ' . ref $line_or_ref . ' not supported');
}
} else			# SCALAR
{
push @lines, $line_or_ref;
}

return @lines;
}



sub format_lines($@)
{
my ($indent,
@lines
) = @_;
my @formatted_lines;

ENV_get_term_size()
if (!defined $XTERM_WIDTH);

if ($indent == 0)
{
push @formatted_lines, ENV_split_lines( $XTERM_WIDTH, @lines);
} else
{
my $call_level = 1;
if ($indent < 0)
{
$call_level = 2;
$indent *= -1;
}
my ($prefix, $prefix_) = ENV_get_prefix( $call_level);
my $text_length = $XTERM_WIDTH - length( $prefix) - 1;
@lines = ENV_split_lines( $text_length, @lines);
if ($indent == 1)
{
my $first_line = shift @lines;
push @formatted_lines, "$prefix: $first_line";
}
if (@lines)
{
my $indents = '  ' x ($indent - 2);
push @formatted_lines, map { "$prefix_ $indents$_" } @lines;
}
}

return @formatted_lines;
}





sub ENV_get_prefix($)
{
my ($top_level) = @_;	    # > 0. 0 forces full prefix
my ($prefix, $prefix_);

if ($top_level == 0 || $DEBUG_ACTIVE && defined $DEBUG_NAMES{PREFIX})
{
my $prev_package = caller(0);	# Caller of ENV_print or ENV_say
my @packages;
my $at_end = 0;
for (my $i = 1; !$at_end; $i++)
{
my $this_package = caller( $i);
if (defined $this_package)
{
if ($this_package eq 'main')
{
unshift @packages, $MAIN_NAME;
$at_end = 1;
} else
{

if ($this_package ne $prev_package)
{
unshift @packages, $this_package;
$prev_package = $this_package;
}
}
} else
{
unshift @packages, "$MAIN_NAME---";	# BEGIN- or END-block
$at_end = 1;
}
}

if ($top_level > 0)
{
for (my $i=1; $i < $top_level; $i++)
{
pop @packages;
}
}

foreach my $package (@packages)
{
$package =~ s/.*:://;
}
$prefix = uc join( '/', @packages);

$prefix_ = $MAIN_NAME_;

} else
{
($prefix, $prefix_) = ($MAIN_NAME, $MAIN_NAME_);

}

return (wantarray) ? ($prefix, $prefix_) : $prefix;
}






sub ENV_sig($@)
{
my ($sig,		# <severity><action> IWEF  EC. May be undef or ''
@lines_or_refs
) = @_;

if (defined $sig && $sig ne '')
{
my @lines = map { "    $_" } map { (ref $_) ? @{$_} : $_ } @lines_or_refs;
handle_sig( undef, $sig, \@lines);
}
}









sub ENV_file_sig($$@)
{
my ($sig,		# <severity><action> IWEF  XC
$file_info_ref,	# [ $prefix, $src, $file, $line_nr, $line, $line2 ]
@lines_or_refs
) = @_;

my ($prefix, $src, $file, $line_nr, $line, $line2) = @{$file_info_ref};

my @lines;
if ($src)
{
if (defined $file && $file ne $src)
{
my $file_part = (defined $line_nr) ? "$file ($line_nr)" : $file;
push @lines, "  File: $src";
push @lines, "  Incl: $file_part";
} else
{
my $src_part = (defined $line_nr) ? "$src ($line_nr)" : $src;
push @lines, "  File: $src_part";
}
}

if ($line)
{
if (defined $line_nr)
{
push @lines, sprintf( "%6d: %s", $line_nr, $line);
push @lines, sprintf( "%6s: %s", ' ',      $line2)
if (defined $line2);
} else
{
push @lines, "  Line: $line";
push @lines, "      : $line2"
if (defined $line2);
}
}
push @lines, map { "  - $_" } map { (ref $_) ? @{$_} : $_ } @lines_or_refs;

handle_sig( $prefix, $sig, \@lines);
}




sub ENV_try($$)
{
my ($sig_on_error,
$func,
) = @_;
my @result;



my $saved_nr_errors = $NR_ERRORS;
if (wantarray)
{
@result = eval { $func->() };
} else
{
$result[0] = eval { $func->() };
}
my $nr_errors_suppressed = $NR_ERRORS - $saved_nr_errors;
$NR_ERRORS = $saved_nr_errors;

if (defined $sig_on_error && $nr_errors_suppressed > 0)
{
ENV_sig( $sig_on_error => "$nr_errors_suppressed Errors suppresssed");
CORE::say( "---\n$@---")
if ($@ ne '');
}

return (wantarray) ? @result : $result[0];
}






sub handle_sig($$$)
{
my ($prefix,
$sig,		# <severity><action> IWEF  EC
$lines_ref,
) = @_;

my ($severity, $action) = split( '', $sig);

$action = ($severity eq 'F') ? 'E' : 'C'
if ($action eq '');
my $sig_text = $SIG_TEXT{$severity};
my $sev = index( 'IWEF', $severity);
$prefix = ENV_get_prefix( 1)
if (!defined $prefix);




{
my $err;
$err = "Invalid Signal Severity '$severity' ($sig)"
if (!defined $sig_text);
$err = "Invalid Signal Action '$action' ($sig)"
if (index( 'EC', $action) < 0);
if (defined $err)
{
CORE::say( "$prefix: ** FATAL**\n    $err");
map { CORE::say( $_) } @{$lines_ref};
ENV_stackdump();
exit 9;
}
}




CORE::say( "$prefix: ** $sig_text **");
map { CORE::say( $_) } @{$lines_ref};




if ($sev > 0)	# WEF
{
$NR_BEEPS++;
if ($sev == 1)	# W
{
$NR_WARNINGS++;
} else		# EF
{
$NR_ERRORS++;
}
}





if ($action eq 'E')	    # Exit
{
my $exit_code = substr( '0129', $sev, 1);
if (defined $^S && $^S == 1)
{



die;
} else
{
if ($severity eq 'F')
{
ENV_stackdump();
exit $exit_code;	    # 9
} else
{
ENV_exit( $exit_code);
}
}
} else  # $action == 'C'
{
ENV_stackdump()
if ($severity eq 'F');
}
}




sub ENV_exit(;$)
{
my ($rc) = @_;

$rc = 0 if (!defined $rc);
$rc = 2 if ($rc < 2 && $NR_ERRORS > 0);
exit $rc;
}




sub ENV_get_nr_errors()
{
return $NR_ERRORS;
}




sub ENV_stackdump()
{
my @call_info = caller(1);
CORE::say( "** at: @call_info[1..2]");
for (my $i = 2; @call_info; $i++)
{
@call_info = caller( $i);
if (@call_info)
{
CORE::say( "** called by: @call_info[1..3]");
}
}
}




sub ENV_is_debug(;$)
{
my ($debug_names_ref,	# optional. Default is caller's package name
) = @_;


if ($DEBUG_ACTIVE)
{
return (defined get_debug_name()) ? 1 : 0;
} else
{
return 0;
}
}




sub get_debug_name()
{
my $debug_name;		    # undef if debug not active for this context

if ($DEBUG_ALL)
{
$debug_name = 'ALL';
} else
{
my ($caller_package, undef) = caller( 1);
if ($caller_package eq 'main')
{
$debug_name = $MAIN_NAME;
} else
{
my @caller_package = split( '::', $caller_package);
$debug_name = uc $caller_package[-1];
}
if (!exists $DEBUG_NAMES{$debug_name})
{
$debug_name = undef;
}
}


return $debug_name;
}




sub ENV_print_end_msg($)
{
my ($always) = @_;		# Bool

if (!$RUNNING_IN_EXEC)
{
my $rc = $?;

$rc = 1 if ($rc == 0 && $NR_ERRORS > 0);
$always = 1 if ($NR_WARNINGS > 0);
ENV_beep() if ($rc != 0);

if ($rc != 0 || $always)
{
my $text = "$MAIN_NAME: ";

if ($rc == 0)
{
$text .= 'Done';
} else
{
$text .= "Failed (rc=$rc)";
}
if ($NR_ERRORS > 0 || $NR_WARNINGS > 0)
{
$text .= ' with ';
my @texts;
push @texts, "$NR_ERRORS error(s)" if ($NR_ERRORS > 0);
push @texts, "$NR_WARNINGS warning(s)" if ($NR_WARNINGS > 0);
$text .= join( ' and ', @texts);
}

ENV_reap_beeps();
ENV_say( 0, $text);
}
}
}




sub ENV_printable($)
{
my ($var,
) = @_;
my $text;

if (!defined $var)
{
$text = 'undef';
} else
{
my $reftype = ref $var;
if (!defined $reftype)
{
$text = "\"$var\"";
} elsif ($reftype eq 'ARRAY')
{
my @items = @{$var};
foreach my $item (@items)
{
if (!defined $item)
{
$item = 'undef';
} elsif (ref $item)
{
$item = 'REF ' . ref $item;
} else
{
$item = $var;
}
}
$text = '[' . join( ',', @items) . ']';
} elsif ($reftype eq 'HASH')
{
my @keys = keys %{$var};
$text = '{' . join( ',', @keys) . '}';
} else
{
$text = $reftype;
}
}
return $text;
}




sub ENV_clone($)
{
my ($this) = @_;

if (not ref $this)
{
$this;
} elsif (ref $this eq 'ARRAY')
{
[ map ENV_clone($_), @$this ];
} elsif (ref $this eq 'HASH')
{
scalar { map { $_ => ENV_clone($this->{$_}) } keys %$this };
} else
{
ENV_sig( F => "clone: What type is $this?");
}
}




sub ENV_dump($$;$$)
{
my ($name,
$item,		# undef, scalar or ref
$max_level,
$unsorted_hash
) = @_;

print( "---$name\n");
$max_level = 100
if (!defined $max_level);
dump_item( 0, $max_level, $item, 1, $unsorted_hash);
print( "---\n");
}




sub dump_item($$$$$)
{
my ($level,
$max_level,
$item,
$must_indent,
$unsorted_hash
) = @_;

my $indent = '  ' x $level;
$level++;
ENV_sig( F => "maxlevel reached") if ($level > 1000);

print( $indent)
if ($must_indent);
if ($level > $max_level)
{
print( "***\n");
} else
{
if (defined $item)
{

if (ref $item)
{
if (ref $item eq 'SCALAR')
{
print( "->");
dump_item( $level, $max_level, $$item, 1, $unsorted_hash);

} elsif (ref $item eq "ARRAY")
{
if ( @{$item})
{
print( "->[\n");
foreach my $value (@{$item})
{
dump_item( $level + 1, $max_level, $value, 1, $unsorted_hash);
}
print( $indent . "  ]\n");
} else
{
print( "->[]\n");
}
} elsif (ref $item eq "HASH")
{
if (keys %{$item})
{
print( "->{\n");
if ($unsorted_hash)
{
while (my ($key, $value) = each %{$item})
{
print( $indent . "    {$key}");
dump_item( $level + 1, $max_level, $value, 0, $unsorted_hash);
}
} else
{
foreach my $key (sort(keys %{$item}))
{
print( $indent . "    {$key}");
dump_item( $level + 1, $max_level, $item->{$key}, 0, $unsorted_hash);
}
}
print( $indent . "  }\n");
} else
{
print( "->{}\n");
}
} else
{
print( "->" . ref( $item) . "\n");
}
} else
{
print( "=") if (!$must_indent);
print( "'$item'\n");
}
} else
{
print( "=") if (!$must_indent);
print( "undef\n");
}
}
$level--;
}






sub ENV_eval_empty($)
{

return (defined $_[0] && $_[0] ne '""' && $_[0] ne "''") ? $_[0] : '';
}






sub ENV_eval_undef($)
{

return (defined $_[0] && $_[0] ne '' && $_[0] ne '""' && $_[0] ne "''") ? $_[0] : undef;
}






sub ENV_undef2empty(@)
{



return (wantarray) ? map { (defined $_) ? $_ : '' } @_ : (defined $_[0]) ? $_[0] : '';
}







sub ENV_getenv($)
{

my $env = ENV_eval_empty( $ENV{$_[0]});

return (wantarray) ? split( ',', $env) : $env;
}







sub ENV_getenv_bool($)
{

my $env = ENV_eval_empty( $ENV{$_[0]});

return ($env =~ /^(1|Y|YES|ON|TRUE$)/i) ? 1 : 0;
}







sub ENV_getenv_perl_path($)
{
my ($name) = @_;
my $env;

$env = ENV_eval_empty( $ENV{$name});
if ($env ne '')
{

$env = ENV_perl_path_nospace( $env)
if ($IS_WIN32);

}

return $env;
}





sub ENV_setenv($$)
{
my ($name,
$value_or_ref
) = @_;
my $value;

if (!exists $CENVS{$name})
{
$CENVS{$name} = $ENV{$name};
}
if (ref $value_or_ref)
{
$value = join ',', @{$value_or_ref};
} else
{
$value = $value_or_ref;
}




ENV_sig( F => "ENV_setenv: Invalid item $name = $value")
if (!defined $name); # || !defined $value);

if (defined $value)
{
$value = '""' if ($value eq '');
$ENV{$name} = $value;
$value = '' if ($value eq '""');
} else
{
$ENV{$name} = $value;
}

return $value;
}






sub ENV_setenv_os_path($$)
{
my ($name,
$value_or_ref
) = @_;
my $value = (ref $value_or_ref) ? join( ',', @{$value_or_ref}) : $value_or_ref;

$value =~ tr!/!\\!
if ($IS_WIN32);

return ENV_setenv( $name => $value);
}




sub ENV_get_changed_envs()
{
my @changed_envs;

foreach my $name (keys %CENVS)
{
my $old_value = $CENVS{$name};
my $cur_value = $ENV{$name};
if (defined $old_value)
{
push @changed_envs, $name if (!defined $cur_value || $cur_value ne $old_value);
} else
{
push @changed_envs, $name;
}
}
@changed_envs = sort @changed_envs;

return @changed_envs;
}





sub ENV_command_filespec($)
{
my ($spec) = @_;

$spec .= ENV_shell_filetype();
$spec =~ tr!/!\\!
if ($IS_WIN32);

return $spec;	# os format
}






sub ENV_shell_filetype()
{
return (wantarray) ? @SHELL_FILETYPES : $SHELL_FILETYPES[0];
}




sub ENV_is_shell_file($)
{
my ($file) = @_;	    # file or filespec

my ($type) = $file =~ /(\.*)/;		# greedy!

return ($type eq '.bat' || $type eq '.sh') ? 1 : 0;
}






sub ENV_chmod($@)
{
my ($symbolic,	# ugoa +-= rwx e.g.:  'ugo-wx'
@files
) = @_;
my $count = 0;

my %bits = ( u => 0, g => 1, o => 2,
r => 4, w => 2, x => 1 );

my ($who, $op, $perms) = $symbolic =~ /^([ugoa]+)([+-=])([rwx]+)$/;
ENV_sig( F => "chmod: Invalid symbolic '$symbolic'") if (!defined $who);
$who = 'ugo' if ($who =~ /a/);

my @ugos;
map { push @ugos, $bits{$_} } split //, $who;

my $value = 0;
map { $value |= $bits{$_} } split //, $perms;

foreach my $file (@files)
{
my $org_mode = (stat $file)[2];
if (!defined $org_mode)
{
ENV_sig( E => "chmod $symbolic: No such file/dir: $file");
next;
}
my @values = (($org_mode & oct(700)) >> 6, ($org_mode & oct(70)) >> 3, $org_mode & oct(7));

if ($op eq '+')
{
map { $values[$_] |=  $value } @ugos;
} elsif ($op eq '-')
{
map { $values[$_] &= ~$value } @ugos;
} else	# ($op eq '=')
{
map { $values[$_]  = $value } @ugos;
}
my $mode = ($values[0] << 6) | ($values[1] << 3) | $values[2];
my $new_mode = $org_mode & ~oct(777) | $mode;
if ($new_mode != $org_mode)
{

$count += chmod $new_mode, $file;
} else
{
$count++;
}
}

return $count;
}





sub ENV_get_hostname()
{
return OS_get_hostname();
}





sub ENV_uname()
{
return OS_get_uname();
}





sub ENV_unique_id()
{
my $unique_id;

my $hostname = ENV_get_hostname();
my $time = time;

$unique_id = "${hostname}_${time}_${$}_${UNIQUE_COUNT}";
$UNIQUE_COUNT++;


return $unique_id;
}





sub ENV_get_userid()
{
return OS_get_userid();
}





sub ENV_is_wildcard($)
{

return ($_[0] =~ /[*?]/) ? 1 : 0;
}





sub ENV_glob(@)
{

my @specs = map { glob $_ } @_;

return (wantarray) ? @specs : $specs[0];	    # undef if scalar context and empty
}





sub ENV_glob_files(@)
{

my @files;

@files = map { $_ =~ m!.*/(.*)! } map { glob $_ } @_;

return (wantarray) ? @files : $files[0];
}




sub ENV_wild_2_re($)
{
my ($wildcard) = @_;
my $re;

$re = quotemeta $wildcard;
$re =~ s/\\\*/.*/g;
$re =~ s/\\\?/./g;
$re = "^$re\$";


return $re;
}









sub ENV_wildcard($$)
{
my ($find,		# this is the wildcard
$list_ref,	# single or ref-list
) = @_;
my @found_items;

$list_ref = [$list_ref] unless( ref $list_ref);

if ($find eq '*')
{
@found_items = @{$list_ref};
} else
{
my $find_re = ENV_wild_2_re( $find);
@found_items = grep( /$find_re/, @{$list_ref});
}


return (wantarray) ? @found_items : $found_items[0];
}








sub ENV_wildcards($$$)
{
my ($finds_ref,	    # this is the wildcards list
$list_ref,	    # single or ref-list
$no_matches_ref,    # may be undef
) = @_;
my @found_items;

$finds_ref = [$finds_ref] unless( ref $finds_ref);
$list_ref = [$list_ref] unless( ref $list_ref);

my %dups;


foreach my $find (@{$finds_ref})
{
my $find_re = ENV_wild_2_re( $find);
my @these_found_items = grep( /$find_re/, @{$list_ref});
if (@these_found_items)
{
foreach my $found_item (@these_found_items)
{
if (!defined $dups{$found_item})
{
push @found_items, $found_item;
$dups{$found_item} = 1;
}
}
} else
{
push @{$no_matches_ref}, $find
if (defined $no_matches_ref);
}
}


return @found_items;
}








sub ENV_not_wildcards($$)
{
my ($finds_ref,	# this are the wildcards
$list_ref,	# single or ref
) = @_;
my @found_items;

$list_ref = [$list_ref] unless( ref $list_ref);
if (ref $finds_ref)
{
my @find_res = map { ENV_wild_2_re( $_) } @{$finds_ref};
foreach my $item (@{$list_ref})
{
my $found = 0;
foreach my $find_re (@find_res)
{
if ($item =~ $find_re)
{
$found = 1;
last;
}
}
push @found_items, $item
if (!$found);
}
} else
{
if ($finds_ref eq '*')
{
@found_items = ();
} else
{
my $find_re = ENV_wild_2_re( $finds_ref);
@found_items = grep( !/$find_re/, @$list_ref);
}
}

return @found_items;
}








sub ENV_get_term_size()
{
if (!defined $XTERM_WIDTH)
{
if (-t STDOUT)
{
($XTERM_WIDTH, $XTERM_HEIGHT, my $width_px, my $height_px) = GetTerminalSize();

}
$XTERM_WIDTH = $XTERM_DEFAULT_WIDTH
if (!defined $XTERM_WIDTH);
$XTERM_WIDTH--;    				# half characters may cause empty lines
$XTERM_HEIGHT = $XTERM_DEFAULT_HEIGHT
if (!defined $XTERM_HEIGHT);

}

return (wantarray) ? ($XTERM_WIDTH, $XTERM_HEIGHT) : $XTERM_WIDTH;
}







sub ENV_get_term_size_default()
{
return (wantarray) ? ($XTERM_DEFAULT_WIDTH, $XTERM_DEFAULT_HEIGHT) : $XTERM_DEFAULT_WIDTH;
}




sub ENV_set_background_term_size()
{
ENV_get_term_size();

$XTERM_WIDTH = $XTERM_BACKGROUND_WIDTH;

return (wantarray) ? ($XTERM_WIDTH, $XTERM_HEIGHT) : $XTERM_WIDTH;
}




sub ENV_is_abs_path($)
{
my ($path) = @_;



if ($IS_WIN32)
{
return $path =~ m!^"?([A-Za-z]:(/|\\)|//.+|\\\\.+)!;
} else
{
return $path =~ m!^"?(/|~/)!;
}
}




sub ENV_is_abs_path_perl($)
{
my ($path) = @_;



if ($IS_WIN32)
{
return $path =~ m!^(//.+|[A-Za-z]:/)!;
} else
{
return $path =~ m!^(/|~/)!;
}
}




sub ENV_is_option($)
{
my ($spec) = @_;

my $fc = substr( $spec, 0, 1);
if ($IS_WIN32)
{
return ($fc eq '-' || $fc eq '/');
} else
{
return ($fc eq '-');
}
}




sub ENV_is_in_path($$)
{
my ($path,			# perl path
$starts_with_path,	# perl path
) = @_;

$starts_with_path = quotemeta $starts_with_path;

return ( $path =~ m!^$starts_with_path($|/)! ) ? 1 : 0;
}




sub ENV_parent_path($)
{
my ($path) = @_;
my $parent_path;

$parent_path = $path;
$parent_path =~ s!/[^/]+$!!;
$parent_path = ''
if ($parent_path eq $path);


return $parent_path;
}




sub ENV_parent_dir($$)
{
my ($path,
$index	    # -1 == last, 0 == first
) = @_;
my $parent_name;

my @dirs = split( '/', $path);
$parent_name = $dirs[$index];
$parent_name = '' if (!defined $parent_name);


return $parent_name;
}




sub ENV_set_tmpdir_path($)
{
my ($tmpdir_path) = @_;	    # may be undef

$TMPDIR_PATH = $tmpdir_path;
if (defined $tmpdir_path)
{
ENV_setenv( ENV_TMPDIR_PATH => $tmpdir_path);		    # used only by this package
} else
{
ENV_setenv( ENV_TMPDIR_PATH => '');			    # used only by this package
}
}




sub ENV_get_tmpdir_path()
{
if (!defined $TMPDIR_PATH)
{
my $tmpdir_path = ENV_getenv_perl_path( 'ENV_TMPDIR_PATH'); # used only by this package
$tmpdir_path = ENV_get_os_tmpdir_path()
if ($tmpdir_path eq '');
$TMPDIR_PATH = $tmpdir_path;

}

return $TMPDIR_PATH;
}




sub ENV_get_os_tmpdir_path()
{
if (!defined $OS_TMPDIR_PATH)
{
my @tmp_envs = ($IS_WIN32) ? qw(TEMP TMP TMPDIR) : qw(TMPDIR);
my $tmpdir_path;
foreach my $tmp_env (@tmp_envs)
{
my $test_path = ENV_getenv_perl_path( $tmp_env);
if (-e $test_path)
{
$tmpdir_path = $test_path;
last;
}
}
if (!defined $tmpdir_path)
{
if ($IS_LUNIX)
{
$tmpdir_path = ENV_glob( '~/tmp');
$tmpdir_path = '/tmp'
if (! -e $tmpdir_path);
} else
{
ENV_sig( EE => "No valid Temp. directory found in (@tmp_envs)");
}
}
$OS_TMPDIR_PATH = $tmpdir_path;
}

return $OS_TMPDIR_PATH;
}




sub ENV_get_tmp_spec($)
{
my ($file) = @_;
my $spec;

my $temp_path = ENV_get_tmpdir_path();
my $dotloc = rindex( $file, '.');
my $name;
my $type;
if ($dotloc == 0)	    # starts with '.'
{
$name = "";
$type = $file;
} elsif ($dotloc > 0)   # has '.'
{
$name = substr( $file, 0, $dotloc);
$type = substr( $file, $dotloc);
} else  	    	    # no '.'
{
$name = $file;
$type = '.tmp';
}
my $unique_id = ENV_unique_id();
$spec = "$temp_path/${name}_${unique_id}$type";


return $spec;
}




sub ENV_get_tmp_path($)
{
my ($dir) = @_;
my $path;

my $temp_path = ENV_get_tmpdir_path();
my $unique_id = ENV_unique_id();
$path = "$temp_path/${dir}_${unique_id}";


return $path;
}




sub ENV_os_path(@)
{
my @paths = @_;	# Perl path

if ($IS_WIN32)
{
foreach my $path (@paths)
{
$path =~ tr!/!\\!;	    #/ os path
}
}

return (wantarray) ? @paths : $paths[0];
}




sub ENV_perl_paths(@)
{
my @paths = @_;


foreach my $path (@paths)
{
$path =~ tr!\\!/!;	    #/ perl path
}

return (wantarray) ? @paths : $paths[0];
}





sub ENV_perl_canon_paths(@)
{
my @paths = @_;


foreach my $path (@paths)
{
$path =~ tr!\\!/!;	    #/ perl path
$path = ENV_canonicalize_path( $path);
}

return (wantarray) ? @paths : $paths[0];
}







sub ENV_perl_path_nospace(@)
{
my @paths = @_;
my @perl_paths;

foreach my $path (@paths)
{
if (substr( $path, 0, 1) eq '"' || index( $path, ' ') >= 0)
{
ENV_sig( F => "DBG:  Path $path contains '$PATH_SEPARATOR'")    # ';' or ':'
if ($path =~ /$PATH_SEPARATOR/);




$path =~ s/"//g;
my $perl_path = $QUOTED_PATHS{$path};
if (defined $perl_path)
{
push @perl_paths, $perl_path;
} else
{

if ($IS_WIN32)
{
$perl_path = OS_short_path_name( $path);
$perl_path =~ tr!\\!/!;	    # Perl path
} else
{
$perl_path = $path;
}
$perl_path = ENV_canonicalize_path( $perl_path);
$QUOTED_PATHS{$path} = $perl_path;
push @perl_paths, $perl_path;
}
} else
{



my $perl_path;
if ($IS_WIN32)
{

$perl_path = $path
if (!defined $perl_path);
$perl_path =~ tr!\\!/!;	    # perl path
} else
{
$perl_path = $path;
}
$perl_path = ENV_canonicalize_path( $perl_path);
push @perl_paths, $perl_path;
}
}

return (wantarray) ? @perl_paths : $perl_paths[0];
}





sub ENV_perl_path_space(@)
{
my @paths = @_;
my @perl_paths;

foreach my $path (@paths)
{
push @perl_paths, OS_long_path_name( $path);
}

return (wantarray) ? @perl_paths : $perl_paths[0];
}





sub ENV_canonicalize_path($)
{
my ($filespec) = @_;	# Perl path!






my $device;
my $dir_file;
if ($filespec =~ m!^(/+)(.*)!)
{



$device = $1;
$dir_file = $2;
} elsif ($filespec =~ m!^([a-zA-Z]:/?)(.*)!)
{



$device = ucfirst $1;
$dir_file = $2;
} else
{



$device = '';
$dir_file = $filespec;
}



$dir_file =~ s!/+!/!g;				    # xx////yy	 -> xx/yy
$dir_file =~ s!(/\.)+/!/!g; 			    # xx/././yy  -> xx/yy
while ($dir_file =~ s!/[^/]+/\.\.(/|$)!/!) {};	    # /xx/yy/../ -> /xx
$dir_file =~ s!/\.$!!;				    # xx/.	 -> xx
$dir_file =~ s!/$!!;				    # xx/	 -> xx

$filespec = "$device$dir_file";


return $filespec;
}





sub ENV_paths_equal($$)
{
my ($l_path,
$r_path,
) = @_;


my ($long_l_path, $long_r_path) = ENV_perl_path_space( $l_path, $r_path);



return ($long_l_path eq $long_r_path) ? 1 : 0;
}






sub ENV_expand_envs($)
{
my ($line) = @_;





$line =~ s/\$(\w+)/$ENV{$1} ? $ENV{$1} : "\$$1"/eg;





$line =~ s/\$\{([^\}]+)\}/$ENV{$1} ? $ENV{$1} : "\${$1}"/eg;


if ($IS_WIN32)
{



$line =~ s/\%(\w+)\%/$ENV{$1} ? $ENV{$1} : "%$1%"/eg;
} else
{




$line = ENV_glob( $line)
if (substr( $line, 0, 1) eq '~');
}


return $line;
}





sub ENV_expand_arg($)
{
my ($arg) = @_;
my $args;

$args = "\\\$$arg\(\\W|\$)|\\\$\\\{$arg\\\}";
$args .= "|\\\%$arg\\\%"
if ($IS_WIN32);

return qr($args);
}





sub ENV_delink_spec($)
{
my ($spec) = @_;
my $new_spec;


$new_spec = $spec;
if ($IS_LUNIX)
{
my $link_replaced;
my $count = 0;
my @spec = split '/', $spec;
do
{
$count++;
ENV_sig( EE => "More than $count nested links in '$spec'") if ($count > 10);
$link_replaced = 0;
my @new_spec;
while (@spec)
{
my $part = shift @spec;

my $part_spec = join( '/', @new_spec, $part);

if (-l $part_spec)
{
my $link = readlink $part_spec;
if (substr( $link, 0, 1) eq '/')
{
@new_spec = ($link);
} else
{
push @new_spec, $link;
}
$link_replaced = 1;
} else
{
push @new_spec, $part
}
}
if ($link_replaced)
{
$new_spec = join( '/', @new_spec);

$new_spec = ENV_canonicalize_path( $new_spec);

@spec = split '/', $new_spec;
}
} while ($link_replaced);
}

return $new_spec;
}




sub ENV_rel_spec($$)
{
my ($from_path,   # Absolute
$to_file     # Absolute
) = @_;
my $result;



my @from_path = split '/', $from_path;
my @to_file = split '/', $to_file;
my $org_nr_from = @from_path;
$org_nr_from -= 1 if (substr( $from_path, 0, 1) eq '/');

if ($from_path[0] eq $to_file[0])
{
while (@from_path && ($from_path[0] eq $to_file[0]))
{
shift @from_path;
shift @to_file;
}
my $nr_from = scalar @from_path;

if ($nr_from < $org_nr_from)
{
$result = '../' x $nr_from . join( '/', @to_file);
} else
{
$result = $to_file;
}
} else
{
$result = $to_file;
}


return $result;
}




sub ENV_abs_path($$)
{
my ($cwd,
$paths_ref	    # ref or scalar
) = @_;
my @paths;

$paths_ref = [ $paths_ref ] if (!ref $paths_ref);

$cwd = ENV_cwd() if (!defined $cwd || $cwd eq '.');


foreach my $path (@{$paths_ref})
{

if (ENV_is_abs_path_perl( $path))
{
push @paths, $path;
} else
{
push @paths, ENV_canonicalize_path( "$cwd/$path");
}
}


(wantarray) ? return @paths : $paths[0];
}




sub ENV_rel_path($$)
{
my ($cwd,
$paths_ref	    # ref or scalar
) = @_;
my @paths;

$paths_ref = [ $paths_ref ] if (!ref $paths_ref);

$cwd = ENV_cwd() if (!$cwd || $cwd eq '.');


foreach my $path (@{$paths_ref})
{

if (ENV_is_abs_path_perl( $path))
{
push @paths, ENV_rel_spec( $cwd, $path);
} else
{
push @paths, $path;
}
}


(wantarray) ? return @paths : $paths[0];
}




sub ENV_shortest_path($$)
{
my ($cwd,
$paths_ref	    # ref or scalar
) = @_;
my @paths;

$paths_ref = [ $paths_ref ] if (!ref $paths_ref);

$cwd = ENV_cwd() if (!$cwd || $cwd eq '.');


foreach my $path (@{$paths_ref})
{

my $abs_path;
my $rel_path;
if (ENV_is_abs_path_perl( $path))
{
$abs_path = $path;
$rel_path = ENV_rel_spec( $cwd, $path);
} else
{
$abs_path = ENV_canonicalize_path( "$cwd/$path");
$rel_path = $path;
}
if (length( $abs_path) < length( $rel_path))
{
push @paths, $abs_path;
} else
{
push @paths, $rel_path;
}
}


(wantarray) ? return @paths : $paths[0];
}







sub ENV_short_path_name(@)
{
my (@paths,
) = @_;
my @short_os_paths;

foreach my $path (@paths)
{
push @short_os_paths, OS_short_path_name( $path);
}

return (wantarray) ? @short_os_paths : $short_os_paths[0];
}






sub ENV_split_spec_dnt($)
{
my ($filespec) = @_;
my ($dir_path, $name, $type);

my $i = rindex( $filespec, '/');
my $file;
if ($i >= 0)
{
$dir_path = substr( $filespec, 0, $i);
$file = substr( $filespec, $i + 1);
} else
{
$dir_path = '';
$file = $filespec;
}

$i = rindex ($file, '.');
if ($i >= 0)
{
$name = substr( $file, 0, $i);
$type = substr( $file, $i);
} else
{
$name = $file;
$type = '';
}


return ($dir_path, $name, $type);
}






sub ENV_split_spec_df($)
{
my ($filespec) = @_;
my ($dir_path, $file);

my $i = rindex( $filespec, '/');
if ($i >= 0)
{
$dir_path = substr( $filespec, 0, $i);
$file = substr( $filespec, $i + 1);
} else
{
$dir_path = '';
$file = $filespec;
}


return ($dir_path, $file);
}






sub ENV_split_spec_d($)
{
my ($filespec) = @_;
my $dir_path = '';

my $i = rindex( $filespec, '/');
$dir_path = substr( $filespec, 0, $i) if ($i >= 0);


return $dir_path;
}






sub ENV_split_spec_f($)
{
my ($filespec) = @_;
my $file;

my $i = rindex( $filespec, '/');
if ($i >= 0)
{
$file = substr( $filespec, $i + 1) ;
} else
{
$file = $filespec;
}


return $file;
}






sub ENV_split_spec_nt($)
{
my ($filespec) = @_;
my ($name, $type);

my $i = rindex( $filespec, '/');
$filespec = substr( $filespec, $i + 1) if ($i >= 0);
$i = rindex( $filespec, '.');
if ($i >= 0)
{
$name = substr( $filespec, 0, $i);
$type = substr( $filespec, $i);
} else
{
$name = $filespec;
$type = '';
}

return ($name, $type);
}






sub ENV_split_spec_n($)
{
my ($filespec) = @_;
my $name;

my $i = rindex( $filespec, '/');
$filespec = substr( $filespec, $i + 1) if ($i >= 0);

$i = rindex( $filespec, '.');
if ($i >= 0)
{
$name = substr( $filespec, 0, $i);
} else
{
$name = $filespec;
}
return $name;
}






sub ENV_split_spec_t($)
{
my ($filespec) = @_;
my $type = '';

my $i = rindex( $filespec, '/');
$filespec = substr( $filespec, $i + 1) if ($i >= 0);

$i = rindex( $filespec, '.');
$type = substr( $filespec, $i) if ($i >= 0);

return $type;
}





sub ENV_split_path($)
{
my ($path) = @_;


return split( $PATH_SEPARATOR, $path);
}





sub ENV_join_path($)
{
my ($path_ref) = @_;


return join( $PATH_SEPARATOR, @{$path_ref});
}





sub ENV_chdir($)
{
my ($path) = @_;

chdir( $path) || ENV_sig( F => "Unable to chdir( $path)", "- $!");
$ENV{PWD} = $path;

return $path;
}





sub ENV_cwd()
{
my $pwd = OS_get_cwd();

$ENV{PWD} = $pwd;

return $pwd;
}

{
my @PUSHD_STACK;




sub ENV_pushd($)
{
my ($path) = @_;	    # if undef: do not cddir to $path

push @PUSHD_STACK, OS_get_cwd();

if (defined $path)
{
chdir( $path) || ENV_sig( F => "Unable to chdir( $path) in pushd", "- $!");
$ENV{PWD} = $path;
}

return $path;
}





sub ENV_popd()
{
ENV_sig( F => 'PUSHD-stack exhaused')
if (@PUSHD_STACK == 0);

my $path = pop @PUSHD_STACK;

chdir( $path) || ENV_sig( F => "Unable to chdir( $path) in popd", "- $!");
$ENV{PWD} = $path;

return $path;
}
}







sub ENV_file_is_newer($$)
{
my ($file1,
$file2,
) = @_;
my $is_newer;

no integer;
my $dmtp1 = -M $file1;	    # date modified to present
if (defined $dmtp1)
{
my $dmtp2 = -M $file2;


if (defined $dmtp2)
{
$is_newer = ($dmtp1 < $dmtp2) ? 1 : 0;		# produce printable result
} else
{
$is_newer = 1;
}
} else
{
$is_newer = 0;
}
use integer;

return $is_newer;
}






sub ENV_file_is_same($$)
{
my ($file1,
$file2,
) = @_;
my $is_same = 0;

no integer;
my $dmtp1 = -M $file1;	    # date modified to present
if (defined $dmtp1)
{
my $size1 = -s _;
my $dmtp2 = -M $file2;

if (defined $dmtp2)
{
my $size2 = -s _;

$is_same = ($dmtp1 == $dmtp2 && $size1 == $size2) ? 1 : 0;	# produce prinatble result
}
}
use integer;

return $is_same;
}





sub ENV_mkdir($;$$)
{
my ($path,
$mask,		    # may be undef
$sig_on_error,	    # opt. '', 'EE', etc. Default == 'F'
) = @_;
my $created = 0;

$mask = oct(777)
if (!$mask);	    # ugo=rwx

if (!mkdir( $path, $mask))
{




my $rt = $!;
if (!-d $path)
{
$sig_on_error = 'F'
if (!defined $sig_on_error);
my $omask = sprintf( "0%o", $mask);
ENV_sig( $sig_on_error => "Unable to mkdir '$path' ($omask)", "- $rt");
}
}

if (-d $path)
{
$created = 1;
} else
{
$sig_on_error = 'F'
if (!defined $sig_on_error);
ENV_sig( $sig_on_error => "Failed to mkdir '$path' for unclear reason");
}


return $created;
}






sub ENV_mkpath($;$$)
{
my ($path,
$mode,		    # undef = OK
$sig_on_error,	    # opt. '', 'EE', etc. Default == 'F'
) = @_;
my $created = 1;

if (!-d $path)
{
my ($parent) = $path =~ m!(.*)/!;
if (defined $parent && ! -d $parent)
{
$created = ENV_mkpath( $parent, $mode, $sig_on_error);
}
$created = ENV_mkdir( $path, $mode, $sig_on_error)
if ($created);
}


return $created;
}




sub ENV_rmdir($;$$)
{
my ($path,
$force,		    # opt. bool
$sig_on_error,	    # opt. '', 'EE', etc. Default == 'F'
) = @_;
my $deleted = 0;

$deleted = rmdir $path;
if (!$deleted)
{
if (defined $force && $force)
{
chmod oct(777), $path;	# ugo+rwx
$deleted = rmdir $path;
}
if (!$deleted)
{
$sig_on_error = 'F'
if (!defined $sig_on_error);
ENV_sig( $sig_on_error => "Unable to rmdir( $path)",
"- $!");
}
}


return $deleted;
}






sub ENV_rmtree($$;$$)
{
my ($top_path,	    # must exist
$also_remove_top,   # bool
$force,		    # opt. bool
$sig_on_error,	    # opt. '', 'EE', etc. Default == 'F'
) = @_;
my $deleted = 1;

chmod oct(777), $top_path;	# ugo+rwx
foreach my $dir_entry (ENV_readdir( $top_path, $sig_on_error))
{
my $entry_spec = "$top_path/$dir_entry";
if (-d $entry_spec)
{
if (-l $entry_spec)    # do not follow symbolic links
{
$deleted = ENV_rmdir( $entry_spec, $force, $sig_on_error)
} else
{
$deleted = ENV_rmtree( $entry_spec, 1, $force, $sig_on_error);
}
} else	# -f
{
$deleted = ENV_unlink( $entry_spec, $force, $sig_on_error);
}
last
if (!$deleted);
}

$deleted = ENV_rmdir( $top_path, $force, $sig_on_error)
if ($deleted && $also_remove_top);


return $deleted;
}






sub ENV_readdir($;$)
{
my ($path,
$sig_on_error,	    # opt. '', 'EE', etc. Default == 'F'
) = @_;
my @all_entries;

if (opendir( my $dir_fh, $path))
{
@all_entries = grep( ! /^\.\.?$/, readdir $dir_fh); # skip . and ..
closedir $dir_fh;
} else
{
$sig_on_error = 'F'
if (!defined $sig_on_error);
ENV_sig( $sig_on_error => "Unable to opendir '$path'", "- $!");
}

return (wantarray) ? @all_entries : join( "\n", @all_entries);
}





sub ENV_rename($$;$)
{
my ($oldname,
$newname,
$sig_on_error,	    # opt. '', 'EE', etc. Default == 'F'
) = @_;
my $renamed = 0;

if (rename( $oldname, $newname))
{
$renamed = 1;
} else
{
$sig_on_error = 'F'
if (!defined $sig_on_error);
ENV_sig( $sig_on_error => "Unable to rename '$oldname'",
"              -> '$newname'",
"- $!");
}


return $renamed;
}




sub ENV_dir_count($;$)
{
my ($path,
$sig_on_error,	    # opt. '', 'EE', etc. Default == 'F'
) = @_;
my $nr_of_entries;	    # undef if path could not be read

if (opendir( my $dir_fh, $path))
{
$nr_of_entries = grep( ! /^\.\.?$/, readdir $dir_fh); # skip . and ..
closedir $dir_fh;
} else
{
$sig_on_error = 'F'
if (!defined $sig_on_error);
ENV_sig( $sig_on_error => "Unable to opendir '$path'", "- $!");
}

return $nr_of_entries;
}




sub ENV_unlink($;$$)
{
my ($filespec,
$force,		    # opt. bool
$sig_on_error,	    # opt. '', 'EE', etc. Default == 'F'
) = @_;
my $deleted = 0;

$deleted = unlink $filespec;
if (!$deleted)
{
if (defined $force && $force)
{
chmod oct(777), $filespec;	# ugo+rwx
$deleted = unlink $filespec;
}
if (!$deleted)
{



sleep( 1);
chmod oct(777), $filespec;	# ugo+rwx
$deleted = unlink  $filespec;
if ($deleted)
{
ENV_sig( I => "Delete of '$filespec' succeeded after delay");
} else
{
$sig_on_error = 'F'
if (!defined $sig_on_error);
ENV_sig( $sig_on_error => "Cannot delete '$filespec'", "- $!");
}
}
}


return $deleted;
}




sub ENV_copy_file($$$)
{
my ($in_file,
$out_file,
$sig_on_error,	    # undef, '', 'EE', 'F', etc
) = @_;
my $replicated = 1;    	    # 1 = replicated, 0 == error

my ($rc, $command, $fail_reason) = OS_copy_file( $in_file, $out_file);
if ($rc != 0)
{
ENV_sig( $sig_on_error => "Copy (replicate) command failed. rc = $rc",
"Command: $command",
"- $fail_reason");
$replicated = 0;
}

return $replicated;
}




sub ENV_backup_file($$$)
{
my ($filespec,
$backup_type,	    # e.g.: .sav
$sig_on_error,	    # undef, '', 'EE', etc
) = @_;
my $backup_filespec;   # undef if failed


my $backup_file_format = "$filespec.%04d$backup_type";
my $i = 1;
do
{
$backup_filespec = sprintf( $backup_file_format, $i++);
} while ( -e $backup_filespec);

spit_file( $backup_filespec, $sig_on_error, slurp_file( $filespec, $sig_on_error));



return $backup_filespec;
}




sub ENV_touch_file($;$)
{
my ($filespec,
$sig_on_error,	    # opt. '', 'EE', etc. Default == 'F'
) = @_;
my $touched = 1;

if (open( my $touch_fh, '>>', $filespec))
{
close( $touch_fh);
} else
{
$sig_on_error = 'F'
if (!defined $sig_on_error);
ENV_sig( $sig_on_error => "Unable to touch (append) file '$filespec'", "- $!");
$touched = 0;
}

return $touched;
}




sub ENV_hide_file($$)
{
my ($filespec,
$must_hide	    # 0 = unhide, 1 = hide, undef = test-only
) = @_;


return OS_hide_file( $filespec, $must_hide);
}




sub ENV_max_command_line()
{
if (!defined $MAX_COMMAND_LINE)
{

if ($IS_WIN32)
{
$MAX_COMMAND_LINE = 8191;

} else  # Unix
{
$MAX_COMMAND_LINE = 32000;
}

$MAX_COMMAND_LINE--;    # just to be sure!
}

return $MAX_COMMAND_LINE;
}






sub ENV_command_split($)
{
my ($command_line) = @_;
my ($command, @args);

($command, @args) = ENV_split_quoted_space( $command_line);

return ($command, [ @args ]);
}




sub ENV_new_command_ref($$$$)
{
my ($type,
$prefix,    # (perl | call | start | '')
$os_exec,
$args_ref,  # may be undef
) = @_;
my $command_ref;


$os_exec = ENV_enquote( $os_exec)
if (defined $os_exec);

my @args = (defined $args_ref) ? @{$args_ref} : ();
$command_ref = [ $type, $prefix, $os_exec, @args ];

return $command_ref;
}




sub ENV_get_command_ref_data($)
{
my ($command_ref	# [ $type, $prefix, $os_exec, @args ]
) = @_;


return @{$command_ref};
}




sub ENV_command_ref($$)
{
my ($os_command_file,
$args_ref,
) = @_;
my $command_ref;


my $type;
my $prefix = '';
my $os_exec;

$type = lc ENV_split_spec_t( $os_command_file);
if ($type eq '.pl')
{
$prefix = 'perl';
$os_command_file = ENV_find_in_perl_inc( $os_command_file, 1);
} else
{
if ($IS_WIN32)
{
if ($os_command_file eq 'start')
{
$prefix = 'start';
$os_command_file = undef;
} else
{
if ($type eq '.bat')
{
$prefix = 'call';
} elsif ($type eq '.exe' || $type eq '.com' || $type eq '')
{
;	# no change
} else
{
$prefix = 'start';
}
}
} else	#Unix
{
;	# no change
}
}
$os_exec = $os_command_file;

$command_ref = ENV_new_command_ref( $type, $prefix, $os_exec, $args_ref);

return $command_ref;
}






sub ENV_command_line($)
{
my ($command_ref) = @_;	# [ $type, $prefix, $os_exec, @args ]


my $command_line;

ENV_sig( F => "Error Calling ENV_command_ref - not a ref")
if (!ref $command_ref);
my (undef, $prefix, $os_exec, @args) = ENV_get_command_ref_data( $command_ref);

my @command;
if ($prefix ne '')
{
$prefix = $PERL_CMD
if ($prefix eq 'perl');
push @command, $prefix;
}
push @command, $os_exec
if ($os_exec);
push @command, ENV_join_quoted_space( @args)
if (@args && $args[0] ne '');
$command_line = "@command";



return $command_line;
}





sub ENV_find_in_perl_inc($$)
{
my ($file,
$must_find,
) = @_;
my $filespec = '';

if (ENV_is_abs_path( $file))
{
$filespec = $file;
} else
{
foreach my $path (@INC)
{
if (-f "$path/$file")
{
$filespec = "$path/$file";
last;
}
}
ENV_sig( F => "Could not find '$file' in (@INC)")
if ($filespec eq '' && $must_find);
}

return $filespec;
}






sub ENV_system($$)
{
my ($command_or_ref,    # command or $command_ref (ADT) ([ $type, $prefix, $os_exec, @args ])
$oks_ref,	    # allowed return-codes (undef == nocheck)
) = @_;
my $rc;

my $command = (ref $command_or_ref) ? ENV_command_line( $command_or_ref): $command_or_ref;


if (length( $command) > ENV_max_command_line())
{
my $command_length = length( $command);
ENV_sig( F => "system(): commandline length ($command_length) > $MAX_COMMAND_LINE.",
"First part is:",
substr( $command, 0, 80),
"Last part is:",
substr( $command, -80));
}


my $full_rc = system( $command ); 	  # execute the command

$rc = handle_rc( $full_rc, $oks_ref, $command);


return $rc;
}







sub ENV_backtick($$$)
{
my ($command_or_ref,    # command or $command_ref (ADT) ([ $type, $prefix, $os_exec, @args ])
$oks_ref,	    # allowed return-codes (undef == nocheck)
$include_stderr,    # bool
) = @_;
my ($rc, $stdout);

my $command = (ref $command_or_ref) ? ENV_command_line( $command_or_ref): $command_or_ref;

$command = "$command 2>&1"
if ($include_stderr);


if (length( $command) > ENV_max_command_line())
{
my $command_length = length( $command);
ENV_sig( F => "system(): commandline length ($command_length) > $MAX_COMMAND_LINE.",
"First part is:",
substr( $command, 0, 80),
"Last part is:",
substr( $command, -80));
}	    #map { ENV_trace( $_) } @lines;


$stdout = qx{ $command };		# backtick
my $full_rc = $?;

$rc = handle_rc( $full_rc, $oks_ref, $command);


return (wantarray) ? ($rc, $stdout) : $stdout;
}




sub handle_rc($$$;$)
{
my ($full_rc,
$oks_ref,	# allowed return-codes (undef == nocheck)
$command,	# for error reporting
$signal,	# Default = 'F'
) = @_;
my $rc;

$signal = 'F'
if (!defined $signal);

if ($full_rc == 0)
{
$rc = 0;
} else
{
if ($full_rc == -1)
{
ENV_sig( $signal => "system: Cannot execute '$command'",
"- $!");
} else
{
$rc = ENV_rc_status( $full_rc, "system $command");
}
}
if (defined $oks_ref)
{
my @oks = (ref $oks_ref) ? @{$oks_ref} : ($oks_ref);
my $rc_ok = grep( $_ == $rc, @oks);
if (!$rc_ok)
{
ENV_sig( $signal => "Unable to execute '$command' (rc=$rc)");
}
}

return $rc;
}




sub ENV_run3($$$$$)
{
my ($command_or_ref,    # command or $command_ref (ADT) # [ $type, $prefix, $os_exec, @args ]
$multipart_ref,	    # undef = ok. Entries with whitespace will be quoted
$oks_ref,	    # allowed return-codes (undef == nocheck)
$stdout_ref,	    # 1 (non-ref==discard)		 can be REF ARRAY or REF SCALAR
$stderr_ref,	    # 2 (undef==print, non-ref==discard) can be REF ARRAY or REF SCALAR
) = @_;
my $rc;

my $command = (ref $command_or_ref) ? ENV_command_line( $command_or_ref): $command_or_ref;


my $stdout_file = ENV_get_tmp_spec( 'run3_stdout');
my $stderr_file = ENV_get_tmp_spec( 'run3_stderr');
my $command_tail = qq(2> "$stderr_file" > "$stdout_file");
my @commands;
if (defined $multipart_ref)
{
my @multiparts = ENV_enquote( @{$multipart_ref});

my $full_command = "$command @multiparts $command_tail";
if (length( $full_command) <= ENV_max_command_line())
{
@commands = ($full_command);
} else
{
my $command_tail_l = length( $command_tail);
my $full_command = $command;
foreach my $multipart (@multiparts)
{
if (length( $full_command) + 1 + length( $multipart) + 1 + $command_tail_l < $MAX_COMMAND_LINE)
{
$full_command .= ' ' . $multipart;
} else
{
push @commands, $full_command . ' '. $command_tail;
$full_command = $command;
}
}
push @commands, $full_command . ' '. $command_tail
if (length( $full_command) != length($command));
}
} else
{
@commands = ("$command $command_tail");
}




my $full_stdout;
my $full_stderr;
foreach my $command_line (@commands)
{

$rc = ENV_system( $command_line, undef);
my $stdout = slurp_file( $stdout_file, 1);
my $stderr = slurp_file( $stderr_file, 1);
unlink( $stdout_file, $stderr_file);
if (defined $oks_ref)
{
my @oks = (ref $oks_ref) ? @{$oks_ref} : ($oks_ref);
my $rc_ok = grep( $_ == $rc, @oks);
if (!$rc_ok)
{
ENV_say(   1, "### ENV_run3 failed (rc=$rc) ###");
ENV_say(   0, "COMMAND:",
"  $command_line",
"STDOUT:");
ENV_print( 0, $stdout);
ENV_say(   0, "STDERR:");
ENV_print( 0, $stderr);
ENV_say(   0, "***");
ENV_sig( F => "Unable to execute command (rc=$rc)");
} else
{
if (!defined $stderr_ref && $stderr ne '')
{
exec_beep();
ENV_say( 0, $command);
ENV_print( 0, $stderr);
}
}
}



$full_stdout .= $stdout;
$full_stderr .= $stderr;
}

de_ref( $stdout_ref, $full_stdout);
de_ref( $stderr_ref, $full_stderr);

return $rc;
}






sub ENV_exec($$)
{
my ($command_or_ref,	    # command or $command_ref (ADT) # [ $type, $prefix, $os_exec, @args ]
$log_file,		    # undef = no redirection
) = @_;

my $command = (ref $command_or_ref) ? ENV_command_line( $command_or_ref): $command_or_ref;

if (defined $log_file)
{
$log_file = ENV_enquote( $log_file);
$command = "$command > $log_file 2>&1";
}

$RUNNING_IN_EXEC = 1;
exec( $command);

die( "$PREFIX: exec '$command' failed\n - $!");	# perl will not accept anything other than 'die' after 'exec'
}




sub ENV_rc_status($$)
{
my ($full_rc,
$msg,
) = @_;
my $rc;

my $rc16 = $full_rc & 0xffff;
$rc = $rc16 >> 8;
my $sig = $rc16 & 127;
my $core = $rc16 & 128;



if ($sig != 0)
{
ENV_sig( E => "Command '$msg' failed with signal $sig (rc=$rc)");
}
if ($core != 0)
{
ENV_sig( E => "Command '$msg' failed with coredump (sig=$sig rc=$rc)");
}

return $rc;
}




sub de_ref($$)
{
my ($value_ref,	# undef: do nothing. ref to scalar or list
$string,
) = @_;

my $ref = ref $value_ref;
if ($ref eq 'ARRAY')
{
@{$value_ref} = split( /\s*\n/, $string);
} elsif ($ref eq 'SCALAR')
{
$$value_ref = $string;
}
}





sub ENV_detab($)
{
my ($line) = @_;

my $tab_pos = index( $line, "\t");
while ($tab_pos >= 0)
{

my $wanted_out_pos = (($tab_pos + 8) / 8) * 8;
my $nr_spaces = $wanted_out_pos - $tab_pos;
my $spaces = ' ' x $nr_spaces;
$line = substr( $line, 0, $tab_pos) .
$spaces .
substr( $line, $tab_pos + 1);

$tab_pos = index( $line, "\t");
}

return $line;
}





sub ENV_split_quoted_space($)
{
my ($line) = @_;
my @list;

if (index( $line, '"') >= 0)

{
my $i = 0;
my $last_i = length( $line) + 1;
$line .= '  ';				    # add 2 dummy trailing space
my $next_char = substr( $line, $i++, 1);
do
{




if ($i < $last_i)
{
while ($next_char eq ' ' || $next_char eq "\t")
{
$next_char = substr( $line, $i++, 1);
}
}
if ($i < $last_i)
{
my $word = '';
if ($next_char eq '"')
{



$next_char = substr( $line, $i++, 1);	# skip opening quote
while ($next_char ne '"' && $i < $last_i)
{
$word .= $next_char;
if ($next_char eq '\\')
{
$next_char = substr( $line, $i++, 1);
$word .= $next_char;
}
$next_char = substr( $line, $i++, 1);
}
$next_char = substr( $line, $i++, 1);	#skip closing quote
} else
{



while ($next_char ne ' ' && $next_char ne "\t")
{
$word .= $next_char;
$next_char = substr( $line, $i++, 1);
if ($next_char eq '"')			# internal '"'
{
$word .= $next_char;		# add opening quote
$next_char = substr( $line, $i++, 1);
while ($next_char ne '"' && $i < $last_i)
{
$word .= $next_char;
if ($next_char eq '\\')
{
$next_char = substr( $line, $i++, 1);
$word .= $next_char;
}
$next_char = substr( $line, $i++, 1);
}
$word .= $next_char; 		# add closing quote
$next_char = substr( $line, $i++, 1);
}
}
}
push @list, $word;
}
} while ($i < $last_i);
} else
{
@list = split( ' ', $line);	    # split on any whitespaces
}

return @list;
}




sub ENV_join_quoted_space(@)
{
my @list = @_;


foreach my $word (@list)
{
ENV_stackdump() if (!defined $word);
if ($word eq '')
{
$word = '""';
} elsif (index( $word, ' ') >= 0 && index( $word, '"') < 0)
{



$word =~ s/"/\\"/g;
$word = "\"$word\"";
}
}

return "@list";
}





sub ENV_enquote(@)
{
my @list = (@_);


foreach my $word (@list)
{
if ($word eq '')
{
$word = '""'
} elsif (substr( $word, 0, 1) ne '"' && $word =~ /"|\s/)
{
$word =~ s/"/\\"/g;
$word = "\"$word\"";
}
}


return wantarray ? @list : $list[0];
}




sub ENV_dequote(@)
{
my @list = (@_);


foreach my $word (@list)
{
if (substr( $word, 0, 1) eq '"')
{
$word =~ s/\\"/"/g;
$word = substr( $word, 1, -1);
}
}


return wantarray ? @list : $list[0];
}




sub ENV_split_lines($@)
{
my ($max_length,
@lines
) = @_;
my @split_lines;

foreach my $line (@lines)
{
my $line_length = length $line;
if ($line_length <= $max_length)
{
push @split_lines, $line;
} else
{
my $indent = '';
my $max_sub_length = $max_length;
if ($line =~ /^(- |\s+)/)			# starts with '- ' or spaces
{

$indent = ' ' x length( $1);		# get indent
}
my $start_index = 0;
my $i = 0;
do
{

my $remaining_length = $line_length - $start_index;

my $part;
if ($remaining_length > $max_sub_length)
{

my $ri = rindex( substr( $line, $start_index, $max_sub_length), ' ');

if ($ri >= 0)
{
$part = substr( $line, $start_index, $ri);
$start_index = $start_index + $ri + 1;
} else
{
$part = substr( $line, $start_index, $max_sub_length);
$start_index = $start_index + $max_sub_length;
$start_index++
if ($start_index < $line_length && substr( $line, $start_index, 1) eq ' '); # exact fit
}
} else
{
$part = substr( $line, $start_index);
$start_index = $line_length;
}
if ($i == 0)
{
$max_sub_length = $max_length - length( $indent);
} else
{
$part = "$indent$part";
}

push @split_lines, $part;
$i++;
} while ($start_index < $line_length && $i < 32000);
}
}

return @split_lines;
}




sub ENV_redirect_stdouterr($)
{
my ($log_file) = @_;

open( $SAVED_STDOUT, '>&', STDOUT) || my_die( "$PREFIX: Cannot save STDOUT\n- $!");
open( $SAVED_STDERR, '>&', STDERR) || my_die( "$PREFIX: Cannot save STDERR\n- $!");
open( STDOUT, '>>', $log_file) || my_die( "$PREFIX: Cannot redirect STDOUT to $log_file\n- $!");
open( STDERR, '>&', STDOUT) || my_die( "$PREFIX: Cannot duplicate STDERR to STDOUT\n- $!");

my $saved_handle = select STDERR;
$| = 1;
select STDOUT;
$| = 1;
select $saved_handle;
}




sub ENV_restore_stdouterr()
{
close( STDOUT) || my_die( "$PREFIX: Cannot close STDOUT\n- $!");
close( STDERR) || my_die( "$PREFIX: Cannot close STDERR\n- $!");
open( STDOUT, '>&', $SAVED_STDOUT) || my_die( "$PREFIX: Cannot restore STDOUT\n- $!");
open( STDERR, '>&', $SAVED_STDERR) || my_die( "$PREFIX: Cannot restore STDERR\n- $!");
}




sub ENV_get_desktop_path($)
{
my ($system_item) = (@_);

return OS_get_desktop_path( $system_item);
}




sub ENV_get_startmenu_path($)
{
my ($system_item) = (@_);

return OS_get_startmenu_path( $system_item);
}






sub ENV_get_programs_path($)
{
my ($system_item) = (@_);

return OS_get_programs_path( $system_item);
}






sub ENV_get_user_path($)
{
my ($system_item) = (@_);

return OS_get_user_path( $system_item);
}




sub ENV_is_administrator()
{
return OS_is_administrator();
}




sub slurp_file($$)
{
my ($filespec,
$sig_on_error,	    # undef, '', 'EE', etc
) = @_;
my $lines;		# undef if error

if (open( my $in_fh, '<', $filespec))
{
my $old_irs = $/;
$/ = undef;
$lines .= <$in_fh>;
$/ = $old_irs;
if (!close( $in_fh))
{
ENV_sig( $sig_on_error => "Unable to close file '$filespec'", "- $!");
$lines = undef;
}
} else
{
ENV_sig( $sig_on_error => "Unable to open file '$filespec'", "- $!");
}

return $lines;
}




sub spit_file($$$)
{
my ($filespec,
$sig_on_error,	    # undef, '', 'EE', etc
$lines_or_ref,
) = @_;
my $result_ok = 1;

if (open( my $out_fh, '>', $filespec))
{
print( $out_fh (ref $lines_or_ref) ? $$lines_or_ref : $lines_or_ref);
if (!close( $out_fh))
{
ENV_sig( $sig_on_error => "Unable to close file '$filespec'", "- $!");
$result_ok = 0;
}
} else
{
ENV_sig( $sig_on_error => "Unable to create file '$filespec'", "- $!");
$result_ok = 0;
}

return $result_ok;
}




sub my_die(@)
{
ENV_stackdump();
die @_;
}

1;
