#===============================================================
#
#   notify_tkx.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;

eval { use Tkx; };
my $TKX_AVAILABLE = ($@) ? 0 : 1;




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use glo::slurp;
use gbsenv;
use run;




sub handle_return();
sub view_log();
sub get_command_args();




my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $GBS_LOG_PATH = ENV_getenv_perl_path( 'GBS_LOG_PATH');

my $VIEW_BUTTON;
my $EXIT_BUTTON;




my ($JOB,
$JOB_RC,
$LOG_FILE_SPEC,
$JOB_STATE);







get_command_args();
if (!$TKX_AVAILABLE)
{
ENV_sig( EE => "Cannot proceed. Perl Tkx Package is not available");
}




{
my $mw = Tkx::widget->new( ".");
$mw->g_wm_title("GBS Notify: $JOB");
$mw->g_wm_resizable( 0, 0);

Tkx::bell();

my $image;
if ($JOB_RC == 0)
{
$image = Tkx::image_create_photo( -file => "$GBS_SCRIPTS_PATH/doc/images/gbs.gif");
$mw->new_ttk__label( -text=> "Batch Job: $JOB $JOB_STATE-END")->g_pack();
$mw->new_ttk__label( -text=> "Exit-Status: $JOB_RC")->g_pack();
} else
{
$image = Tkx::image_create_photo( -file => "$GBS_SCRIPTS_PATH/doc/images/gbs_test.gif");
$mw->new_ttk__label( -foreground => 'red', -text => "Batch Job: $JOB $JOB_STATE-END")->g_pack();
$mw->new_ttk__label( -foreground => 'red', -text => "Exit-Status: $JOB_RC")->g_pack();
}
$mw->g_wm_iconphoto( $image);

$mw->new_ttk__label( -text => "Logfile: $LOG_FILE_SPEC")->g_pack(-padx => 10);




my $b1 = $mw->new_button( -text => "View Log & Exit",
-default => "active",
-command => sub { view_log(); exit });
$b1->g_pack( -side => 'left',
-expand => 1,
-pady => 5);




my $b2 = $mw->new_button( -text => "Exit",
-command => sub { exit });
$b2->g_pack( -side => 'left',
-expand => 1,
-pady => 5);




$VIEW_BUTTON = $b1->_mpath();
$EXIT_BUTTON = $b2->_mpath();
Tkx::focus( $VIEW_BUTTON);
Tkx::bind( ".", '<Return>', sub { handle_return(); });
}




Tkx::MainLoop();




END
{
ENV_print_end_msg(0);
}




sub handle_return()
{
my $focus = Tkx::focus();

if ($focus eq $VIEW_BUTTON)
{
view_log(); exit;
} elsif ($focus eq $EXIT_BUTTON)
{
exit;
} else
{
Tkx::bell();

}
}




sub view_log()
{
RUN_viewer( $LOG_FILE_SPEC);
}




sub get_command_args()
{


($JOB,
$JOB_RC,
$LOG_FILE_SPEC,
$JOB_STATE) = ARGS_get( [1,4], 'notify_tkx.pl [ <job> | test ] <job_rc> <log_filespec> <job_end>', undef);
if ($JOB eq 'test')
{
$JOB_RC = 0
if (!defined $JOB_RC);
if (!defined $LOG_FILE_SPEC)
{
($LOG_FILE_SPEC) = SLURP_dir_file_paths( $GBS_LOG_PATH, 0);		# get first logfile
$LOG_FILE_SPEC = "$GBS_LOG_PATH/gbssysaudit_test_2003-243.log"
if (!defined $LOG_FILE_SPEC);
}
$JOB_STATE = ($JOB_RC == 0) ? 'NORMAL' : 'FAILED'
if (!defined $JOB_STATE);
} else
{
ARGS_fatal( "4 Arguments expected (@ARGV)")
if (@ARGV != 4);
}
}


