#===============================================================
#
#   cpptestinit.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*CPPTEST_INIT @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib "$ENV{GBS_SCRIPTS_PATH}";
use glo::env;
use glo::args;
use glo::flock;
use glo::slurp;
use glo::spit;
use gbsenv;
use gbsglo;
use target;
use audit;
use bldopt;








my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $GBS_ROOT_PATH = ENV_getenv_perl_path( 'GBS_ROOT_PATH');
my $AUDIT_PLUGIN_BIN_PATH = ENV_getenv_perl_path( 'GBSEXT_CPPTEST_BIN_PATH');
my $GBSEXT_CPPTEST_BUILD = ENV_getenv( 'GBSEXT_CPPTEST_BUILD');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');




my $CPPTESTCLI = "$AUDIT_PLUGIN_BIN_PATH/cpptestcli";


my $GBS_SILO_CPPTEST_PATH = "$GBS_ROOT_PATH/silo/.gbs/cache/cpptest";
my $DATA_PATH = "$GBS_SILO_CPPTEST_PATH/$GBS_TARGET";







my (
@ARGS
) = ARGS_get( 0, 'perl cpptestinit.pl', undef);






$| = 1;
my $RC = 0;
my $BDF_SPEC;

{



my $fh = FLOCK_ex_wait( "$DATA_PATH.lck");




my @audit_src_types = AUDIT_get_src_types( $GBS_AUDIT);
my %commands;

my @src_types_re;
foreach my $src_type (TARGET_get_src_types( $GBS_TARGET))
{
if (grep( $src_type eq $_, @audit_src_types))
{
my @data_refs = TARGET_get_src_command_data_refs( $GBS_TARGET, $src_type);


my $command_line = $data_refs[0]->[1];	    # first command: $command_line
my ($command, @args) = split( ' ', $command_line);
my @leading_args;
foreach my $arg (@args)
{
my $fc = substr( $arg, 0, 1);
if ($fc eq '%' || $fc eq '$')
{
last;
} else
{
push @leading_args, $arg;
}
}
$commands{$src_type} = [ ENV_expand_envs( $command), @leading_args ];
push @src_types_re, substr( $src_type, 1);
}
}





BLDOPT_init( undef);
my @lines;
foreach my $subsys (GBSGLO_subsystems_for_audit( $GBS_AUDIT, $GBS_TARGET))
{
if (GBSGLO_subsystem_is_full_gbs( $subsys))
{
ENV_whisper( 1, "BDF: $subsys");
$ENV{GBS_SUBSYS} = $subsys;	    # for scope.pm
foreach my $component (GBSGLO_components_for_audit( $subsys, $GBS_AUDIT, $GBS_TARGET))
{

my $wdir = "$GBS_ROOT_PATH/dev/$subsys/comp/$component/src";
my $out_file_path = "../bld/$GBS_TARGET";
my @all_files = SLURP_dir_files( $wdir, 0);
ENV_chdir( $wdir);
foreach my $src_type_re (@src_types_re)
{
my @files = grep(/\.($src_type_re)$/, @all_files);
if (@files)
{
my $src_type = ".$src_type_re";
my $project_name = "${subsys}_$component";
my ($this_command, @leading_args) = @{$commands{$src_type}};
my @env_def_refs = BLDOPT_get_envs_settings( $subsys, $component, $src_type);

my @envvar_args;
foreach my $ref (@env_def_refs)
{
my ($envvar_name, @envvar_values) = @{$ref};
push @envvar_args, @envvar_values;		# -I and -D

}
push @envvar_args, BLDOPT_get_options_flags( $src_type, undef);
foreach my $file (@files)
{

my ($file_name) = $file =~ /^(.*)\.*$/;
my $out_file_base = "$out_file_path/$file_name";
my @args = ($this_command, @leading_args);
push @args, ( $file );
push @args, @envvar_args;
push @args, ( '-o', ENV_os_path( "$out_file_base.o") );




push @lines, "------- cpptestscan v.$GBSEXT_CPPTEST_BUILD (cpptesttrace) -------";
push @lines, "working_dir=$wdir";
push @lines, "project_name=$project_name";
push @lines, map { "arg=$_" } @args;
}
}
}
}
}
}





$BDF_SPEC = "$GBS_SILO_CPPTEST_PATH/cpptestscan_$GBS_TARGET.bdf";
my $new_lines = join( "\n", (@lines, ''));
my $old_lines = (-e $BDF_SPEC) ? SLURP_file( $BDF_SPEC) : '';
if ($new_lines ne $old_lines)
{
ENV_say( 1, "Generating C++test WorkSpace....");




ENV_rmtree( $DATA_PATH, 1, 0, 'EE')
if (-e $DATA_PATH);

SPIT_file_nl( $BDF_SPEC, \@lines);





{
my $gbsext_use_stub = ENV_getenv_bool( "GBSEXT_CPPTEST_USE_STUB");
if ($gbsext_use_stub)
{
ENV_say( 1, "** Running in Development Mode from STUBS...");
my $gbsext_plugin_bin_path = "$GBS_SCRIPTS_PATH/STUBS/cpptest";
ENV_setenv( "GBSEXT_CPPTEST_BIN_PATH" => ENV_os_path( $gbsext_plugin_bin_path));
$CPPTESTCLI = "$gbsext_plugin_bin_path/cpptestcli";
}
}




my $command = "$CPPTESTCLI -nobuild -bdf \"$BDF_SPEC\" -data \"$DATA_PATH\"";
CORE::say( $command);
$RC = ENV_system( $command, undef);
if ($RC != 0)
{
ENV_sig( 'EE', "Unable to execute '$command'", "(RC=$RC)");
}
} else
{
ENV_say( 1, "No change in bdf");
}




FLOCK_unlock( $fh);
}




ENV_exit( $RC);




END
{
my $rc = $?;

if ($rc != 0)
{
if (-e $BDF_SPEC)
{
unlink $BDF_SPEC;
ENV_say( 1, "bdf_file deleted");
}
}
ENV_print_end_msg( 0);
}



