#===============================================================
#
#   cpptestmaint.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*CPPTESTMAINT @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use glo::ask;
use gbsenv;
use plugin;
use audit;
use scamaint;
use scaglo;
use lib "$ENV{GBS_SCRIPTS_PATH}/plugins/audit/cpptest/bin";
use cpptestglo;
use cpptest;




sub do_recreate_msg($$$);
sub do_convert_settings($$$);
sub convert_settings_line($$);

sub get_args();




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_AUDIT_PLUGIN = ENV_getenv( 'GBS_AUDIT_PLUGIN');	    # cpptest
my $GBSEXT_PLUGIN_PATH = ENV_getenv( 'GBSEXT_CPPTEST_PATH');
my $GBSEXT_PLUGIN_REL = ENV_getenv( 'GBSEXT_CPPTEST_REL');

my $PLUGIN_NICE_NAME;




my %ALL_MSG_IDS;








$| = 1;
my $RC = 0;
GBSENV_init();




my @MENU_ENTRIES;
get_args();




ENV_sig( EE => "Audit plugin mismatch: $GBS_AUDIT [ $GBS_AUDIT_PLUGIN ] <> cpptest")
if ($GBS_AUDIT_PLUGIN ne 'cpptest');




SCAMAINT_init();

{



$PLUGIN_NICE_NAME = AUDIT_get_section_items( $GBS_AUDIT, 'SETUP', 'NICE_NAME');
SCAGLO_init( $GBS_AUDIT_PLUGIN, $GBSEXT_PLUGIN_REL, $PLUGIN_NICE_NAME);    # Includes SCAMSG_init




my $gbs_plugin_path = PLUGIN_get_gbs_path( audit => $GBS_AUDIT_PLUGIN);
SCAGLO_validate( CPPTEST_init( $GBS_AUDIT_PLUGIN, $GBSEXT_PLUGIN_PATH, $gbs_plugin_path));

my @main_menu_items = (
[ 'Show Help file (html)',						    \&SCAMAINT_show_help ],
[ 'Recreate SCA Msg and SCA Help file',					    \&do_recreate_msg ],
[ "Convert $GBS_AUDIT_PLUGIN warning settings file to sca_<*>.gbs settings file", \&do_convert_settings ],
);
ASK_menu( 'Select function to perform', \@main_menu_items, \@MENU_ENTRIES);
}




ENV_exit( $RC);




END
{
ENV_print_end_msg( 1);
}




sub do_recreate_msg($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref,
) = @_;

SCAGLO_create_msg_and_help_files();
}




sub do_convert_settings($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref,
) = @_;

foreach my $msg_ref (CPPTESTGLO_read_msg_file( 0))		# Minimal info
{

my $msg_id = $msg_ref->[0];
$ALL_MSG_IDS{$msg_id} = $msg_ref->[3];	# $msg_text
}

my @builtin_file_refs;
{
my @prefixes = qw( builtin user);
foreach my $config_path (CPPTESTGLO_get_config_paths())	# ($config_builtin_path, $config_user_path
{
my $prefix = shift @prefixes;
my @filespecs = ENV_glob( "$config_path/*.properties");
foreach my $filespec (ENV_glob( "$config_path/*.properties"))
{
my $filename = ENV_split_spec_n( $filespec);
push @builtin_file_refs, [ "$prefix://$filename", $filespec ];
}
}
}
SCAMAINT_convert_settings( \@builtin_file_refs, \&convert_settings_line, '#');

%ALL_MSG_IDS = ();
}




sub convert_settings_line($$)
{
my ($inline,
$line_nr,
) = @_;
my @outlines;
















my ($item, $value) = split( '=', $inline, 2);
if (defined $value && $value eq 'true' && $item !~ /^com\./)
{
if ($item =~ /^[A-Z][A-Z0-9-]*-\d+[a-z0-9_]*$/)	# e.g.: MISRA2004-12_2_g
{
my $msg_id = $item;
my $msg_text = $ALL_MSG_IDS{$msg_id};
if (defined $msg_text)
{
push @outlines, "+E $msg_id\t\t# $msg_text";
} else
{
push @outlines, "#+E $msg_id";
ENV_say( 1, "Msg-nr $msg_id not found in msg file", sprintf( "%6d  %s", $line_nr, $inline));
}
} else
{
push @outlines, "# Ignored: $inline";
ENV_whisper( 1, "Ignored", sprintf( "%6d  %s", $line_nr, $inline));
}
}

return @outlines;
}




sub get_args()
{
@MENU_ENTRIES = ARGS_get( undef, 'perl pclintmaint.pl <menu-entries>...', undef);
foreach my $entry (@MENU_ENTRIES)
{
ARGS_error_exit( "Menu Entry '$entry' of (@MENU_ENTRIES) must be numeric")
if ($entry !~ /^\d+$/);
ARGS_error_exit( "Menu Entry '$entry' of (@MENU_ENTRIES) must be at least 1")
if ($entry <= 0);
}
}


