#===============================================================
#
#   pclintmaint.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*PCLINTMAINT @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use glo::ask;
use gbsenv;
use plugin;
use audit;
use scamaint;
use scaglo;
use lib "$ENV{GBS_SCRIPTS_PATH}/plugins/audit/pclint/bin";
use pclintglo;
use pclint;




sub do_recreate_msg($$$);
sub do_convert_settings($$$);
sub convert_settings_line($$);

sub handle_msg_nr($$$$$);
sub get_args();




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_AUDIT_PLUGIN = ENV_getenv( 'GBS_AUDIT_PLUGIN');	    # pclint
my $GBSEXT_PLUGIN_PATH = ENV_getenv( 'GBSEXT_PCLINT_PATH');
my $GBSEXT_PLUGIN_REL = ENV_getenv( 'GBSEXT_PCLINT_REL');

my $PLUGIN_NICE_NAME;




my %ALL_MSG_NRS;

my @ALL_MSG_NRS;	# for wildcard match







$| = 1;
my $RC = 0;
GBSENV_init();




my @MENU_ENTRIES;
get_args();




ENV_sig( EE => "Audit plugin mismatch: $GBS_AUDIT [ $GBS_AUDIT_PLUGIN ] <> pclint")
if ($GBS_AUDIT_PLUGIN ne 'pclint');




SCAMAINT_init();

{



$PLUGIN_NICE_NAME = AUDIT_get_section_items( $GBS_AUDIT, 'SETUP', 'NICE_NAME');
SCAGLO_init( $GBS_AUDIT_PLUGIN, $GBSEXT_PLUGIN_REL, $PLUGIN_NICE_NAME);    # Includes SCAMSG_init




my $gbs_plugin_path = PLUGIN_get_gbs_path( audit => $GBS_AUDIT_PLUGIN);
SCAGLO_validate( PCLINT_init( $GBS_AUDIT_PLUGIN, $GBSEXT_PLUGIN_PATH, $gbs_plugin_path));

my @main_menu_items = (
[ 'Show Help file (html)',						     \&SCAMAINT_show_help ],
[ 'Recreate SCA Msg and SCA Help file',					     \&do_recreate_msg ],
[ "Convert $GBS_AUDIT_PLUGIN warning settings file to sca_<*>.gbs settings file", \&do_convert_settings ],
);
ASK_menu( 'Select function to perform', \@main_menu_items, \@MENU_ENTRIES);
}




ENV_exit( $RC);




END
{
ENV_print_end_msg( 1);
}




sub do_recreate_msg($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref,
) = @_;

SCAGLO_create_msg_and_help_files();
}




sub do_convert_settings($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref,
) = @_;

foreach my $msg_ref (PCLINTGLO_read_msg_file( 0))		# Minimal info
{

my $msg_id = $msg_ref->[0];
$ALL_MSG_NRS{$msg_id} = $msg_ref->[3];	# $msg_text
}
@ALL_MSG_NRS = keys %ALL_MSG_NRS;

SCAMAINT_convert_settings( undef, \&convert_settings_line, '//');

%ALL_MSG_NRS = ();
@ALL_MSG_NRS = ();
}




sub convert_settings_line($$)
{
my ($inline,
$line_nr,
) = @_;
my @outlines;


















if (substr( $inline, 0, 1) eq '-' || substr( $inline, 0, 1) eq '+')
{
my ($valid_part, $comments) = split( /\s*\/\//, $inline, 2);

my ($plus_or_minus, $msg_nr) = $valid_part =~ /^(-|\+)e([0-9?*]+)/;	    # -e365   // comments
if (defined $msg_nr)
{
my $sca_option;
if (ENV_is_wildcard( $msg_nr))
{
my @msg_nrs = ENV_wildcard( $msg_nr, \@ALL_MSG_NRS);
push @outlines, "# Wildcard: $inline";
push @outlines, map { handle_msg_nr( $plus_or_minus, $_, $comments, $line_nr, \$inline)} @msg_nrs;
} else
{
push @outlines, handle_msg_nr( $plus_or_minus, $msg_nr, $comments, $line_nr, \$inline);
}
} else
{
push @outlines, "# Ignored: $inline";
ENV_say( 1, "Ignored", sprintf( "%6d  %s", $line_nr, $inline));
}
} else
{
push @outlines, "# Ignored: $inline";
ENV_say( 1, "Line ignored (does not start with '-')", sprintf( "%6d  %s", $line_nr, $inline));
}

return @outlines;
}




sub handle_msg_nr($$$$$)
{
my ($plus_or_minus,
$msg_nr,
$comments,	# may be undef
$line_nr,
$inline_ref,
) = @_;
my @outlines;

my $msg_text = $ALL_MSG_NRS{$msg_nr};
if (defined $msg_text)
{
if (defined $comments)
{
push @outlines, "${plus_or_minus}E $msg_nr\t\t# $comments";
} else
{
push @outlines, "${plus_or_minus}E $msg_nr\t\t# $msg_text";
}
} else
{
push @outlines, "#${plus_or_minus}E $msg_nr";
ENV_say( 1, "Msg-nr $msg_nr not found in msg file", sprintf( "%6d  %s", $line_nr, $$inline_ref));
}

return @outlines;
}




sub get_args()
{
@MENU_ENTRIES = ARGS_get( undef, 'perl pclintmaint.pl <menu-entries>...', undef);
foreach my $entry (@MENU_ENTRIES)
{
ARGS_error_exit( "Menu Entry '$entry' of (@MENU_ENTRIES) must be numeric")
if ($entry !~ /^\d+$/);
ARGS_error_exit( "Menu Entry '$entry' of (@MENU_ENTRIES) must be at least 1")
if ($entry <= 0);
}
}


