#===============================================================
#
#   prqamaint.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*PRQAMAINT @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use glo::ask;
use gbsenv;
use plugin;
use audit;
use scamaint;
use scaglo;
use lib "$ENV{GBS_SCRIPTS_PATH}/plugins/audit/qac/bin";
use prqa;
use prqaglo;




sub do_recreate_msg($$$);
sub do_convert_settings($$$);
sub convert_settings_line($$);
sub parse_comma_range($$$$);
sub parse_threshold($$$);
sub get_args();




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_AUDIT_PLUGIN = ENV_getenv( 'GBS_AUDIT_PLUGIN');	    # qac or qacpp
my $GBS_AUDIT_PLUGIN_UC = uc $GBS_AUDIT_PLUGIN;
my $GBSEXT_PLUGIN_PATH = ENV_getenv( "GBSEXT_${GBS_AUDIT_PLUGIN_UC}_PATH");
my $GBSEXT_PLUGIN_REL = ENV_getenv( "GBSEXT_${GBS_AUDIT_PLUGIN_UC}_REL");

my $PLUGIN_NICE_NAME;




my %ALL_MSG_NRS;








$| = 1;
my $RC = 0;
GBSENV_init();




my @MENU_ENTRIES;
get_args();




ENV_sig( EE => "Audit plugin mismatch: $GBS_AUDIT [ $GBS_AUDIT_PLUGIN ] <> qac or qacpp")
if ($GBS_AUDIT_PLUGIN ne 'qac' && $GBS_AUDIT_PLUGIN ne 'qacpp');




SCAMAINT_init();

{



$PLUGIN_NICE_NAME = AUDIT_get_section_items( $GBS_AUDIT, 'SETUP', 'NICE_NAME');
SCAGLO_init( $GBS_AUDIT_PLUGIN, $GBSEXT_PLUGIN_REL, $PLUGIN_NICE_NAME);    # Includes SCAMSG_init




my $gbs_plugin_path = PLUGIN_get_gbs_path( audit => $GBS_AUDIT_PLUGIN);
SCAGLO_validate( PRQA_init( $GBS_AUDIT_PLUGIN, $GBSEXT_PLUGIN_PATH, $gbs_plugin_path));

my @main_menu_items = (
[ 'Show Help file (html)',						     \&SCAMAINT_show_help ],
[ 'Recreate SCA Msg file',						     \&do_recreate_msg ],
[ "Convert $GBS_AUDIT_PLUGIN warning settings file to sca_<*>.gbs settings file", \&do_convert_settings ],
);
ASK_menu( 'Select function to perform', \@main_menu_items, \@MENU_ENTRIES);
}




ENV_exit( $RC);




END
{
ENV_print_end_msg( 1);
}




sub do_recreate_msg($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref,
) = @_;

SCAGLO_create_msg_and_help_files();
}




sub do_convert_settings($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref,
) = @_;

foreach my $msg_ref (PRQAGLO_read_msg_file( 0))
{

my $msg_id = $msg_ref->[0];
$ALL_MSG_NRS{$msg_id} = $msg_ref->[3];	# $msg_text
}

SCAMAINT_convert_settings( undef, \&convert_settings_line, '*');

%ALL_MSG_NRS = ();
}




sub convert_settings_line($$)
{
my ($inline,
$line_nr,
) = @_;
my @outlines;




















if (substr( $inline, 0, 1) eq '-')
{
if ($inline =~ /^-echo\s/i)
{
push @outlines, "# Ignored: $inline";
ENV_say( 1, "-echo not supported", sprintf( "%6d  %s", $line_nr, $inline));
} else
{

my @opts = split( /\s+-/, " $inline");
shift @opts;	    # skip the added first ' '
foreach my $opt (@opts)
{
my ($key, $value) = split( /\s+/, $opt, 2);
my $key_lc = lc $key;
if ($key_lc eq 'o' || $key_lc eq 'only')
{

push @outlines, parse_comma_range( '+', $value, $line_nr, \$inline);
} elsif ($key_lc eq 'n' || $key_lc eq 'no' || $key_lc eq 'nomsg')
{

push @outlines, parse_comma_range( '-', $value, $line_nr, \$inline);
} elsif ($key_lc eq 'q' || $key_lc eq 'quiet')
{

push @outlines, "+Q $value";
} elsif ($key_lc eq 'forget' || $key_lc eq 'forgetall')
{

my $value_lc = lc $value;
if ($value_lc eq 'o' || $value_lc eq 'only')
{
push @outlines, '-E ALL';
} elsif ($value_lc eq 'no' || $value_lc eq 'nomsg')
{
push @outlines, '+E ALL';
} else
{
push @outlines, "# Ignored: $inline";
ENV_say( 1, "'-$key $value' not supported", sprintf( "%6d  %s", $line_nr, $inline));
}
} elsif ($key_lc eq 'thresh' || $key_lc eq 'thereshold')
{

push @outlines, parse_threshold( $value, $line_nr, \$inline);
} else
{
push @outlines, "# Ignored: $inline";
ENV_say( 1, "-$key not supported", sprintf( "%6d  %s", $line_nr, $inline));
}
}
}
} else
{
ENV_sig( E => "Invalid line", sprintf( "%6d  %s", $line_nr, $inline));
}

return @outlines;
}




sub parse_comma_range($$$$)
{
my ($plus_or_minus,
$value,
$line_nr,
$inline_ref,
) = @_;
my @outlines;

foreach my $possible_range (split( ',', $value))
{
my @msg_nrs;
my ($msg_nr1, $msg_nr2) = $possible_range =~ /^(\d+)(-\d+|)$/;
if (defined $msg_nr1)
{
if ($msg_nr2 ne '')
{
$msg_nr2 = substr( $msg_nr2, 1);	# remove leading '-'
if ($msg_nr2 >= $msg_nr1)
{

foreach my $msg_nr ($msg_nr1..$msg_nr2)
{
push @msg_nrs, $msg_nr
if (exists( $ALL_MSG_NRS{$msg_nr}));
}
} else
{
ENV_sig( E => "Range error $msg_nr2 < $msg_nr1", sprintf( "%6d  %s", $line_nr, $$inline_ref));
}
} else
{
@msg_nrs = ($msg_nr1);
}
} else
{
ENV_sig( E => "'$value' Unknown syntax", sprintf( "%6d  %s", $line_nr, $$inline_ref));
}

foreach my $msg_nr (@msg_nrs)
{
my $msg_text = $ALL_MSG_NRS{$msg_nr};
if (defined $msg_text)
{
push @outlines, "${plus_or_minus}E $msg_nr\t\t# $msg_text";
} else
{
push @outlines, "#${plus_or_minus}E $msg_nr";
ENV_say( 1, "Msg-nr $msg_nr not found in msg file", sprintf( "%6d  %s", $line_nr, $$inline_ref));
}
}
}

return @outlines;
}




sub parse_threshold($$$)
{
my ($value,
$line_nr,
$inline_ref,
) = @_;
my @outlines;


my ($metric, $op, $met_value, $msgnr) = $value =~ /^ST(...)(<|<=|=|>|>=)([\d.]+)(:\d+|)$/;

if (defined $metric)
{

my $min = '';
my $max = '';
if ($op eq '<')
{
$max = $met_value;
$max--
unless ($max =~ /\./);
} elsif ($op eq '<=')
{
$max = $met_value;
} elsif ($op eq '=')
{
$min = $met_value;
$max = $met_value;
} elsif ($op eq '>')
{
$min = $met_value;
$min++
unless ($min =~ /\./);
} else #($op eq '>=')
{
$min = $met_value;
}
$max = ",$max"
if ($max ne '');
push @outlines, "+T $metric=$min$max$msgnr";
} else
{
push @outlines, "# Ignored: $$inline_ref";
ENV_sig( E => "'Invalid syntax", sprintf( "%6d  %s", $line_nr, $$inline_ref));
}

return @outlines;
}




sub get_args()
{
@MENU_ENTRIES = ARGS_get( undef, 'perl prqamaint.pl <menu-entries>...', undef);
foreach my $entry (@MENU_ENTRIES)
{
ARGS_error_exit( "Menu Entry '$entry' of (@MENU_ENTRIES) must be numeric")
if ($entry !~ /^\d+$/);
ARGS_error_exit( "Menu Entry '$entry' of (@MENU_ENTRIES) must be at least 1")
if ($entry <= 0);
}
}


