#===========================================================================
#
#   GHSBF_GLK: glk: .glk -> .exe
#      arg_1   = file_name     (without type)
#      arg_2   = in_filespec  (relative file spec)
#      arg_3   = out_filespec (relative file spec)
#      arg_4   = out_file_dir  (out_filespec directory only)
#      arg_5   = out_filetype (with dot)
#      arg_6-N = command_flags (from command line)
#  Environment:
#      General:
#	   GBS_INCS	    libraries
#	   GBS_GLKB	    files to include
#	   GBS_MAP	    NO / YES
#	   GBS_DEBUGGER     NO / YES
#      Application:
#	   <none>
#============================================================================
use strict;
use warnings;
use integer;
print "*GHSBF_GLK @ARGV\n" if ($ENV{GBSDEBUG_FILE});








sub get_dbas();




my $RC = 0;
my $PREFIX =  'GHSBF_GLK';
my $PREFIX_ = ' 	';

my $FILE_NAME;
my $IN_FILE;
my $OUT_FILE_SPEC;
my $OUT_FILE_DIR;
my $OUT_FILE_TYPE;
my @COMMAND_ARGS;
my $OUT_FILE_BASE;

my $GBSEXT_GHSBF_PATH = $ENV{GBSEXT_GHSBF_PATH};
my $GBS_INCS = $ENV{GBS_INCS};
my $GBS_GLKB = $ENV{GBS_GLKB};
my $GBS_DEBUGGER = $ENV{GBS_DEBUGGER}; $GBS_DEBUGGER = '' if (!defined $GBS_DEBUGGER);
my $GBS_EXT_PATH = $ENV{GBS_EXT_PATH};




if (@ARGV == 1 && $ARGV[0] =~ /^--?(h|help)$/)
{
print "$PREFIX: Usage is: perl ghsbf_glk.pl <file_name> <in_file> <out_filespec> <out_file_dir> <out_filetype> [ <arg> ]... \n";
exit 0;
} else
{
die "$PREFIX: Nr args must be >= 5\n($PREFIX_ (@ARGV)\n\a"
if (@ARGV < 5);
}
( $FILE_NAME,
$IN_FILE,
$OUT_FILE_SPEC,
$OUT_FILE_DIR,
$OUT_FILE_TYPE,
@COMMAND_ARGS ) = @ARGV;

$OUT_FILE_BASE = "$OUT_FILE_DIR\\$FILE_NAME";






{
my $std_options = '-nochecksum -farcallpatch -integrity -Qn -e __start -strict_overlap_check -propagate_weak_constants';
my $command = "$GBSEXT_GHSBF_PATH\\elxr @COMMAND_ARGS $std_options $GBS_INCS $GBS_GLKB -o $OUT_FILE_SPEC";
print "$command\n";
$RC = system( $command) >> 8;

}


if ($GBS_DEBUGGER eq 'YES')
{
if ($RC == 0)
{
my $command = "$GBSEXT_GHSBF_PATH\\gnm.exe -s $OUT_FILE_SPEC > $OUT_FILE_BASE.nm";
print "$command\n";
$RC = system( $command) >> 8;


if ($RC == 0)
{
my @command = ("$GBSEXT_GHSBF_PATH\\dblink.exe -a -nm=auto $OUT_FILE_BASE.elf -o $OUT_FILE_BASE.dnm");
push @command, get_dbas();
my $command = "@command";
print "$command\n";
$RC = system( $command) >> 8;

} else
{
print "$PREFIX: Prematurely terminated because of RC=$RC)\n";
}
} else
{
print "$PREFIX: Prematurely terminated because of RC=$RC)\n";
}
}




print "$PREFIX: Failed ($RC)\n" if ($RC != 0);
exit $RC;




sub get_dbas()
{
my @dbas;

my $gbs_incs = $GBS_INCS;
$gbs_incs =~ s/-L //g;
my @gbs_incs = split( ' ', $gbs_incs);


my @gbs_glk_lines = split( ' ', $GBS_GLKB);


foreach my $item (@gbs_glk_lines)
{
if (substr( $item, 0, 2) eq '-l')
{
my $name = 'lib' . substr( $item, 2) . '.dba';
my $found = 0;
foreach my $dir (@gbs_incs)
{
if (-f "$dir/$name")
{


push @dbas, "$dir\\$name";
$found = 1;
last;
}
}
print "$PREFIX: $name not found\n" if (!$found);
} elsif (substr( $item, -2, 2) eq '.o')
{
my $spec = substr( $item, 0, -2) . '.dbo';
if ( -f $spec)
{


push @dbas, $spec;

} else
{
print "$PREFIX: $spec not found\n";
}
} else
{

}
}
return @dbas;
}


