#===============================================================
#
#   copy_export.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#
#   Call:
#	arg_1	= From-subsys_comma_list
#	arg_2	= From-export: Relative path from EXPORT-dir (may be .)
#	arg_3	= To-dir:   Absolute path do output-directory
#============================================================================
use strict;
use warnings;
use integer;
print "*COPY_EXPORT @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use glo::file;
use glo::flock;
use gbsenv;





sub run_copy_export();
sub get_command_arguments();




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();




my @SUBSYS_REFS;


my @GBS_SKIPTYPES = ENV_getenv( 'GBS_SKIPTYPES');











$| = 1;
my $RC = 0;
GBSENV_init();

get_command_arguments();

ENV_sig( EE => "No GBS_ROOT_PATH defined")
if (!$GBS_ROOT_PATH);




run_copy_export();




ENV_exit( $RC);




END
{
ENV_print_end_msg(1);
}




sub run_copy_export()
{
foreach my $ref (@SUBSYS_REFS)
{
my ($subsys, $from_dir, $to_dir) = @{$ref};
ENV_say( 1, "SubSys: $subsys");
ENV_say( 1, "From  :   $from_dir");
ENV_say( 1, "To    :   $to_dir");

if (!-e $to_dir)
{
ENV_say( 1, "Creating '$to_dir'");
FILE_mkpath( $to_dir);
} else
{
ENV_say( 1, "Existing '$to_dir'");
}
my $from_fh = FLOCK_sh_wait( "$from_dir/.export.lck");
my $to_fh = FLOCK_ex_wait( "$to_dir/.export.lck");
my ($dirs, $files) = FILE_replicate_tree( $from_dir, $to_dir, \@GBS_SKIPTYPES, 0);
FLOCK_unlock( $to_fh);
FLOCK_unlock( $from_fh);

ENV_say( 1, "$dirs dir(s) and $files file(s) replicated");
}
}




sub get_command_arguments()
{

my ($subsystems,
$export_path,
$to_dir,
) = ARGS_get( 3, 'perl copy_export.pl subsys_comma_list gbs_export-path to-path', undef);








my @subsystems = split( ',', $subsystems);




ARGS_error_exit( "No .. allowed in export_path ($export_path)")
if ($export_path =~ /\.\./);

foreach my $subsys (@subsystems)
{
ARGS_error_exit( "No such SubSystem '$subsys'")
if (!-d "$GBS_ROOT_PATH/dev/$subsys");

ENV_setenv( GBS_SUBSYS => $subsys);

my $from_dir = "$GBS_ROOT_PATH/dev/$subsys/export";
$from_dir .= "/$export_path"
if ($export_path ne '.');
$from_dir = ENV_perl_canon_paths( ENV_expand_envs( $from_dir));
ARGS_error_exit( "Export path '$export_path' does not exist:", $from_dir)
if (!-d $from_dir);

my $to_dir = ENV_perl_canon_paths( ENV_expand_envs( $to_dir));
ARGS_error_exit( "To_dir: Must be absolute path:", $to_dir)
if (!ENV_is_abs_path( $to_dir));
ARGS_error_exit( "Cannot copy to inside this ROOT:",
$GBS_ROOT_PATH,
$to_dir)
if ($to_dir =~ /^$GBS_ROOT_PATH/i);
push @SUBSYS_REFS, [ $subsys, $from_dir, $to_dir ];
}
}



