#=================================================
#
#  scmsmaint.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package scmsmaint;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SCMSMAINT_main
);
}




use glo::env;
use glo::ask;
use glo::slurp;
use glo::scm;
use gbsenv;
use gbsscm;
use system;
use files;




sub SCMSMAINT_main($$$);

sub scm_modify($$$);

sub scm_fix_ignore($$$);
sub fix_ignore_level($);
sub fix_ignore_level_recurse($);
sub fix_ignore_wild();
sub fix_ignore($);

sub scm_disable_enable($$$);
sub scm_change_this($$$);
sub scm_change_other($$$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');

my $SAVED_SCMS;
my $OTHER_OSNAME;




sub SCMSMAINT_main($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

my $scms_state = (defined $SAVED_SCMS) ? 'Disabled' : 'Enabled';
$OTHER_OSNAME = (SYSTEM_get_osnames( $GBS_ROOT_PATH, 'scms'))[1];	    # first (0) is current OS
$OTHER_OSNAME = ''
if (!defined $OTHER_OSNAME);
my @scm_menu_items = (
[ "Modify (fix) SCMS settings (Repository, Data)",
\&scm_modify ],
[ "Fix SCMS 'ignore' settings. (svn and git only)",
\&scm_fix_ignore ],
[ "Temporarily disable/enable SCMS   ($scms_state)",
\&scm_disable_enable ],
[ "Not Used",
sub { ENV_say( 1, "Not used!") } ],
[ "Not Used",
sub { ENV_say( 1, "Not used!") } ],
[ "Not Used",
sub { ENV_say( 1, "Not used!") } ],
[ "Not Used",
sub { ENV_say( 1, "Not used!") } ],
[ "Change SCMS",
\&scm_change_this ],
[ "Change SCMS for Other Platform ($OTHER_OSNAME)",
\&scm_change_other ],
);

ASK_menu( 'Select function to perform', \@scm_menu_items, $entries_ref);
}




sub scm_modify($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;




my ($gbs_scms, $gbs_scms_repository, $gbs_scms_data) = GBSSCM_preset( 0);

($gbs_scms, $gbs_scms_repository, $gbs_scms_data) =
SCM_select( undef, $GBS_ROOT_PATH, $gbs_scms_repository, $gbs_scms_data);

ENV_say( 1, "Selected: $gbs_scms: $gbs_scms_repository ($gbs_scms_data)");
if (ASK_YNQ( 'OK?', 'Y', 1) eq 'Y')
{



SCM_preset( $GBS_ROOT_PATH, [$gbs_scms, $gbs_scms_repository, $gbs_scms_data ]);
GBSSCM_setenv( [ $gbs_scms, $gbs_scms_repository, $gbs_scms_data ]);
GBSSCM_connect();

SYSTEM_put_os( $GBS_ROOT_PATH, '.', scms	    => $gbs_scms);
SYSTEM_put_os( $GBS_ROOT_PATH, '.', scms_repository => $gbs_scms_repository);
SYSTEM_put(    $GBS_ROOT_PATH,      scms_data	    => $gbs_scms_data);
SYSTEM_write();	   # Includes backup and SCM_checkout
}
}




sub scm_fix_ignore($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

GBSSCM_preset( 0);
my $current_id = SCM_get_current_id();
if ($current_id eq 'svn' || $current_id eq 'git')
{
ENV_say( 1, "GBS_ROOT_PATH: $GBS_ROOT_PATH");
my @item_refs;
push @item_refs, [ 'Whole System. (may take a while!)',
sub { fix_ignore_level( $GBS_ROOT_PATH) } ];
if ($GBS_SUBSYS)
{
push @item_refs, [ "Current SubSys ($GBS_SUBSYS)",
sub { fix_ignore_level( "$GBS_ROOT_PATH/dev/$GBS_SUBSYS") } ];
} else
{
push @item_refs, [ "Current SubSys - Not Available",
sub { ENV_say( 1, "Not Availabe!") } ],
}
if ($GBS_COMPONENT)
{
push @item_refs, [ "Current Component ($GBS_COMPONENT)",
sub { fix_ignore_level( "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp/$GBS_COMPONENT") } ]
} else
{
push @item_refs, [ "Current Component - Not Available",
sub { ENV_say( 1, "Not Availabe!") } ],
}
push @item_refs, [ 'Wildcard',
\&fix_ignore_wild ];

ASK_menu( 'Select function', \@item_refs, $entries_ref);
} else
{
ENV_sig( W => 'Not SubVersioN or Git');
}
}




sub fix_ignore_level($)
{
my ($spec) = @_;	# SYSTEM, SUBSYS, COMPONENT

GBSSCM_preset( 0);

ENV_say( 1, "Caching states...");
SCM_cache_states( $spec);
fix_ignore_level_recurse( $spec);
}




sub fix_ignore_level_recurse($)
{
my ($spec) = @_;

fix_ignore( $spec);
my @dirs = SLURP_dir_dirs( $spec, 0);
foreach my $dir (@dirs)
{
fix_ignore_level_recurse( "$spec/$dir");    #recurse
}
}




sub fix_ignore_wild()
{
ENV_say( 1, "WildCards relative to GBS_ROOT_PATH");
my $wild_spec;
do
{
$wild_spec = ASK_text( 'Enter Wildcard', '', 0, undef);
if ($wild_spec)
{
my $abs_wild_spec = "$GBS_ROOT_PATH/$wild_spec";
my @specs = glob( $abs_wild_spec);
if (@specs)
{
foreach my $spec (@specs)
{
if (-e $spec)
{
fix_ignore( $spec)
if (-d _);
} else
{
ENV_say( 1, "No such file/dir '$spec'");
}
}
} else
{
ENV_say( 1, "No files/directories found!");
}
}
} while ($wild_spec);
}




sub fix_ignore($)
{
my ($spec) = @_;


my ($ignores_ref, $scm) = FILES_attrib( $spec);
my @ignores = @{$ignores_ref};

if (@ignores && $scm != 0)			# scm: 0 = no, 1 = yes, -1 = don't know
{
if (SCM_get_states( $spec) >= 0)	# state: 0 = checked in, 1 = checked_out, -1 = Not in scm
{
ENV_say( 1, "  $spec (@ignores)");
SCM_set_ignore( $spec, $ignores_ref, 0);
}
}
}




sub scm_disable_enable($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

if (defined $SAVED_SCMS)
{



ENV_say( 1, "SCMS is currently disabled");
if ( ASK_YN( "Enable SCM '$SAVED_SCMS'?", 'Y') eq 'Y')
{
GBSSCM_resume();
$SAVED_SCMS = undef;
}
} else
{



my $current_scms = ENV_getenv( 'GBS_SCMS');
if ($current_scms eq 'No_SCMS')
{
ENV_sig( W => "No current SCMS");
} else
{
ENV_say( 1, "SCMS ('$current_scms') is currently enabled");
if ( ASK_YN( "Disable SCMS?", 'Y') eq 'Y')
{
GBSSCM_suspend();
$SAVED_SCMS = $current_scms;
my $gbs_root_path = GBSENV_gbs_root_path();
SCM_preset( $gbs_root_path, [ 'no_scms', undef, undef ]);
}
}
}
}




sub scm_change_this($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;




my ($gbs_scms, $gbs_scms_repository, $gbs_scms_data) = GBSSCM_preset( 0);

($gbs_scms, $gbs_scms_repository, $gbs_scms_data) =
SCM_select( $gbs_scms, $GBS_ROOT_PATH, $gbs_scms_repository, $gbs_scms_data);

ENV_say( 1, "Selected: $gbs_scms: $gbs_scms_repository ($gbs_scms_data)");
if (ASK_YNQ( 'OK?', 'Y', 1) eq 'Y')
{



SCM_preset( $GBS_ROOT_PATH, [$gbs_scms, $gbs_scms_repository, $gbs_scms_data ]);
GBSSCM_setenv( [ $gbs_scms, $gbs_scms_repository, $gbs_scms_data ]);
GBSSCM_connect();

SYSTEM_put_os( $GBS_ROOT_PATH, '.', scms	    => $gbs_scms);
SYSTEM_put_os( $GBS_ROOT_PATH, '.', scms_repository => $gbs_scms_repository);
SYSTEM_put(    $GBS_ROOT_PATH,      scms_data	    => $gbs_scms_data);
SYSTEM_write();	   # Includes backup and SCM_checkout
}
}




sub scm_change_other($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

if ($OTHER_OSNAME ne '')
{
my $cur_other = SYSTEM_get_os( $GBS_ROOT_PATH, $OTHER_OSNAME, 'scms');
ENV_say( 1, "The SCMS for the other platform ($OTHER_OSNAME) is '$cur_other'");
if (ASK_YN( 'Change?', 'N') eq 'Y')
{
my @scm_names = SCM_get_names( '!');    # Other OS
my $new_other = ASK_value_from_menu( "Select SCMS for Platform $OTHER_OSNAME", \$cur_other,
undef, [ @scm_names ]);
ENV_say( 1, "Selected: $new_other");
if ($new_other ne $cur_other)
{
if (ASK_YN( 'Ok?', 'N') eq 'Y')
{
GBSSCM_preset( 0);
SYSTEM_put_os( $GBS_ROOT_PATH, $OTHER_OSNAME, scms => $new_other);
SYSTEM_write();    # Includes backup and SCM_checkout
ENV_say( 1, 'Changed');
}
} else
{
ENV_say( 1, 'Unchanged');
}
}
} else
{
ENV_sig( W => "There is no other Platform instance for this System");
}
}

1;
