#===============================================
#
#   scope.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package scope;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SCOPE_get_components
SCOPE_get_includes
);
}




use glo::env;
use gbsenv;
use gbsfile;




sub SCOPE_get_components($$);
sub SCOPE_get_includes($$@);

sub read_scope($$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my @GBS_ALL_TARGETS = ENV_getenv('GBS_ALL_TARGETS');
my %GBS_ALL_TARGETS = map { $_ => 1 } ENV_getenv('GBS_ALL_TARGETS');

my %TARGET_TOKENS = (
'='     => '=',
'=='    => '=',
'!'     => '!',
'!='    => '!',
);

my %SCOPE_CACHE;





sub SCOPE_get_components($$)
{
my ($subsys,
$component,
) = @_;

return read_scope( $subsys, $component);
}




sub SCOPE_get_includes($$@)
{
my ($subsys,
$component,
@sub_dirs
) = @_;
my @scope_dirs;

foreach my $component (read_scope( $subsys, $component))
{
foreach my $sub_dir (@sub_dirs)
{
push @scope_dirs, "../../$component$sub_dir";
}
}

return @scope_dirs;
}




sub read_scope($$)
{
my ($subsys,
$component,
) = @_;
my @components;

my $subsys_comp = "$subsys:$component";

if (exists( $SCOPE_CACHE{$subsys_comp}))
{
@components = @{$SCOPE_CACHE{$subsys_comp}};
} else
{
my $gbs_comp_path = "$GBS_ROOT_PATH/dev/$subsys/comp";
my $scope_file = "$gbs_comp_path/$component/scope";
if (GBSFILE_open( $scope_file, 0, undef, ENV_getenv( 'GBS_TARGET')))
{
my $line = GBSFILE_get_line();
while ($line)
{

if (-d "$gbs_comp_path/$line")
{
push @components, $line;
}  else
{
GBSFILE_sig( E => "No such Component '$line'");
}
$line = GBSFILE_get_line();
}
}
$SCOPE_CACHE{$subsys_comp} = [ @components ];
}

return @components;
}

1;

