@ECHO OFF
::========================================
::  define.bat <rel> <command> [<command_arg> ...]
::
::  Called by a GBS shortcut
::  Called by gbsexit
::
::  - Call %GBS_BOOT_PATH%\boot.bat
::  - Call %GBS_BASE_PATH%\gbscmd - if present
::  - Call %GBS_BASE_PATH%\setup.bat
::  - Define the main gbs commands or Execute the command
::
::  - Args:
::    <rel>:     'cur', 'beta' or actual version
::    <command>: 'gbs', 'perl', 'call' or 'exec'
::
::  GBS Startup:    gbs %*
::  GBS Setup:	    exec gbssetup.bat %*
::  GBS Uninstall:  perl gbsuninstall.pl %*
::  GBS Help:	    ** Direct, does not use define.bat
::========================================

echo DEFINE_: %*

::
::  Handle Args
::
set GBS_DEFINE_REL=%1
shift
set GBS_DEFINE_COMMAND=%1
shift
set GBS_DEFINE_COMMAND_ARGS=%1
shift
:args_next
    if [%1]==[] goto args_end
    set GBS_DEFINE_COMMAND_ARGS=%GBS_DEFINE_COMMAND_ARGS% %1
    shift
  goto args_next
:args_end
if [%GBS_DEFINE_COMMAND%]==[] set GBS_DEFINE_COMMAND=gbs

::
::  Set GBS_BOOT_PATH, GBS_BASE_PATH and GBS_PERL_CMD
::
call "%APPDATA%\.gbs\boot.bat"

::
::  Execute gbscmd.bat if it exist
::
set GBS_LOGIN_FILE="%GBS_BASE_PATH%\gbscmd.bat"
if exist %GBS_LOGIN_FILE% echo DEFINE_: Executing %GBS_LOGIN_FILE%...
if exist %GBS_LOGIN_FILE% call %GBS_LOGIN_FILE%
set GBS_LOGIN_FILE=

::
::  Execute profile
::	Defines GBS_SCRIPTS_ROOT, GBS_SCRIPTS_REL, GBS_SCRIPTS_PATH and more
::
call "%GBS_BASE_PATH%\profile.bat"

::
::  Adjust GBS_SCRIPTS_REL
::
echo DEFINE_: Current GBS version is '%GBS_SCRIPTS_REL%'
set GBS_WANTED_REL=%GBS_DEFINE_REL%
if [%GBS_DEFINE_REL%]==[cur] set GBS_WANTED_REL=%GBS_SCRIPTS_REL%
if [%GBS_DEFINE_REL%]==[beta] set GBS_WANTED_REL=beta

if [%GBS_WANTED_REL%]==[] goto execute_else
if not exist "%GBS_SCRIPTS_ROOT%\%GBS_WANTED_REL%" goto execute_else
    ::
    ::	Set GBS_SCRIPTS_REL and GBS_SCRIPTS_PATH
    ::
    if not [%GBS_SCRIPTS_REL%]==[%GBS_WANTED_REL%] echo DEFINE_: GBS version set to '%GBS_WANTED_REL%'
    set GBS_SCRIPTS_REL=%GBS_WANTED_REL%
    set GBS_SCRIPTS_PATH=%GBS_SCRIPTS_ROOT%\%GBS_SCRIPTS_REL%

    ::
    ::  Select the command
    ::
    if not [%GBS_DEFINE_COMMAND%]==[gbs] goto select_nogbs
        ::
        ::  Define 'gbs' command
        ::
        doskey gbs=%GBS_SCRIPTS_PATH%\gbsinit.bat $*
        doskey gbshelp=%GBS_PERL_CMD% %GBS_SCRIPTS_PATH%\gbshelp.pl $*
        doskey gbssetup=call %GBS_SCRIPTS_PATH%\gbssetup.bat $*
        doskey gbsuninstall=%GBS_PERL_CMD% %GBS_SCRIPTS_PATH%\gbsuninstall.pl $*

        echo DEFINE_: GBS commands defined:
        echo          gbs
        echo          gbshelp
        echo          gbssetup
        echo          gbsuninstall
        echo DEFINE_: Type 'gbs' to startup GBS [%GBS_DEFINE_REL%]
        echo -
        goto select_end
    :select_nogbs
        ::
        ::  Execute the specified command
        ::
        echo DEFINE_: Executing %GBS_DEFINE_COMMAND% %GBS_DEFINE_COMMAND_ARGS%...
        set GBS_COMMAND_OK=0
        if [%GBS_DEFINE_COMMAND%]==[perl] set GBS_COMMAND_OK=1 && %GBS_PERL_CMD% %GBS_SCRIPTS_PATH%\%GBS_DEFINE_COMMAND_ARGS%
        if [%GBS_DEFINE_COMMAND%]==[call] set GBS_COMMAND_OK=1 && call %GBS_SCRIPTS_PATH%\%GBS_DEFINE_COMMAND_ARGS%
        if [%GBS_DEFINE_COMMAND%]==[exec] set GBS_COMMAND_OK=1 && %GBS_SCRIPTS_PATH%\%GBS_DEFINE_COMMAND_ARGS%
        if [%GBS_COMMAND_OK%]==[0] echo DEFINE_: **ERROR** Invalid command '%GBS_DEFINE_COMMAND%' (perl call exec)
        if [%GBS_COMMAND_OK%]==[0] echo          define %GBS_DEFINE_REL% %GBS_DEFINE_COMMAND% %GBS_DEFINE_COMMAND_ARGS%
        set GBS_COMMAND_OK=
    :select_end
    goto execute_end
:execute_else
    echo DEFINE_: GBS Version %GBS_DEFINE_REL%[%GBS_SCRIPTS_REL%]' does not exist - Cannot continue with GBS
:execute_end

::
::  Cleanup
::
set GBS_DEFINE_REL=
set GBS_DEFINE_COMMAND=
set GBS_DEFINE_COMMAND_ARGS=
set GBS_WANTED_REL=

:end

:: **EOF**
