#==============================================================
#
#   ssprop.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package ssprop;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SSPROP_reset
SSPROP_is_full_gbs
SSPROP_get_all_types
SSPROP_get_type
SSPROP_set_type
);
}




use glo::env;
use glo::slurp;
use glo::spit;
use gbsenv;




sub SSPROP_reset();
sub SSPROP_is_full_gbs($$);
sub SSPROP_get_all_types();
sub SSPROP_get_type($$);
sub SSPROP_set_type($$$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $IS_WIN32 = ENV_is_win32();




my %CACHE;



my @ALL_TYPE_REFS = (

[ 'GBS',			    'GBS' ,  1, 1 ],
[ 'Microsoft Visual Studio',    'MSVS',  1, 0 ],
[ 'make',			    'make',  1, 1 ],
[ 'Other',			    'Other', 1, 1 ],
);
my @ALL_VALID_TYPE_REFS;


if ($IS_WIN32)
{
@ALL_VALID_TYPE_REFS = grep( $_->[2] == 1, @ALL_TYPE_REFS);	# $for_win32
} else
{
@ALL_VALID_TYPE_REFS = grep( $_->[3] == 1, @ALL_TYPE_REFS);	# $for lunix
}




sub SSPROP_reset()
{
%CACHE = ();
}




sub SSPROP_is_full_gbs($$)
{
my ($root_path,	# If undef, $GBS_ROOT_PATH is assumed
$subsys,
) = @_;


return (SSPROP_get_type( $root_path, $subsys) eq 'GBS') ? 1 : 0;
}




sub SSPROP_get_all_types()
{
return @ALL_VALID_TYPE_REFS;
}




sub SSPROP_get_type($$)
{
my ($root_path,	# If undef, $GBS_ROOT_PATH is assumed
$subsys,
) = @_;
my $type;		# GBS, MSVS, make, Other

my $key;
my $subsys_path;
if (defined $root_path)
{
$key = "$root_path/$subsys";
$subsys_path = "$root_path/dev/$subsys";
} else
{
$key = $subsys;
$subsys_path = "$GBS_ROOT_PATH/dev/$subsys";
}

$type = $CACHE{$key};
if (!defined $type)
{
my $prop_filespec = "$subsys_path/ssprop.gbs";
if (-e $prop_filespec)
{
($type) = SLURP_file( $prop_filespec);
} else
{
$type = 'UNK';
}
}

return $type;
}





sub SSPROP_set_type($$$)
{
my ($root_path,	# If undef, $GBS_ROOT_PATH is assumed
$subsys,
$type,		# from @ALL_VALID_TYPE_REFS
) = @_;
my $ssproc_filespec;

my @valid_types = map { $_->[1] } @ALL_VALID_TYPE_REFS; # $type
ENV_sig( F => "Invalid type '$type' (@valid_types)")
if (!grep( $type eq $_, @valid_types));

my $key;
my $subsys_path;
if (defined $root_path)
{
$key = "$root_path/$subsys";
$subsys_path = "$root_path/dev/$subsys";
} else
{
$key = $subsys;
$subsys_path = "$GBS_ROOT_PATH/dev/$subsys";
}
$ssproc_filespec = "$subsys_path/ssprop.gbs";
SPIT_file( $ssproc_filespec, $type);
$CACHE{$key} = $type;

return $ssproc_filespec;
}

1;


