#==============================================================
#
#   swa.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package swa;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SWA_validate_dirs
SWA_set
);
}




use glo::env;
use gbsenv;
use gbsglo;
use gbsrc;
use plugin;




sub SWA_validate_dirs($$$$$);
sub SWA_set($$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();




sub SWA_set($$)
{
my ($audit,
$must_write,
) = @_;
my $audit_plugin;

ENV_debug( 1, "Set $GBS_ROOT_PATH ($audit)");




$audit_plugin = ($audit eq '') ? '' : PLUGIN_get_att_plugin_name( $audit);
ENV_setenv( GBS_AUDIT => $audit);
ENV_setenv( GBS_AUDIT_PLUGIN => $audit_plugin);




GBSRC_write_root( audit => $audit)
if ($must_write && GBSENV_mode_is_interactive());

return $audit_plugin;
}




sub SWA_validate_dirs($$$$$)
{
my ($root_path,
$subsys,
$component,
$target,
$audit,
) = @_;
my $new_audit = $audit;


if ($subsys ne '')
{



my $audit_path = "$root_path/dev/$subsys/audit";
if (-d "$audit_path/$audit")
{
if (-d "$audit_path/$audit/$target")
{



if ( $component ne '' && GBSGLO_subsystem_is_full_gbs( $subsys, $root_path))
{
my $aud_path = "$root_path/dev/$subsys/comp/$component/aud";
if (-d "$aud_path/$audit")
{
if (!-d "$aud_path/$audit/$target")
{
ENV_sig( W => "Component '$component' does not audit for Audit '$audit' and Target ($target)");
$new_audit = '';
}
} else
{
ENV_sig( W => "Component '$component' does not audit for Audit '$audit'");
$new_audit = '';
}
}
} else
{
ENV_sig( W => "SubSystem '$subsys' does not audit for Audit '$audit' and Target ($target)");
$new_audit = '';
}
} else
{
ENV_sig( W => "SubSystem '$subsys' does not audit for Audit '$audit'");
$new_audit = '';
}
}

return $new_audit;
}

1;



