#==============================================================
#
#   swc.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package swc;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SWC_set
);
}




use glo::env;
use gbsenv;
use gbsrc;
use gbsglo;




sub SWC_set($$$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();




sub SWC_set($$$)
{
my ($subsys,
$component,
$must_write,
) = @_;

ENV_debug( 1, "Set $GBS_ROOT_PATH, $subsys: $component");




my $component_path;
my @env_refs = (



[ COMPONENT	    => 1, 'f', sub { return $component } ],
[ COMPONENT_PATH    => 1, 'f', sub { return $component_path } ],
[ AUD_PATH	    => 0, 'p', undef ],
[ BLD_PATH	    => 0, 'p', undef ],
[ DAT_PATH	    => 0, 'p', undef ],
[ INC_PATH	    => 0, 'p', undef ],
[ LOC_PATH	    => 0, 'p', undef ],
[ OPT_PATH	    => 0, 'p', undef ],
[ SAV_PATH	    => 0, 'p', undef ],
[ SRC_PATH	    => 1, 'p', undef ],
);

if ($component eq '')
{



map { ENV_setenv( "GBS_$_->[0]", '') } @env_refs;
} else
{



ENV_sig( F => "Cannot specify Component ($component) for empty-named SubSys")
if ($subsys eq '');
ENV_sig( F => "Cannot specify Component ($component) for Non-GBS SubSys ($subsys)")
if (!GBSGLO_subsystem_is_full_gbs( $subsys));




$component_path = "$GBS_ROOT_PATH/dev/$subsys/comp/$component";
my $do_allways = ENV_is_development();
foreach my $ref (@env_refs)
{
my ($name, $allways, $type, $arg) = @{$ref};

my $env_name = "GBS_$name";
my $value;
if ($allways || $do_allways)
{
if ($type eq 'p')
{
my $path_part = lc( substr( $name, 0, -5));	    # minus "_PATH"
$value = "$component_path/$path_part";
} elsif ($type eq 'c')
{
$value = '';
} elsif ($type eq 'f')
{
$value = $arg->();
} else
{
ENV_sig( F => "Impossible type ($type)");
}
} else
{
$value = '';
}

GBSENV_setenv( $env_name => $value );
}
}




GBSRC_write_subsys( $subsys, component => $component)
if ($must_write && $subsys ne '' &&
GBSENV_mode_is_interactive());
}

1;


