#==============================================================
#
#   swglo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package swglo;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SWGLO_validate_subsys
SWGLO_validate_component
SWGLO_validate_target
SWGLO_validate_audit

SWGLO_new_cwd
SWGLO_set_title
);
}




use glo::env;
use gbsenv;
use gbsrc;
use gbsglo;
use gbscmd;
use gbscheck;




sub SWGLO_validate_subsys($);
sub SWGLO_validate_component($$);
sub SWGLO_validate_target($$$);
sub SWGLO_validate_audit($$$$);
sub SWGLO_new_cwd($$$);
sub SWGLO_set_title(@);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();




sub SWGLO_validate_subsys($)
{
my ($subsys,	    # '', '-', number or value
) = @_;
my $new_subsys = '';

if ($subsys eq '-')
{
$new_subsys = '';
} else # $subsys eq '', '.' or <name>
{
my $default_subsys = ENV_getenv( 'GBS_SUBSYS');
$default_subsys = GBSRC_get_root( $GBS_ROOT_PATH, 'subsys')
if ($default_subsys eq '');
$new_subsys = GBSCHECK_subsys( $GBS_ROOT_PATH, $subsys, $default_subsys);
}

return $new_subsys;
}




sub SWGLO_validate_component($$)
{
my ($subsys,
$component,	    # '', '-', number or value
) = @_;
my $new_component = '';

if ($component eq '-')
{
$new_component = '';
} else # $component eq '', '.' or <name>
{
if (GBSGLO_subsystem_is_full_gbs( $subsys))
{
my $default_component = GBSRC_get_subsys( $GBS_ROOT_PATH, $subsys, 'component');
$new_component = GBSCHECK_component( $GBS_ROOT_PATH, $subsys, $component, $default_component);
} else
{
if ($component eq '')
{
$new_component = '';
} else
{
ENV_sig( EE => "Cannot specify a component ($component) for Non GBS SubSystem");
}
}
}

return $new_component;
}




sub SWGLO_validate_target($$$)
{
my ($subsys,
$component,
$target,	    # '', '-', number or value
) = @_;
my $new_target = '';

if ($target eq '-')
{
$new_target = '';
} else # $target eq '.' or <name>
{
my $default_target = ENV_getenv( 'GBS_TARGET');
$default_target = GBSRC_get_root( $GBS_ROOT_PATH, 'target')
if ($default_target eq '');
my @targets = ENV_getenv( 'GBS_TARGETS');
$new_target = GBSCHECK_target( $GBS_ROOT_PATH, $subsys, $component, $target, $default_target, \@targets);
}

return $new_target;
}




sub SWGLO_validate_audit($$$$)
{
my ($subsys,
$component,
$target,
$audit,	    # '', '-', number or value
) = @_;
my $new_audit = '';

if ($audit eq '-')
{
$new_audit = '';
} else # $audit eq '.' or <name>
{
my $default_audit = ENV_getenv( 'GBS_AUDIT');
$default_audit = GBSRC_get_root( $GBS_ROOT_PATH, 'audit')
if ($default_audit eq '');
my @audits = ENV_getenv( 'GBS_AUDITS');
if (@audits)
{
$new_audit = GBSCHECK_audit( $GBS_ROOT_PATH, $subsys, $component, $target, $audit, $default_audit, \@audits);
}
}

return $new_audit;
}




sub SWGLO_new_cwd($$$)
{
my ($root_path,
$subsys,
$component,
) = @_;
my $new_cwd;

if ($component ne '')
{
$new_cwd = "$root_path/dev/$subsys/comp/$component/src";
} elsif ($subsys ne '')
{
$new_cwd = "$root_path/dev/$subsys";
} elsif ($root_path ne '')
{
$new_cwd = $root_path;
} else  # $root_path eq ''
{





my $cwd = ENV_cwd();
if ($GBS_ROOT_PATH ne '' &&
substr( $cwd, 0, length($GBS_ROOT_PATH)) . '/' eq $GBS_ROOT_PATH . '/')
{
$new_cwd = ENV_parent_path( $GBS_ROOT_PATH);
ENV_say( 1, "Setting CWD to $new_cwd...");
} else
{
$new_cwd = $cwd;
}
}

return $new_cwd;
}





sub SWGLO_set_title(@)
{
my @text = @_;  # $root_path, $subsys, $component
my @lines;

if (!GBSENV_mode_is_background())	    # INTERACTIVE or FOREGROUND
{
if (@text == 1 && $text[0] eq '-')
{
push @lines, GBSCMD_set_xtitle( '');
} else
{
my @text = reverse @text;
my $text = (@text) ? join ( ' - ', @text) : '-';
push @lines, GBSCMD_set_xtitle( "GBS: $text");
}
}

return @lines;
}

1;


