#==============================================================
#
#   swr.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package swr;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SWR_set
SWR_entry
SWR_exit
SWR_cleanup
);
}




use glo::env;
use glo::shell;
use glo::scm;
use gbsenv;
use gbsrc;
use system;
use gbsres;
use sys;
use steps;
use plugin;




sub SWR_set($$);
sub SWR_entry($);
sub SWR_exit($);
sub SWR_cleanup();

sub find_switch_files($$);










sub SWR_set($$)
{
my ($root_path,	    # '' == clear all
$must_write,
) = @_;

ENV_debug( 1, "Set $root_path");




my $plugin_root;
my $root_parent_dir;
my $log_path;
my @os_names;
my @scm_skiptypes;
my @all_skiptypes;
my @env_refs = (



[ PLUGIN_ROOT	    => 0, 'f', sub { return $plugin_root } ],
[ ROOT_PATH	    => 0, 'f', sub { return $root_path } ],
[ ROOT_PARENT       => 0, 'f', sub { return $root_parent_dir} ],
[ LOG_PATH	    => 0, 'f', sub { return $log_path } ],
[ SUBSYS	    => 0, 'c', undef ],
[ COMPONENT	    => 0, 'c', undef ],
[ TARGET	    => 0, 'c', undef ],
[ AUDIT		    => 0, 'c', undef ],
[ TOOL		    => 0, 'c', undef ],


[ EXT_PATH	    => 0, 'p', undef ],
[ DEV_PATH	    => 0, 'p', undef ],
[ RES_PATH	    => 0, 'p', undef ],
[ TMP_PATH	    => 0, 'p', undef ],
[ SILO_PATH	    => 0, 'p', undef ],
[ SYS_PATH	    => 0, 'p', undef ],
[ SYSAUDIT_PATH	    => 0, 'p', undef ],
[ SYSBUILD_PATH	    => 0, 'p', undef ],
[ SYSTOOLS_PATH	    => 0, 'p', undef ],


[ SYSTEM_NAME	    => 0, 'ss', undef ],
[ SCMS		    => 0, 'so', undef ],
[ SCMS_DATA	    => 0, 'ss', undef ],
[ SCMS_REPOSITORY   => 0, 'so', undef ],
[ ROOT_VERSION	    => 0, 'ss', undef ],
[ TARGETS	    => 1, 'sr', undef ],
[ AUDITS	    => 1, 'sr', undef ],
[ TOOLS		    => 1, 'sr', undef ],


[ ALL_SUBSYSTEMS    => 1, 'f',  sub { return STEPS_get_subsystems_order() } ],
[ ALL_TARGETS	    => 1, 'sa', undef ],
[ ALL_AUDITS	    => 1, 'sa', undef ],
[ ALL_TOOLS	    => 1, 'sa', undef ],
[ TARGET_PLUGIN	    => 0, 'c',  undef ],
[ AUDIT_PLUGIN	    => 0, 'c',  undef ],
[ TOOL_PLUGIN	    => 0, 'c',  undef ],

[ PLATFORMS	    => 1, 'f', sub { return SYSTEM_get_osnames( $root_path, 'targets') } ],
[ SKIPTYPES	    => 1, 'f', sub { return @all_skiptypes } ],
[ SCM_SKIPTYPES	    => 1, 'f', sub { return @scm_skiptypes } ],
[ IS_INTEGRATOR	    => 0, 'f', sub { GBSENV_is_integrator() } ],
[ IS_ADMINISTRATOR  => 0, 'f', sub { GBSENV_is_administrator() } ],
);

if ($root_path eq '')
{



map { ENV_setenv( "GBS_$_->[0]", '') } @env_refs;
ENV_set_tmpdir_path( undef);
} else
{



$plugin_root = ENV_parent_path( $root_path) . '/.gbs';
if (!-e $plugin_root)
{
ENV_mkdir( $plugin_root);
ENV_hide_file( $plugin_root, 1);	# hide
}
$plugin_root .= '/plugins';
mkdir $plugin_root;

$root_parent_dir = ENV_parent_dir( $root_path, -1);

$log_path = ENV_getenv_perl_path( 'GBS_LOG_ROOT') . "/$root_parent_dir";
mkdir $log_path;




@scm_skiptypes = @{SCM_preinfo( SYSTEM_get_os( $root_path, '.', 'scms'), $root_path)};
@all_skiptypes = ( GBSRES_skiptypes(), @scm_skiptypes, SYS_skiptypes( $root_path) );
foreach my $ref (@env_refs)
{
my ($name, $is_list, $type, $arg) = @{$ref};

my $env_name = "GBS_$name";
my $value;
my @values;
if ($type eq 'p')
{
my $path_part = lc( substr( $name, 0, -5));	    # minus "_PATH"
$value = "$root_path/$path_part";
} elsif ($type eq 'c')
{
$value = '';
} elsif ($type eq 'f')
{
if ($is_list)
{
@values = $arg->();
} else
{
$value = $arg->();
}
} elsif ($type eq 'ss')
{
my $system_part = lc $name;				    # 'audits', 'targets', 'tools'
$value = SYSTEM_get( $root_path, $system_part);
} elsif ($type eq 'so')
{
my $system_part = lc $name;
$value = SYSTEM_get_os( $root_path, '.', $system_part);	    # 'audits', 'targets', 'tools'
} elsif ($type eq 'sr')
{
my $system_part = lc $name;				    # 'audits', 'targets', 'tools'
@values = SYSTEM_get_os_ref_names( $root_path, '.', $system_part);
} elsif ($type eq 'sa')
{
my $system_part = lc( substr( $name, 4));	    # minus 'ALL_', 'audits', 'targets', 'tools'
@values = SYSTEM_get_os_ref_names( $root_path, '*', $system_part);
} else
{
ENV_sig( F => "Impossible type ($type)");
}

if ($is_list)
{

GBSENV_setenv( $env_name => [ @values ]);
} else
{

GBSENV_setenv( $env_name => $value );
}
}
ENV_set_tmpdir_path( "$root_path/tmp");




foreach my $plugin_type (qw( audit build tools ))
{
my %plugins_done;

foreach my $att_name (PLUGIN_get_att_names( $plugin_type))
{

my $plugin = PLUGIN_get_att_plugin_name( $att_name);
if (!$plugins_done{$plugin})
{

PLUGIN_check_envs( $plugin, 'W');
$plugins_done{$plugin} = 1;
}
}
}
}




GBSRC_write_base( root => $root_path)
if ($must_write && GBSENV_mode_is_interactive());

}




sub SWR_entry($)
{
my ($root_path,
) = @_;

my @lines;

my $system_name = SYSTEM_get( $root_path, 'system_name');
push @lines, SHELL_echo( "Starting $root_path($system_name)...");




foreach my $cust_env_name (GBSENV_get_custom_envs())
{
my $all_env_name = 'GBSALL_' . substr( $cust_env_name, 4);   # replace 'GBS_' by 'GBSALL_'
my $all_value = ENV_getenv( $all_env_name);

if ($all_value ne '')
{
push @lines, SHELL_echo( "$all_env_name => $cust_env_name => $all_value");
push @lines, SHELL_setenv( $cust_env_name => $all_value);
}
}



foreach my $env (keys %ENV)
{
if (substr( $env, 0, 10) eq 'GBSALLEXT_')
{
my $all_value = ENV_getenv( $env);
my $cust_env_name = 'GBSEXT_' . substr( $env, 10);
push @lines, SHELL_echo( "$env => $cust_env_name => $all_value");
push @lines, SHELL_setenv( $cust_env_name => $all_value);
}
}





foreach my $switch_spec (find_switch_files( $root_path, 'EE'))
{
if ($switch_spec =~ /switch\.usr\./)
{
my $short_switch_spec = substr( $switch_spec, length( $root_path) + 1);
push @lines, SHELL_echo( "*** Executing System $system_name $short_switch_spec entry ***");
}
push @lines, SHELL_source( $switch_spec, 'entry');
}

return @lines;
}




sub SWR_exit($)
{
my ($root_path,
) = @_;
my @lines;

my $gbs_system_name = ENV_getenv( 'GBS_SYSTEM_NAME');




foreach my $switch_spec (reverse find_switch_files( $root_path, 'W'))
{
if ($switch_spec =~ /switch\.usr\./)
{
my $short_switch_spec = substr( $switch_spec, length( $root_path) + 1);
push @lines, SHELL_echo( "*** Executing System $gbs_system_name $short_switch_spec exit ***");
}
push @lines, SHELL_source( $switch_spec, 'exit');
}




push @lines, map { SHELL_unsetenv( $_) } GBSENV_get_envs_to_reset();



return @lines;
}




sub SWR_cleanup()
{
my @lines;




push @lines, map { SHELL_unsetenv( $_) } GBSENV_get_envs_to_reset();



return @lines;
}




sub find_switch_files($$)
{
my ($root_path,
$sig_or_error,
) = @_;
my @files;

foreach my $switch_file ('switch.gbs', 'switch.usr')
{
my $switch_spec = ENV_command_filespec( "$root_path/$switch_file");
if (-f $switch_spec)
{
push @files, $switch_spec;
}
}

ENV_sig( $sig_or_error => "No switch.gbs(.bat/.sh) file found in '$root_path'")
if (@files == 0);

return @files;
}

1;


