#==============================================================
#
#   swt.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package swt;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SWT_validate_dirs
SWT_set
);
}




use glo::env;
use gbsenv;
use gbsrc;
use plugin;




sub SWT_validate_dirs($$$$);
sub SWT_set($$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();




sub SWT_validate_dirs($$$$)
{
my ($root_path,
$subsys,
$component,
$tool,
) = @_;
my $new_tool = $tool;


if ($subsys ne '')
{



my $gen_dir = "$root_path/dev/$subsys/tools";
if (!-d "$gen_dir/$tool")
{
ENV_sig( W => "SubSystem '$subsys' does not execute for Tool '$tool'");
$new_tool = '';
}
}

return $new_tool;
}




sub SWT_set($$)
{
my ($tool,
$must_write,
) = @_;

ENV_debug( 1, "Set $GBS_ROOT_PATH ($tool)");




my $tool_plugin = ($tool eq '') ? '' : PLUGIN_get_att_plugin_name( $tool);
ENV_setenv( GBS_TOOL => $tool);
ENV_setenv( GBS_TOOL_PLUGIN => $tool_plugin);




GBSRC_write_root( tool => $tool)
if ($must_write && GBSENV_mode_is_interactive());
}

1;


