#!/bin/bash
#
#   Install.sh
#       Must NOT be sourced
#	--q-
#      --site=<site>
#      --verbose-
#	--help
#

if [[ ${BASH_SOURCE[0]} != "" ]]
then
    # on bash this will work even if the script is sourced
    #\echo "BASH_SOURCE[0]=${BASH_SOURCE[0]}"
    \pushd `dirname ${BASH_SOURCE[0]}` > /dev/null
else
    #\echo "No BASH_SOURCE"
    \pushd `dirname $0` > /dev/null
fi

#
#   Sanity Check: Assert TMPDIR
#
if [[ ! -z TMPDIR ]]
then
    \export "TMPDIR=/tmp/$USER"
    \mkdir "$TMPDIR" 2>/dev/null
fi

#
#   Sanity Check: Assert Install Dir
#
if [[ ! -e ./Install.bat || ! -e ./Install.sh || ! -e ./Install.pl || ! -e ./Readme.txt ]]
then
    \echo "INSTALL_: Fatal Error: You are not in the proper Install directory"
else
    #
    #	set GBS_BOOT_PATH
    #
    \export GBS_BOOT_PATH=~/.gbs

    #
    #	Create the ~/.gbs directory if it does not exist
    #
    if [[ ! -e $GBS_BOOT_PATH ]]
    then
	mkdir $GBS_BOOT_PATH
    fi
	
    #
    #   Define GBS_SCRIPTS_PATH
    #
    for i in *; do
	if [ -d "$i" ]; then
	    SUBDIR="$i"
	fi
    done
    if [[ "$SUBDIR" != "" ]]
    then
	\echo "INSTALL_: GBS version: $SUBDIR"
	\export GBS_SCRIPTS_PATH="$PWD/$SUBDIR"
    else
	#echo "INSTALL_: PWD=$PWD"
	echo "INSTALL_: *** In Development Environment ***"
	\pushd .. > /dev/null
	\export GBS_SCRIPTS_PATH="$PWD"
	\popd > /dev/null
    fi
    echo "INSTALL_: GBS_SCRIPTS_PATH set to $GBS_SCRIPTS_PATH"

    #
    #  Set GBS_BOOT_PATH, GBS_BASE_PATH and GBS_PERL_CMD and XDG_DESKTOP_DIR
    #
    source  $GBS_SCRIPTS_PATH/setup/boot.sh

    #
    #   Run Install
    #       --q
    #       --site=<stite>
    #       --help
    #
    $GBS_PERL_CMD -w Install.pl $*

    popd > /dev/null
fi

#
#   Exit
#
read -p "INSTALL_: Hit <Enter> to finish (and maybe close window): "

###EOF###

